#include "SmageComicEllipseShape.h"
#include "SmageSettings.h"

#include <iostream>
#include <math.h>

#include <qtextstream.h>

using namespace std;

SmageThinkBubbleCircle::SmageThinkBubbleCircle()
{
  pos = QPoint();
  size = QSize(10,10);
}

SmageComicEllipseObj::SmageComicEllipseObj()
{
  ellipseObjData = SmageEllipseObj();
  bubblePeakPos = QPoint();
  openAngle = 0;
  closeAngle = 5760; // for a closed ellipse
  thinkBubbleCircles.clear();
}

//  Constructor
SmageComicEllipseShape::SmageComicEllipseShape() : SmageEllipseShape()
{
  _comicEllipseObjVec.clear();
}

// Destructor
SmageComicEllipseShape::~SmageComicEllipseShape()
{  
  if (!_comicEllipseObjVec.empty())
    {
      for (uint i=0; i<_comicEllipseObjVec.size(); i++)
	delete _comicEllipseObjVec[i];
    }
}

//   Generate a character string holding all text shape parameters
void SmageComicEllipseShape::generateComicEllipseShapeString()
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageComicEllipseShape::generateComicEllipseShapeString()\n");
#endif

  if (_comicEllipseObjVec.size() > 0)
    {
      vector<SmageComicEllipseObj *>::const_iterator comicEllipseObjIter;
      unsigned int stringSize = 0;
      appendUChar4(_charVector, stringSize);

      // (1) Number of Text Shape Objects (unsigned char);
      //     the individual objects are separated by the character '$'.
      unsigned char size;
      intToChar(_comicEllipseObjVec.size(), size);
      _charVector.push_back(size);
      stringSize++;

      if (_comicEllipseObjVec.size() > 255)
	{
#ifdef SMAGE_PRINT_DEBUG
	  _SmageLogFile.lprintf("*** Warning (SmageComicEllipseShape::generateComicEllipseShapeString): too many objects ***\n");
#endif
	  return;
	}

      for (comicEllipseObjIter=_comicEllipseObjVec.begin(); comicEllipseObjIter!=_comicEllipseObjVec.end(); 
	   comicEllipseObjIter++)
	{
	  // (2) Append the ellipse shape data
	  getEllipseShapeString(_charVector, &((*comicEllipseObjIter)->ellipseObjData), stringSize);

	  if ((*comicEllipseObjIter)->ellipseObjData.textObjData.shapeType == SMAGE_SPEECH_BUBBLE)
	    {
	      // (3a) Append bubble peak position (2 int)
	      appendChar4(_charVector, (*comicEllipseObjIter)->bubblePeakPos.x());
	      appendChar4(_charVector, (*comicEllipseObjIter)->bubblePeakPos.y());

	      // (3b) Append open and close angle (2 int)
	      appendChar4(_charVector, (*comicEllipseObjIter)->openAngle);
	      appendChar4(_charVector, (*comicEllipseObjIter)->closeAngle);

	      stringSize += 16;
	    }
	  else if ((*comicEllipseObjIter)->ellipseObjData.textObjData.shapeType == SMAGE_THINK_BUBBLE)
	    {
	      // (3a) Append number of bubble circles (unsigned char)
	      intToChar((*comicEllipseObjIter)->thinkBubbleCircles.size(), size);
	      _charVector.push_back(size);
	      stringSize++;

	      // (3b) Append the circles
	      for (unsigned char i=0; i<size; i++)
		{
		  // (3b1) Append the circle's position (2 int)
		  appendChar4(_charVector, (*comicEllipseObjIter)->thinkBubbleCircles[i].pos.x());
		  appendChar4(_charVector, (*comicEllipseObjIter)->thinkBubbleCircles[i].pos.y());

		  // (3b2) Append the circle's size
		  appendChar4(_charVector, (*comicEllipseObjIter)->thinkBubbleCircles[i].size.width());
		  appendChar4(_charVector, (*comicEllipseObjIter)->thinkBubbleCircles[i].size.height());

		  stringSize += 16;
		}
	    }
	  
	  _charVector.push_back('$');
	  stringSize++;
	}
      
      putFrontUChar4(_charVector, stringSize);
    }
}


bool SmageComicEllipseShape::getComicEllipseShapeObject(const vector<char> &charVec, vector<char>::const_iterator &charVecIter,
							SmageComicEllipseObj *obj)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageComicEllipseShape::getComicEllipseShapeObject()\n");
#endif

  char char4[4];
  unsigned char size;
  int anInt1, anInt2;
  
  // Ellipse object data
  getEllipseShapeObject(charVec, charVecIter, &obj->ellipseObjData);
 
  if (obj->ellipseObjData.textObjData.shapeType == SMAGE_SPEECH_BUBBLE)
    {
      // Bubble's peak position
      unsigned int i;
      for (i=0; i<4; i++) {
	char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
      charToInt(char4, anInt1);
      for (i=0; i<4; i++) {
	char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
      charToInt(char4, anInt2);
      
      obj->bubblePeakPos = QPoint(anInt1, anInt2);

      // Bubble's open and close angle
      for (i=0; i<4; i++) {
	char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
      charToInt(char4, anInt1);
      for (i=0; i<4; i++) {
	char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
      charToInt(char4, anInt2);
      
      obj->openAngle = anInt1;
      obj->closeAngle = anInt2;
    }
  else if (obj->ellipseObjData.textObjData.shapeType == SMAGE_THINK_BUBBLE)
    {
      size = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
      vector<SmageThinkBubbleCircle> bubbleCircles;

      for (unsigned char i=0; i<size; i++)
	{
	  SmageThinkBubbleCircle bubbleCircle;
	  unsigned int j;
	  for (j=0; j<4; j++) {
	    char4[j] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
	  charToInt(char4, anInt1);
	  for (j=0; j<4; j++) {
	    char4[j] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
	  charToInt(char4, anInt2);
      
	  bubbleCircle.pos = QPoint(anInt1, anInt2);

	  for (j=0; j<4; j++) {
	    char4[j] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
	  charToInt(char4, anInt1);
	  for (j=0; j<4; j++) {
	    char4[j] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
	  charToInt(char4, anInt2);

	  bubbleCircle.size = QSize(anInt1, anInt2);

	  bubbleCircles.push_back(bubbleCircle);
	}

      obj->thinkBubbleCircles = bubbleCircles;
    }

  return TRUE;
}

bool SmageComicEllipseShape::getComicEllipseShapeObjects(const vector<char> &charVec)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageComicEllipseShape::getComicEllipseShapeObjects()\n");
#endif

  char aChar, char4[4];
  unsigned char size;
  unsigned int anUInt;

  vector<char>::const_iterator charVecIter = charVec.begin();
  unsigned int i;
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToUInt(char4, anUInt);

  size = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  if (size <= 0) return FALSE;

  for (unsigned char j=0; j<size; j++)
    {
      if (charVecIter == charVec.end()) return FALSE;

      SmageComicEllipseObj *comicEllipseObj = new SmageComicEllipseObj;
      if (!getComicEllipseShapeObject(charVec, charVecIter, comicEllipseObj))
	return FALSE;

      aChar = *charVecIter; charVecIter++; 
      if (aChar != '$') return FALSE;

      _comicEllipseObjVec.push_back(comicEllipseObj);
    }

#ifdef SMAGE_PRINT_DEBUG
  for (i=0; i<_comicEllipseObjVec.size(); i++)
    printDebugInfos(_comicEllipseObjVec[i]);
#endif

  return TRUE;
}

void SmageComicEllipseShape::setComicEllipseObject(const SmageComicEllipseObj *obj)
{
  SmageComicEllipseObj *cObj = new SmageComicEllipseObj;
  cObj->ellipseObjData.textObjData.position = obj->ellipseObjData.textObjData.position;
  cObj->ellipseObjData.textObjData.lineWidth = obj->ellipseObjData.textObjData.lineWidth;
  cObj->ellipseObjData.textObjData.textPosition = obj->ellipseObjData.textObjData.textPosition;
  cObj->ellipseObjData.textObjData.shapeType = obj->ellipseObjData.textObjData.shapeType;
  cObj->ellipseObjData.textObjData.textData.text = obj->ellipseObjData.textObjData.textData.text;
  cObj->ellipseObjData.textObjData.textData.textFont = obj->ellipseObjData.textObjData.textData.textFont;
  cObj->ellipseObjData.textObjData.textData.textAlignment = obj->ellipseObjData.textObjData.textData.textAlignment;

  cObj->ellipseObjData.textObjData.colors.textColor = obj->ellipseObjData.textObjData.colors.textColor;
  cObj->ellipseObjData.textObjData.colors.outlineColor = obj->ellipseObjData.textObjData.colors.outlineColor;
  cObj->ellipseObjData.textObjData.colors.backgroundColor = obj->ellipseObjData.textObjData.colors.backgroundColor;
  cObj->ellipseObjData.textObjData.colors.backgroundAlpha = obj->ellipseObjData.textObjData.colors.backgroundAlpha;
  cObj->ellipseObjData.ellipseShapeRect = obj->ellipseObjData.ellipseShapeRect;

  cObj->bubblePeakPos = obj->bubblePeakPos;
  cObj->openAngle = obj->openAngle;
  cObj->closeAngle = obj->closeAngle;

  for (unsigned int i=0; i<obj->thinkBubbleCircles.size(); i++)
    cObj->thinkBubbleCircles.push_back(obj->thinkBubbleCircles[i]);

  _comicEllipseObjVec.push_back(cObj);
}

void SmageComicEllipseShape::printDebugInfos(const SmageComicEllipseObj *obj)
{  
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageComicEllipseShape::printDebugInfos()\n");

  SmageEllipseShape::printDebugInfos(&obj->ellipseObjData);
  
  _SmageLogFile.lprintf("Comic ellipse type: %i\n", obj->ellipseObjData.textObjData.shapeType);
  _SmageLogFile.lprintf("Bubble peak position: %i %i\n", obj->bubblePeakPos.x(), obj->bubblePeakPos.y());
  _SmageLogFile.lprintf("Bubble open angle: %i  close angle: %i\n", obj->openAngle, obj->closeAngle);

  _SmageLogFile.lprintf("Think bubble circles:\n");

  for (unsigned int i=0; i<obj->thinkBubbleCircles.size(); i++)
    {
      _SmageLogFile.lprintf("Circle position: %i %i\n", obj->thinkBubbleCircles[i].pos.x(), obj->thinkBubbleCircles[i].pos.y());
      _SmageLogFile.lprintf("Circle size: width: %i  height: %i\n", 
			    obj->thinkBubbleCircles[i].size.width(), obj->thinkBubbleCircles[i].size.height());
    }
#endif
}
