#include "SmageEllipseShape.h"
#include "SmageSettings.h"

#include <iostream>
#include <math.h>

#include <qtextstream.h>

using namespace std;

SmageEllipseObj::SmageEllipseObj()
{
  textObjData = SmageTextObj();
  ellipseShapeRect = QRect();
}

//  Constructor
SmageEllipseShape::SmageEllipseShape() : SmageTextShape()
{
  _ellipseObjVec.clear();
}

// Destructor
SmageEllipseShape::~SmageEllipseShape()
{
  if (!_ellipseObjVec.empty())
    {
      for (uint i=0; i<_ellipseObjVec.size(); i++)
	delete _ellipseObjVec[i];
    }
}


void SmageEllipseShape::getEllipseShapeString(vector<char> &charVec, SmageEllipseObj *ellipseObj,
					      unsigned int &stringSize)
{
  // Append text object data
  generateTextShapeString(charVec, ellipseObj->textObjData, stringSize);

  // Append ellipse object data
  
  // (1) Append the rectangle that describes the ellipse (4 x int)
  QRect rect = ellipseObj->ellipseShapeRect;
  appendChar4(charVec, rect.left());
  appendChar4(charVec, rect.top());
  appendChar4(charVec, rect.width());
  appendChar4(charVec, rect.height());

  stringSize+=16;
}


//   Generate a character string holding all text shape parameters
void SmageEllipseShape::generateEllipseShapeString()
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageEllipseShape::generateEllipseShapeString()\n");
#endif

  if (_ellipseObjVec.size() > 0)
    {
      vector<SmageEllipseObj *>::const_iterator ellipseObjIter;
      unsigned int stringSize = 0;
      appendUChar4(_charVector, stringSize);

      // (1) Number of Text Shape Objects (unsigned char);
      //     the individual objects are separated by the character '$'.
      unsigned char size;
      intToChar(_ellipseObjVec.size(), size);
      _charVector.push_back(size);
      stringSize++;

      if (_ellipseObjVec.size() > 255)
	{
#ifdef SMAGE_PRINT_DEBUG
	  _SmageLogFile.lprintf("*** Warning (SmageEllipseShape::generateEllipseShapeString): too many objects ***\n");
#endif
	  return;
	}
  
      for (ellipseObjIter=_ellipseObjVec.begin(); ellipseObjIter!=_ellipseObjVec.end(); ellipseObjIter++)
	{
	  getEllipseShapeString(_charVector, *ellipseObjIter, stringSize);
	  
	  _charVector.push_back('$');
	  stringSize++;
	}

      putFrontUChar4(_charVector, stringSize);
    }
}

bool SmageEllipseShape::getEllipseShapeObject(const vector<char> &charVec, vector<char>::const_iterator &charVecIter, 
					      SmageEllipseObj *obj)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageEllipseShape::getEllipseShapeObject()\n");
#endif

  char char4[4];
  int anInt1, anInt2, anInt3, anInt4;

  // Text object data
  getTextShapeObject(charVec, charVecIter, obj->textObjData);

  // Ellipse rectangle data
  unsigned int i;
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt1);
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt2);
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt3);
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt4);

  obj->ellipseShapeRect = QRect(QPoint(anInt1, anInt2), QSize(anInt3, anInt4));

#ifdef SMAGE_PRINT_DEBUG
  printDebugInfos(obj);
#endif

  return TRUE;
}

bool SmageEllipseShape::getEllipseShapeObjects(const vector<char> &charVec)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageEllipseShape::getEllipseShapeObjects()\n");
#endif

  char aChar, char4[4];
  unsigned char size;
  unsigned int anUInt;

  vector<char>::const_iterator charVecIter = charVec.begin();
  unsigned int i;
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToUInt(char4, anUInt);

  size = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  if (size <= 0) return FALSE;

  for (unsigned char j=0; j<size; j++)
    {
      if (charVecIter == charVec.end()) return FALSE;

      SmageEllipseObj *ellipseObj = new SmageEllipseObj;
      if (!getEllipseShapeObject(charVec, charVecIter, ellipseObj))
	return FALSE;

      aChar = *charVecIter; charVecIter++; 
      if (aChar != '$') return FALSE;

      _ellipseObjVec.push_back(ellipseObj);
    }

#ifdef SMAGE_PRINT_DEBUG
  for (i=0; i<_ellipseObjVec.size(); i++)
    printDebugInfos(_ellipseObjVec[i]);
#endif

  return TRUE;
}

void SmageEllipseShape::setEllipseObject(const SmageEllipseObj *obj)
{
  SmageEllipseObj *eObj = new SmageEllipseObj;
  eObj->textObjData.position = obj->textObjData.position;
  eObj->textObjData.lineWidth = obj->textObjData.lineWidth;
  eObj->textObjData.textPosition = obj->textObjData.textPosition;
  eObj->textObjData.shapeType = obj->textObjData.shapeType;
  eObj->textObjData.textData.text = obj->textObjData.textData.text;
  eObj->textObjData.textData.textFont = obj->textObjData.textData.textFont;
  eObj->textObjData.textData.textAlignment = obj->textObjData.textData.textAlignment;

  eObj->textObjData.colors.textColor = obj->textObjData.colors.textColor;
  eObj->textObjData.colors.outlineColor = obj->textObjData.colors.outlineColor;
  eObj->textObjData.colors.backgroundColor = obj->textObjData.colors.backgroundColor;
  eObj->textObjData.colors.backgroundAlpha = obj->textObjData.colors.backgroundAlpha;
  eObj->ellipseShapeRect = obj->ellipseShapeRect;

  _ellipseObjVec.push_back(eObj);
}

void SmageEllipseShape::printDebugInfos(const SmageEllipseObj *obj)
{  
#ifdef SMAGE_PRINT_DEBUG

  _SmageLogFile.lprintf("SmageEllipseShape::printDebugInfos()\n");

  SmageTextShape::printDebugInfos(obj->textObjData);
  
  _SmageLogFile.lprintf("Ellipse rectangle: topLeft (%i,%i) bottomRight (%i,%i)\n", 
			obj->ellipseShapeRect.left(), obj->ellipseShapeRect.top(),
			obj->ellipseShapeRect.right(), obj->ellipseShapeRect.bottom());
#endif
}


