#include "SmageObjShape.h"
#include "SmageSettings.h"

#include <iostream>
#include <math.h>

using namespace std;

SmageObjShape::SmageObjShape()
{
  _charVector.clear();
}

SmageObjShape::~SmageObjShape()
{
}

unsigned char
SmageObjShape::getTextShapeType(const TextShapeType &shapeType)
{
  unsigned char type = 0;

  switch (shapeType)
  {
    case SMAGE_PLAIN_TEXT: type = 0; break;
    case SMAGE_RECT: type = 1; break;
    case SMAGE_ELLIPSE: type = 2; break;
    case SMAGE_SPEECH_BUBBLE: type = 3; break;
    case SMAGE_THINK_BUBBLE: type = 4; break;
    default:
    {
#ifdef SMAGE_PRINT_DEBUG
      _SmageLogFile.lprintf("*** Warning (SmageObjShape::getTextShapeType): Unknown type ***\n");
#endif
      break;
    }
  }
  
  return type;
}

TextShapeType SmageObjShape::getTextShapeType(const unsigned char &shapeType)
{
  TextShapeType retShapeType = SMAGE_RECT;
  
  switch (shapeType)
  {
    case 0: retShapeType = SMAGE_PLAIN_TEXT; break;
    case 1: retShapeType = SMAGE_RECT; break;
    case 2: retShapeType = SMAGE_ELLIPSE; break;
    case 3: retShapeType = SMAGE_SPEECH_BUBBLE; break;
    case 4: retShapeType = SMAGE_THINK_BUBBLE; break;
    default:
    {
#ifdef SMAGE_PRINT_DEBUG
      _SmageLogFile.lprintf("*** Warning (SmageObjShape::getTextShapeType): Unknown type ***\n");
#endif
      break;
    }
  }

  return retShapeType;
}

void SmageObjShape::intToChar(const int &src, char dest[4])
{
  dest[0] = 0; dest[1] = 0; dest[2] = 0; dest[3] = 0;

  int srcCopy = src;
  dest[0] = (srcCopy & 255); srcCopy = (srcCopy >> 8);
  dest[1] = (srcCopy & 255); srcCopy = (srcCopy >> 8);
  dest[2] = (srcCopy & 255); srcCopy = (srcCopy >> 8);
  dest[3] = (srcCopy & 255);
}

void SmageObjShape::intToChar(const int &src, unsigned char &dest)
{
  dest = 0;

  if (src > 0 && src < 256)
    dest |= src & 255;
  else if (src > 255)
    dest = 255;
}

void SmageObjShape::charToInt(const char src[4], int &dest)
{
  dest = 0;

  dest |= ((src[3] & 255) << 24);
  dest |= ((src[2] & 255) << 16);
  dest |= ((src[1] & 255) << 8);
  dest |= (src[0] & 255);
}


void SmageObjShape::uIntToChar(const unsigned int &src, char dest[4])
{
  dest[0] = 0; dest[1] = 0; dest[2] = 0; dest[3] = 0;

  unsigned int srcCopy = src;
  dest[0] = (srcCopy & 255); srcCopy = (srcCopy >> 8);
  dest[1] = (srcCopy & 255); srcCopy = (srcCopy >> 8);
  dest[2] = (srcCopy & 255); srcCopy = (srcCopy >> 8);
  dest[3] = (srcCopy & 255);
}


void SmageObjShape::charToUInt(const char src[4], unsigned int &dest)
{
  dest = 0;

  dest |= ((src[3] & 255) << 24);
  dest |= ((src[2] & 255) << 16);
  dest |= ((src[1] & 255) << 8);
  dest |= (src[0] & 255);
}


void SmageObjShape::appendChar4(vector<char> &charVec, const int &data)
{
  char charData[4];
  intToChar(data, charData);

  for (unsigned int i=0; i<4; i++)
    charVec.push_back(charData[i]);
}

void SmageObjShape::appendUChar4(vector<char> &charVec, const unsigned int &data)
{
  char charData[4];
  uIntToChar(data, charData);

  for (unsigned int i=0; i<4; i++)
    charVec.push_back(charData[i]);
}

void SmageObjShape::putFrontUChar4(vector<char> &charVec, const unsigned int &data)
{
  char charData[4];
  uIntToChar(data, charData);

  for (unsigned int i=0; i<4; i++)
    charVec[i] = charData[i];
}

void SmageObjShape::charVecToString(const vector<char> &charVec, QString &string)
{
  vector<char>::const_iterator vecIter;
  string = QString("");

  for (vecIter=charVec.begin(); vecIter!=charVec.end(); vecIter++)
    string += *vecIter;
}


void SmageObjShape::getTextShapes(const vector<char> &charVec, vector<SmageTextShapes> &shapesVec)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageObjShape::getTextShapes()\n");
#endif

  if (charVec.size() < 4) 
    {
#ifdef SMAGE_PRINT_DEBUG
      _SmageLogFile.lprintf("*** Warning (SmageObjShape::getTextShapes): Char vector has no data\n");
#endif

      return;
    }

  unsigned int currentSize;
  char char4[4];
  unsigned char anUChar;

  vector<char>::const_iterator charVecIter = charVec.begin();
  vector<char>::const_iterator tmpCharVecIter1, tmpCharVecIter2;

  while (charVecIter != charVec.end())
    {
      SmageTextShapes textShapes;

      tmpCharVecIter1 = charVecIter;

      // Extract the string length of the current encoded text shape
      for (unsigned int i=0; i<4; i++) {
	char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) break; }
      charToUInt(char4, currentSize);

      tmpCharVecIter2 = charVecIter + currentSize;

      unsigned char size;
      size = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) break;
      if (size <= 0) break;

      // Extract the encoded text shape's type
      anUChar = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) break;
      textShapes.shapeType = getTextShapeType(anUChar);

      textShapes.shapeString = vector<char>(tmpCharVecIter1, tmpCharVecIter2);
      charVecIter = tmpCharVecIter2;

      shapesVec.push_back(textShapes);
    }
}
