#ifndef  _SMAGE_OBJ_SHAPE_H
#define  _SMAGE_OBJ_SHAPE_H

#include <qstring.h>
#include <vector>

/** Shape types.
  */
enum TextShapeType { SMAGE_PLAIN_TEXT, SMAGE_RECT, SMAGE_ELLIPSE, SMAGE_SPEECH_BUBBLE, SMAGE_THINK_BUBBLE };

struct SmageTextShapes {
  SmageTextShapes()
  {
    shapeType = SMAGE_PLAIN_TEXT;
    shapeString.clear();
  }

  TextShapeType shapeType;
  std::vector<char> shapeString;
};

/** Class SmageObjShape is the base class for arbitrary shapes that can contain texts or images.
  *
  * @ingroup LibSmage
  *
  *
  */
class SmageObjShape
{
  public:
    /** Constructor.
      */
    SmageObjShape();

    /**
      */
    unsigned char getTextShapeType(const TextShapeType &shapeType);
    TextShapeType getTextShapeType(const unsigned char &shapeType);

    static void intToChar(const int &src, char dest[4]);
    static void intToChar(const int &src, unsigned char &dest);
    static void charToInt(const char src[4], int &dest);
    static void uIntToChar(const unsigned int &src, char dest[4]);
    static void charToUInt(const char src[4], unsigned int &dest);
    
    
    static void putFrontUChar4(std::vector<char> &charVec, const unsigned int &data);
    static void appendChar4(std::vector<char> &charVec, const int &data);
    static void appendUChar4(std::vector<char> &charVec, const unsigned int &data);
    static void charVecToString(const std::vector<char> &charVec, QString &string);

    std::vector<char> getCharVector(){ return _charVector; }
    void getTextShapes(const std::vector<char> &charVec, std::vector<SmageTextShapes> &shapesVec);
    
    /** Destructor.
     */
    ~SmageObjShape();
     
  protected: 
    /** Vector holding all necessary data for displaying the shape object.
      */
    std::vector<char> _charVector;
};

#endif
