#include "SmageTextShape.h"
#include "SmageSettings.h"

#include <iostream>
#include <math.h>

#include <qtextstream.h>

using namespace std;

SmageTextShapeColors::SmageTextShapeColors()
{
  textColor = QColor(0,0,0);
  outlineColor = QColor(0,0,0);
  backgroundColor = QColor(255,255,255);
  backgroundAlpha = 255;
}

SmageTextData::SmageTextData()
{
  text = QString("");
  textFont = QFont();
  textAlignment = 2;
}

SmageTextObj::SmageTextObj()
{
  position = QPoint(0,0);
  textPosition = QPoint(0,0);
  lineWidth = 2;
  shapeType = SMAGE_RECT;
  textData = SmageTextData();
  colors = SmageTextShapeColors();
}

//  Constructor
SmageTextShape::SmageTextShape() : SmageObjShape()
{
}

// Destructor
SmageTextShape::~SmageTextShape()
{
}

// Appending colors to "textStream"
void SmageTextShape::appendTextColors(vector<char> &charVec, const SmageTextObj *textObj, unsigned int &stringSize)
{
  // Appending the 3 colors ("textColor", "outlineColor", "backgroundColor") to "textStream";
  // each color is composed of 3 values (unsigned chars): Red, Green, Blue
  unsigned char aChar;

  intToChar(textObj->colors.textColor.red(), aChar); charVec.push_back(aChar);
  intToChar(textObj->colors.textColor.green(), aChar); charVec.push_back(aChar);
  intToChar(textObj->colors.textColor.blue(), aChar); charVec.push_back(aChar);

  intToChar(textObj->colors.outlineColor.red(), aChar); charVec.push_back(aChar);
  intToChar(textObj->colors.outlineColor.green(), aChar); charVec.push_back(aChar);
  intToChar(textObj->colors.outlineColor.blue(), aChar); charVec.push_back(aChar);

  intToChar(textObj->colors.backgroundColor.red(), aChar); charVec.push_back(aChar);
  intToChar(textObj->colors.backgroundColor.green(), aChar); charVec.push_back(aChar);
  intToChar(textObj->colors.backgroundColor.blue(), aChar); charVec.push_back(aChar);

  // Appending the background alpha value (unsigned char)
  charVec.push_back((unsigned char)(textObj->colors.backgroundAlpha));

  stringSize+=10;
}

// Appending text objects (their fonts and actual texts) to the "textStream"
void SmageTextShape::appendText(vector<char> &charVec, const SmageTextObj *textObj, unsigned int &stringSize)
{
  // Font data: style (uchar), look (uchar), size (uchar)
  unsigned char style, look, size;
  getTextFont(textObj->textData.textFont, textObj->textData.textAlignment, style, look, size);
  charVec.push_back(style);
  charVec.push_back(look);
  charVec.push_back(size);
  stringSize+=3;

  // Size of text (uint)
  appendUChar4(charVec, textObj->textData.text.length());
  stringSize+=4;

  // The actual text to be displayed
  for (unsigned int i=0; i<textObj->textData.text.length(); i++)
    charVec.push_back(textObj->textData.text.at(i));

  stringSize+=(unsigned long) textObj->textData.text.length();
}

// Code text font characteristics in a number of characters
void SmageTextShape::getTextFont(const QFont &font, const int &textAlignment, unsigned char &style, 
				 unsigned char &look, unsigned char &size)
{
  getStyle(font, style);
  getLook(font, textAlignment, look);

  int pointSize = font.pointSize();
  if (pointSize < 0)
    pointSize = 10;

  if (pointSize > 255)
    pointSize = 255;

  size = (unsigned char)(pointSize);
}

void SmageTextShape::getStyle(const QFont &font, unsigned char &style)
{
  if (font.family() == "Arial") style = 1;
  else if (font.family() == "Arial Unicode MS") style = 2;
  else if (font.family() == "Bookman L") style = 3;
  else if (font.family() == "Caslon") style = 4;
  else if (font.family() == "Century Schoolbook L") style = 5;
  else if (font.family() == "Chancery L") style = 6;
  else if (font.family() == "Chaparral Display") style = 7;
  else if (font.family() == "Charter") style = 8;
  else if (font.family() == "Clearlyu Arabic Extra") style = 9;
  else if (font.family() == "Clearlyu Devangari Extra") style = 10;
  else if (font.family() == "Code2000") style = 11;
  else if (font.family() == "Comic Sans MS") style = 12;
  else if (font.family() == "Console") style = 13;
  else if (font.family() == "Courier") style = 14;
  else if (font.family() == "Courier New") style = 15;
  else if (font.family() == "Cursor") style = 16;
  else if (font.family() == "Dialog") style = 17;
  else if (font.family() == "Dingbats") style = 18;
  else if (font.family() == "Fixed") style = 19;
  else if (font.family() == "Fixedsys") style = 20;
  else if (font.family() == "Franklin Gothic Medium") style = 21;
  else if (font.family() == "Gentium") style = 22;
  else if (font.family() == "Georgia") style = 23;
  else if (font.family() == "Gothic") style = 24;
  else if (font.family() == "Haettenschweiler") style = 25;
  else if (font.family() == "Helvetica") style = 26;
  else if (font.family() == "Lucida") style = 27;
  else if (font.family() == "Lucidabright") style = 28;
  else if (font.family() == "Lucidatypewriter") style = 29;
  else if (font.family() == "Lucida Console") style = 30;
  else if (font.family() == "Lucida Sans") style = 31;
  else if (font.family() == "Luxi Mono") style = 32;
  else if (font.family() == "Luxi Sans") style = 33;
  else if (font.family() == "Luxi Serif") style = 34;
  else if (font.family() == "Modern") style = 35;
  else if (font.family() == "MS Sans Serif") style = 36;
  else if (font.family() == "MS Serif") style = 37;
  else if (font.family() == "New Century Schoolbook") style = 38;
  else if (font.family() == "Newspaper") style = 39;
  else if (font.family() == "Nimbus Mono L") style = 40;
  else if (font.family() == "Nimbus Roman No9 L") style = 41;
  else if (font.family() == "Nimbus Sans l") style = 42;
  else if (font.family() == "Palatino Linotype") style = 43;
  else if (font.family() == "Palladio L") style = 44;
  else if (font.family() == "Roman") style = 45;
  else if (font.family() == "SansSerif") style = 46;
  else if (font.family() == "Script") style = 47;
  else if (font.family() == "Serif") style = 48;
  else if (font.family() == "SF Diego Sans") style = 49;
  else if (font.family() == "SF Diego Sans Outline") style = 50;
  else if (font.family() == "SF Diego Sans Shaded") style = 51;
  else if (font.family() == "Small Fonts") style = 52;
  else if (font.family() == "Standard Symbols L") style = 53;
  else if (font.family() == "Symbol") style = 54;
  else if (font.family() == "System") style = 55;
  else if (font.family() == "Tahoma") style = 56;
  else if (font.family() == "Terminal") style = 57;
  else if (font.family() == "Times") style = 58;
  else if (font.family() == "Times ExtraBold") style = 59;
  else if (font.family() == "Times New Roman") style = 60;
  else if (font.family() == "Times SC") style = 61;
  else if (font.family() == "Trebuchet MS") style = 62;
  else if (font.family() == "Unifont") style = 63;
  else if (font.family() == "Utopia") style = 64;
  else if (font.family() == "Verdana") style = 65;
  else if (font.family() == "Vivaldi") style = 66;
  else if (font.family() == "Vladimir Script") style = 67;
  else if (font.family() == "Willow") style = 68;
  else if (font.family() == "Wingdings") style = 69;
  else if (font.family() == "Y.OzFont") style = 70;
  else style = 1;
}

void SmageTextShape::setStyle(QFont &font, const unsigned char &style)
{  
  switch (style)
  {
    case 1: font.setFamily("Arial"); break;
    case 2: font.setFamily("Arial Unicode MS"); break;
    case 3: font.setFamily("Bookman L"); break;
    case 4: font.setFamily("Caslon"); break;
    case 5: font.setFamily("Century Schoolbook L"); break;
    case 6: font.setFamily("Chancery L"); break;
    case 7: font.setFamily("Chaparral Display"); break;
    case 8: font.setFamily("Charter"); break;
    case 9: font.setFamily("Clearlyu Arabic Extra"); break;
    case 10: font.setFamily("Clearlyu Devangari Extra"); break;
    case 11: font.setFamily("Code2000"); break;
    case 12: font.setFamily("Comic Sans MS"); break;
    case 13: font.setFamily("Console"); break;
    case 14: font.setFamily("Courier"); break;
    case 15: font.setFamily("Courier New"); break;
    case 16: font.setFamily("Cursor"); break;
    case 17: font.setFamily("Dialog"); break;
    case 18: font.setFamily("Dingbats"); break;
    case 19: font.setFamily("Fixed"); break;
    case 20: font.setFamily("Fixedsys"); break;
    case 21: font.setFamily("Franklin Gothic Medium"); break;
    case 22: font.setFamily("Gentium"); break;
    case 23: font.setFamily("Georgia"); break;
    case 24: font.setFamily("Gothic"); break;
    case 25: font.setFamily("Haettenschweiler"); break;
    case 26: font.setFamily("Helvetica"); break;
    case 27: font.setFamily("Lucida"); break;
    case 28: font.setFamily("Lucidabright"); break;
    case 29: font.setFamily("Lucidatypewriter"); break;
    case 30: font.setFamily("Lucida Console"); break;
    case 31: font.setFamily("Lucida Sans"); break;
    case 32: font.setFamily("Luxi Mono"); break;
    case 33: font.setFamily("Luxi Sans"); break;
    case 34: font.setFamily("Luxi Serif"); break;
    case 35: font.setFamily("Modern"); break;
    case 36: font.setFamily("MS Sans Serif"); break;
    case 37: font.setFamily("MS Serif"); break;
    case 38: font.setFamily("New Century Schoolbook"); break;
    case 39: font.setFamily("Newspaper"); break;
    case 40: font.setFamily("Nimbus Mono L"); break;
    case 41: font.setFamily("Nimbus Roman No9 L"); break;
    case 42: font.setFamily("Nimbus Sans l"); break;
    case 43: font.setFamily("Palatino Linotype"); break;
    case 44: font.setFamily("Palladio L"); break;
    case 45: font.setFamily("Roman"); break;
    case 46: font.setFamily("SansSerif"); break;
    case 47: font.setFamily("Script"); break;
    case 48: font.setFamily("Serif"); break;
    case 49: font.setFamily("SF Diego Sans"); break;
    case 50: font.setFamily("SF Diego Sans Outline"); break;
    case 51: font.setFamily("SF Diego Sans Shaded"); break;
    case 52: font.setFamily("Small Fonts"); break;
    case 53: font.setFamily("Standard Symbols L"); break;
    case 54: font.setFamily("Symbol"); break;
    case 55: font.setFamily("System"); break;
    case 56: font.setFamily("Tahoma"); break;
    case 57: font.setFamily("Terminal"); break;
    case 58: font.setFamily("Times"); break;
    case 59: font.setFamily("Times ExtraBold"); break;
    case 60: font.setFamily("Times New Roman"); break;
    case 61: font.setFamily("Times SC"); break;
    case 62: font.setFamily("Trebuchet MS"); break;
    case 63: font.setFamily("Unifont"); break;
    case 64: font.setFamily("Utopia"); break;
    case 65: font.setFamily("Verdana"); break;
    case 66: font.setFamily("Vivaldi"); break;
    case 67: font.setFamily("Vladimir Script"); break;
    case 68: font.setFamily("Willow"); break;
    case 69: font.setFamily("Wingdings"); break;
    case 70: font.setFamily("Y.OzFont"); break;
    default: font.setFamily("Arial"); break;
  }
}


void SmageTextShape::getLook(const QFont &font, const int &alignment, unsigned char &look)
{
  look = 0;

  // Last bit of "look" is set if font is italic
  if (font.italic())
    look |= 128;

  // 6th and 7th bit are reserved for alignment
  if (alignment == 1) // AlignLeft - 7th bit is set
    look |= 64;
  else if (alignment == 2) // AlignHCenter - 6th bit is set
    look |= 32;
  else if (alignment == 3) // AlignRight - 6th and 7th bit are set
    look |= 96;

  // The first 5 bits are reserved for font weights
  switch (font.weight())
  {
    case QFont::Normal: break; // first bit remains unset
    case QFont::Light: look |= 1; break;
    case QFont::DemiBold: look |= 2; break;
    case QFont::Bold: look |= 3; break;
    case QFont::Black: look |= 4; break;
    default: break; // equal to normal
  }
}

void SmageTextShape::setLook(QFont &font, int &alignment, const unsigned char &look)
{
  font.setItalic(FALSE);

  // Last bit of "look" is set if font is italic
  if ((look & 128) == 128)
    font.setItalic(TRUE);

  // 6th and 7th bit are reserved for alignment
  if ((look & 96) == 96) // AlignRight
    alignment = 3;
  else if ((look & 64) == 64) // AlignLeft
    alignment = 1;
  else  // AlignHCenter
    alignment = 2;

  // The first 7 bits are reserved for font weights
  if (look == 0) font.setWeight(QFont::Normal);
  else if ((look & 4) == 4) font.setWeight(QFont::Black);
  else if ((look & 3) == 3) font.setWeight(QFont::Bold);
  else if ((look & 2) == 2) font.setWeight(QFont::DemiBold);
  else if ((look & 1) == 1) font.setWeight(QFont::Light);
  else font.setWeight(QFont::Normal);
}

//   Generate a character string holding all text shape parameters
void SmageTextShape::generateTextShapeString(vector<char> &charVector, const SmageTextObj &textObj,
					     unsigned int &stringSize)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageTextShape::generateTextShapeString()\n");
#endif

  // (1) Type of Text Shape Object (unsigned char)
  charVector.push_back(getTextShapeType(textObj.shapeType));
  stringSize++;

  // (2) Text object's position (2 int -> 8 signed char)
  appendChar4(charVector, textObj.position.x());
  appendChar4(charVector, textObj.position.y());
  stringSize+=8;

  // (3) Outline width (unsigned char)
  charVector.push_back(textObj.lineWidth);
  stringSize++;

  // (4) Text position (2 int -> 8 signed char)
  appendChar4(charVector, textObj.textPosition.x());
  appendChar4(charVector, textObj.textPosition.y());
  stringSize+=8;

  // (5) Text shape colors
  appendTextColors(charVector, &textObj, stringSize);

  // (6) Font of texts including style, weight/italic, and size 
  //     as well as the actual text strings
  appendText(charVector, &textObj, stringSize);
}


bool SmageTextShape::getTextShapeObject(const vector<char> &charVec, vector<char>::const_iterator &charVecIter, 
					SmageTextObj &obj)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageTextShape::getTextShapeObject()\n");
#endif

  obj = SmageTextObj();
  char char4[4];
  unsigned char anUChar1, anUChar2, anUChar3;
  int anInt1, anInt2;
  unsigned int anUInt;
  QString aString;

  // Object's type
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  obj.shapeType = getTextShapeType(anUChar1);
  unsigned int i;
  // Object's position
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt1);
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt2);

  obj.position = QPoint(anInt1, anInt2);

  // Object's line width
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; 
  obj.lineWidth = anUChar1;

  // Object's text position
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt1);
  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToInt(char4, anInt2);

  obj.textPosition = QPoint(anInt1, anInt2);

  // #### Object colors
  
  // Text color
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  anUChar2 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  anUChar3 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;

  obj.colors.textColor = QColor(int(anUChar1), int(anUChar2), int(anUChar3));

  // Outline color
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  anUChar2 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  anUChar3 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;

  obj.colors.outlineColor = QColor(int(anUChar1), int(anUChar2), int(anUChar3));

  // Background color
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  anUChar2 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  anUChar3 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;

  obj.colors.backgroundColor = QColor(int(anUChar1), int(anUChar2), int(anUChar3));

  // Background alpha
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  obj.colors.backgroundAlpha = anUChar1;

  // #### Object's text
  QFont aFont;
  int alignment;
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  setStyle(aFont, anUChar1);
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  setLook(aFont, alignment, anUChar1);
  anUChar1 = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE;
  aFont.setPointSize(int(anUChar1));
  
  obj.textData.textFont = QFont(aFont);
  obj.textData.textAlignment = alignment;

  for (i=0; i<4; i++) {
    char4[i] = *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }
  charToUInt(char4, anUInt);

  aString = QString("");
  for (i=0; i<anUInt; i++) {
    aString += *charVecIter; charVecIter++; if (charVecIter == charVec.end()) return FALSE; }

  obj.textData.text = QString(aString);

  // **********************************

#ifdef SMAGE_PRINT_DEBUG
  printDebugInfos(obj);
#endif

  return TRUE;
}


void SmageTextShape::printDebugInfos(const SmageTextObj &obj)
{
#ifdef SMAGE_PRINT_DEBUG
  _SmageLogFile.lprintf("SmageTextShape::printDebugInfos()\n");
  
  _SmageLogFile.lprintf("Object type: %i\n", obj.shapeType);
  _SmageLogFile.lprintf("Object position: %i %i\n", obj.position.x(), obj.position.y());
  _SmageLogFile.lprintf("Object outline width: %i\n", int(obj.lineWidth));
  _SmageLogFile.lprintf("Object's text position: %i %i\n", obj.textPosition.x(), obj.textPosition.y());
  _SmageLogFile.lprintf("Object's text: %s\n", obj.textData.text.ascii());
  _SmageLogFile.lprintf("Object's font family: %s  point size: %i\n", 
			obj.textData.textFont.family().ascii(), obj.textData.textFont.pointSize());
  _SmageLogFile.lprintf("Object's colors: text: (%i,%i,%i) outline: (%i,%i,%i) background: (%i,%i,%i)\n", 
			obj.colors.textColor.red(), obj.colors.textColor.green(), obj.colors.textColor.blue(),
			obj.colors.outlineColor.red(), obj.colors.outlineColor.green(), obj.colors.outlineColor.blue(),
			obj.colors.backgroundColor.red(), obj.colors.backgroundColor.green(), obj.colors.backgroundColor.blue());
#endif
}
