#ifndef  _SMAGE_TEXT_SHAPE_H
#define  _SMAGE_TEXT_SHAPE_H

#include "SmageObjShape.h"

#include <qcolor.h>
#include <qfont.h>
#include <qpointarray.h>


/** Structure for the different colors (text, outline, background)
  *
  * @ingroup LibSmage
  *
  */
struct SmageTextShapeColors {
  SmageTextShapeColors();           // Constructor
  QColor textColor;                 // Text color
  QColor outlineColor;              // Outline color
  QColor backgroundColor;           // Background color
  unsigned char backgroundAlpha;    // Background's transparency 
                                    // (0=fully transparent, 255=opaque)
};

/**  Structure for text objects
  *
  * @ingroup LibSmage
  *
  */
struct SmageTextData {
  SmageTextData();
  QString text;
  QFont textFont;
  int textAlignment;                // 1 - AlignLeft, 2 - AlignHCenter, 3 - AlignRight
};

/** Structure for text shape items .
  * (a comic bubble may be composed of several bubbles each of which may 
  * hold a text fragment)
  *
  * @ingroup LibSmage
  *
  */
struct SmageTextObj {
  SmageTextObj();                   // Constructor
  QPoint position;                  // Text object's position
  unsigned char lineWidth;          // Width of outline
  QPoint textPosition;              // Text's position within the text object
  TextShapeType shapeType;          // Type of shape
  SmageTextData textData;           // text 
  SmageTextShapeColors colors;      // Text shape colors
};

/** Class SmageTextShape for text boxes, bubbles, and other shapes in which text can be placed.
  *
  * @ingroup LibSmage
  *
  */
class SmageTextShape : public SmageObjShape
{
  public:
    /** Constructor.
      */
    SmageTextShape();
      
    /** Destructor.
      */
    ~SmageTextShape();

    static void appendTextColors(std::vector<char> &charVec, const SmageTextObj *textObj, unsigned int &stringSize);
    static void appendText(std::vector<char> &charVec, const SmageTextObj *textObj, unsigned int &stringSize);
    static void getTextFont(const QFont &font, const int &alignment, unsigned char &style, unsigned char &look, unsigned char &size);
    static void getStyle(const QFont &font, unsigned char &style);
    static void setStyle(QFont &font, const unsigned char &style);
    static void getLook(const QFont &font, const int &alignment, unsigned char &look);
    static void setLook(QFont &font, int &alignment, const unsigned char &look);

    void generateTextShapeString(std::vector<char> &charVec, const SmageTextObj &textObj, unsigned int &stringSize);
    bool getTextShapeObject(const std::vector<char> &charVec, std::vector<char>::const_iterator &charVecIter, SmageTextObj &obj);

    void printDebugInfos(const SmageTextObj &obj);
};

#endif
