/** @defgroup NPSmagePlugin Mozilla Plugin
  *
  * <h3>Building The Plugin</h3>
  *
  * The plugin uses a modified version of the qnp api. To build the qnp library
  * the following files are required:
  * <ul>
  * <li>qnp.h</li>
  * <li>qnp.cpp</li>
  * </ul>
  * both provided by Trolltech/Qt.
  *
  * <ul>
  * <li>jri.h</li>
  * <li>jri_md.h</li>
  * <li>jritypes.h</li>
  * <li>npapi.h</li>
  * <li>npmac.cpp</li>
  * <li>npunix.c</li>
  * <li>npupp.h</li>
  * <li>npwin.cpp</li>
  * </ul>
  * Provided by the
  * <a href="ftp://ftp.netscape.com/pub/sdk/plugin/windows/oct_21_97/winNPSDK.zip">Netscape Plugin SDK</a>
  *
  * To build the modified qnp library, the following modifications need to be done:
  * <ol>
  * <li>change the QNPInstance::getURLNotify function in qnp.cpp and qnp.h to return an error code:
    <code><pre>
      int QNPInstance::getURLNotify(const char* url, const char* window, void*data)
      {
        return NPN_GetURLNotify( pi->npp, url, window, data );
      }
    </pre></code>
    </li>
  * <li>to be able to use the QNPInstance::getURLNotify function on linux, replace
    the file npunix.c with <a href="http://www.damtp.cam.ac.uk/user/kp229/ndvi/ndvi/ndvi/npunix.c">this one</a>
    (see <a href="http://www.damtp.cam.ac.uk/user/kp229/ndvi/netscape_x11.html">Plugin programming for Netscape on Unix</a>,
     University of Cambridge, DAMTP)</li>
  * </ol>
  *
  * <h3>Installing The Plugin</h3>
  *
  * <h3>Embedding The Plugin</h3>
  *
  * <h3>Testpages</h3>
  * scripted pages (all browsers)<br>
  * <ul>
  * <li><a href="../../../testpage/invalid img url.html">#</a> invalid img url</li>
  * <li><a href="../../../testpage/4 texts.html">#</a> 4 texts</li>
  * </ul>
  * <br>
  * special tests:
  * <ul>
  * <li><a href="../../../testpage/embed-src-tag.html">#</a> embed tag with src param</li>
  * <li><a href="../../../testpage/embed-src+img-tag.html">#</a> embed tag with src and img param</li>
  * <li><a href="../../../testpage/embed-src+img(invalid)-tag.html">#</a> embed tag with src and (invalid) img param</li>
  * <li><a href="../../../testpage/embed-src-tag0.html">#</a> embed tag with src param without url</li>
  * <li><a href="../../../testpage/embed-img-tag0.html">#</a> embed tag with img param without url</li>
  * <li><a href="../../../testpage/object-src-tag.html">#</a> object tag with src tag</li>
  * <li><a href="../../../testpage/object-src+img-tag.html">#</a> object tag with src and img tag</li>
  *
  * </ul>
  */

/** The NPSmagePlugin class provides the plugin central factory.
  *
  * @ingroup NPSmagePlugin
  *
  */

#ifndef NP_SMAGE_PLUGIN_H
#define NP_SMAGE_PLUGIN_H

#include "qnp.h"

class NPSmagePlugin : public QNPlugin
{
  public:
    /** Returns a new NPSmagePluginInstance.
      */
    QNPInstance *newInstance();

    /** Return the MIME description of the data formats supported by the plugin.
      * The MIME-type is application/smage
      * The supported images are *.bmp, uncompressed *.jpeg, *.png.
      */
    const char *getMIMEDescription() const;

    /** Returns the plain-text name of the plugin.
      */
    const char *getPluginNameString() const;

    /** Returns the plain-text description of the plugin.
      */
    const char *getPluginDescriptionString() const;
};

#endif //NP_SMAGE_PLUGIN_H
