/** The NPSmagePluginInstance class provides a QObject that is a Web-browser plugin.
  *
  * @ingroup NPSmagePlugin
  *
  */
#ifndef NP_SMAGE_PLUGIN_INSTANCE_H
#define NP_SMAGE_PLUGIN_INSTANCE_H

// project header
#include "NPSmagePluginWidget.h"

// qt header
#include "qnp.h"
#include <qimage.h>

class NPSmagePluginInstance : public QNPInstance
{
  Q_OBJECT

  public:
    /** Constructor.
      */
    NPSmagePluginInstance();
  
    /** Destructor.
      */
    ~NPSmagePluginInstance();
    
    /** Creates a new NPSmagePluginWidget for this plugin.
      */
    QNPWidget* newWindow();
    
    /**
      */
    bool newStreamCreated( QNPStream *stream, StreamMode& smode );

    /** Called when a stream is finished.
      * Creates a QImage _img from the received _data. Updates the layout.
      */
    void streamDestroyed( QNPStream *stream );

    /** Called when incoming data is available for processing by the instance. 
      * Adds the incoming data to the <emph>buffer</emph>.
      */
    int write( QNPStream* stream, int offset, int len, void *buffer );

    /** Returns the minimum amount of data the instance is willing to receive from the given stream.
      * Returns 64000.
      */
    int writeReady( QNPStream* );

    /** Called whenever a url is notified after a call to NPN_GetURLNotify with notifyData.
      * The reason is given in r.
      */
    void notifyURL( const char * url, Reason r, void * notifyData );

    /**
      */
    void visitLinkLocation();

    /**
      */
    bool hasRefURL();

    /**
      */
    const char *getAbsRefURL();

  private:
    /**
      */
    QImage _img;

    /**
      */
    int _width;

    /**
      */
    int _height;

    /**
      * @see write()
      */
    QByteArray _data;

    /**
      */
    const char *_imgUrl;
    
    /**
      */
    bool _imgStreamCreated;

    /**
      */
    const char *_refUrl;

    /**
      */
    const char *_target;

    /**
      * @see newWindow()
      */
    NPSmagePluginWidget *_npspw;
};

#endif // NP_SMAGE_PLUGIN_INSTANCE_H
