// project header
#include "SmageGenerator.h"

// qt header
#include <qpainter.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

TheS_ImgDataItem::TheS_ImgDataItem()
{
  imgSnip = NULL;
}

TheS_DataItem::TheS_DataItem()
{
  textVec.clear();
  pixVec.clear();
  formTextVec.clear();
  dictItem = NULL;
}

TheS_Data::TheS_Data()
{  
  disabled = FALSE;
  polygonVec.clear();
}


SmageGenerator::SmageGenerator()
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::SmageGenerator()\n");
#endif

  _progressDlg = NULL;
}

SmageGenerator::~SmageGenerator()
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::~SmageGenerator()\n");
#endif
}

void SmageGenerator::initProgressDlg(QWidget *parent, const int &totalSteps)
{
  if (_progressDlg != NULL)
    delete _progressDlg;

  _progressDlg = new QProgressDialog(QString::null, QString::null, totalSteps+2, parent, QString::null, TRUE);
  _progressDlg->setAutoReset(TRUE);
  _progressDlg->setAutoClose(TRUE);
  _progressDlg->setProgress(totalSteps);
}

bool SmageGenerator::isCodedImg(const QImage &img)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::isCodedImg()\n");
#endif

  SmageImgLSBsDecoder imgDecoder(img);
  return imgDecoder.isInformationEncoded();
}


void SmageGenerator::eraseDataFromImage(const QImage &img)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::eraseDataFromImage()\n");
#endif

  SmageImgLSBsDecoder imgDecoder(img);
  imgDecoder.eraseDataFromImage(TRUE);
}


void SmageGenerator::eraseDataPartFromImage(const QImage &img, const QImage &idBuffer, const int &colorIndex)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::eraseDataPartFromImage()\n");
#endif

  SmageIdColorStruct colorStruct;

  if (SmageIdBufferGenerator::isEven(colorIndex))
    {
      if (idBuffer.numColors() <= (colorIndex+1))
	return;

      colorStruct.fillColorIndex = colorIndex;
      colorStruct.outlineColorIndex = colorIndex + 1;
    }
  else
    {
      colorStruct.fillColorIndex = colorIndex - 1;
      colorStruct.outlineColorIndex = colorIndex;
    }

  vector<SmageIdColorStruct> colorsVec;
  colorsVec.push_back(colorStruct);

  SmageImgLSBsDecoder imgDecoder(img);
  imgDecoder.eraseDataFromImage(FALSE, idBuffer, colorsVec);
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("x___eraseDataPartFromImage()\n");
#endif
}


bool SmageGenerator::buildImg(const QImage &img, QImage &resultImg, const map<int, TheS_Data> *dataMap, 
			      const bool &resetImg, const int &startId)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::buildImg()\n");
#endif

  QPixmap idBuffer(img.size());
  idBuffer.fill(QColor(COL_BACKGROUND));

  QPainter painter(&idBuffer);
  vector<SmageDataObjs> dataObjsVec;

  map<int, TheS_Data>::const_iterator dataMapIter;

  if (_progressDlg != NULL)
    {
      _progressDlg->setProgress(1);
      _progressDlg->show();
    }
  
  for (dataMapIter=dataMap->begin(); dataMapIter!=dataMap->end(); dataMapIter++)
    {
      if (dataMapIter->first > 255) break;

      QColor fillColor(dataMapIter->first, 255, 0);
      QColor outlineColor(255, dataMapIter->first, 0);

      painter.setBrush(fillColor);
      painter.setPen(outlineColor);

      for (unsigned int i=0; i<dataMapIter->second.polygonVec.size(); i++)
	painter.drawPolygon(dataMapIter->second.polygonVec[i]);

      SmageDataObjs dataObjs;
      
      // Plain text
      vector<SmageTextHolderWidget *>::const_iterator textIter;
      for (textIter=dataMapIter->second.data.textVec.begin(); 
	   textIter!=dataMapIter->second.data.textVec.end(); textIter++)
	{
	  SmageData data;
	  data.dataType = SMAGE_TEXT;
	  convertToVec((*textIter)->getText(), (*textIter)->getText().length(), data.byteStream);
	  data.idColor[0]=fillColor.red(); data.idColor[1]=fillColor.green(); 
	  data.idColor[2]=fillColor.blue();

	  dataObjs.data.push_back(data);
	}

      // Image snips
      vector<TheS_ImgDataItem>::const_iterator pixIter;
      for (pixIter=dataMapIter->second.data.pixVec.begin(); 
	   pixIter!=dataMapIter->second.data.pixVec.end(); pixIter++)
	{
	  SmageData data;
          data.dataType = SMAGE_IMG;
          data.idColor[0]=fillColor.red(); data.idColor[1]=fillColor.green(); 
          data.idColor[2]=fillColor.blue();

          vector<SmageImageObjs> imgObjects;
          if (getPreparedImgData(fillColor, outlineColor, pixIter, imgObjects))
          {
            if (!imgObjects.empty())
            {
              data.byteStream = imgObjects[0].tableIndices;
        
              dataObjs.data.push_back(data);
              dataObjs.colorTable = imgObjects[0].colorTable;
            }
          }

	}
      
      // Formatted texts
      vector<TheS_TextDataItem>::const_iterator formTextIter;
      for (formTextIter=dataMapIter->second.data.formTextVec.begin(); 
	   formTextIter!=dataMapIter->second.data.formTextVec.end(); formTextIter++)
	{
	  SmageData data;
          data.dataType = SMAGE_FORM_TEXT;
          data.byteStream = formTextIter->shapedTextString;
          data.idColor[0]=fillColor.red(); data.idColor[1]=fillColor.green(); 
          data.idColor[2]=fillColor.blue();

          dataObjs.data.push_back(data);

	}

      // Dictionary entries
      if (dataMapIter->second.data.dictItem != NULL)
	{
	  SmageData data;
          data.dataType = SMAGE_DICT;
          data.idColor[0]=fillColor.red(); data.idColor[1]=fillColor.green(); 
          data.idColor[2]=fillColor.blue();

	  if (getDictEntry(dataMapIter->second.data.dictItem, data.byteStream))
	    dataObjs.data.push_back(data);
	}

      if (!dataObjs.data.empty())
	{
	  dataObjs.codePattern = SMAGE_AUTO;
	  dataObjsVec.push_back(dataObjs);
	}
    }
  
  QImage tmpIdBuffer = idBuffer.convertToImage();

  SmageIdBufferStruct idBufferData;
  idBufferData.idBuffer = tmpIdBuffer;
  idBufferData.backgroundColor = QColor(COL_BACKGROUND);
  idBufferData.bufferType = SMAGE_OUTLINED_REGIONS;

  resultImg = img.copy();
  
  bool returnValue = TRUE;

  SmageImgLSBsEncoder *imgCoder = new SmageImgLSBsEncoder(resultImg);
  imgCoder->setProgressCB(this, showProgressCB);
  if (!imgCoder->encodeData(idBufferData, dataObjsVec, resetImg, startId))
    returnValue = FALSE;

  delete imgCoder;

  if (_progressDlg != NULL)
    _progressDlg->setProgress(_progressDlg->totalSteps());

  return returnValue;
}


bool SmageGenerator::getPreparedImgData(const QColor &fillColor, const QColor &outlineColor, 
					const std::vector<TheS_ImgDataItem>::const_iterator &imgDataItem, 
					vector<SmageImageObjs> &imgObjects)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageGenerator::getPreparedImgData()\n");
#endif

  if (imgDataItem->image.isNull())
  {
#ifdef THE_S_PRINT_DEBUG
    _TheSmageLogFile.lprintf("*** Warning (SmageGenerator::getPreparedImgData): Pixmap is empty ***\n");
#endif
    return FALSE;
  }

  // (1) Generate ID-Buffer according to "imgDataItem", filled with "idColor"
  // (2) Process image data (image is converted to an indexed image)

  // (1) Generating ID-Buffer
  QPixmap imgIdBuffer(imgDataItem->image.size());
  imgIdBuffer.fill(QColor(COL_BACKGROUND));

  QPainter painter(&imgIdBuffer);
  painter.setBrush(fillColor);
  painter.setPen(outlineColor);
  painter.drawPolygon(imgDataItem->regionPoints);

  //imgIdBuffer.save("imgIdBuffer.png", "PNG", 100);
  
  // (2) Converting the image
  SmageImageObjs imgObject;
  imgObject.idColor = fillColor;
  imgObjects.push_back(imgObject);

  SmageIdBufferStruct idBufferData;
  idBufferData.idBuffer = imgIdBuffer;
  idBufferData.backgroundColor = QColor(COL_BACKGROUND);
  idBufferData.bufferType = SMAGE_OUTLINED_REGIONS;
  
  SmageImgProcessor imgProcessor;
  imgProcessor.getIndexedImgData(imgDataItem->image, imgObjects, idBufferData);

  if (imgObjects[0].tableIndices.empty() || imgObjects[0].colorTable.empty())
  {
#ifdef THE_S_PRINT_DEBUG
    _TheSmageLogFile.lprintf("*** Warning (SmageGenerator::getPreparedImgData): Color table or indices are empty ***\n");
#endif
    return FALSE;
  }

  return TRUE;
}


void SmageGenerator::convertToVec(const char *src, const uint &size, vector<char> &dest)
{
  const char *srcPointer = src;

  for (unsigned int i=0; i<size; i++)
  {
    dest.push_back(*srcPointer);
    srcPointer++;
  }
}

bool SmageGenerator::getDictEntry(const SmageTextHolderWidget *textHolder, vector<char> &dest)
{
  QString idString, entry;
  int idNum;

  textHolder->getDictEntries(idString, idNum, entry);

  if (idString.isEmpty())
    return FALSE;

  QString result;
  if (idNum >= 0)
    result = QString("<_ID %1>").arg(idNum);

  QString resultPart2 = QString("<%1>%2").arg(idString).arg(entry);
  result += resultPart2;

#ifdef THE_S_PRINT_DEBUG
  cout << result.ascii() << endl;
#endif
  convertToVec(result.ascii(), result.length(), dest);

  return TRUE;
}

void SmageGenerator::showProgressCB(void *userData)
{
  SmageGenerator *generator = (SmageGenerator *) userData;
  if (generator == NULL)
    return;

  generator->_progressDlg->setProgress(generator->_progressDlg->progress() + 1);
}
