#ifndef _SMAGE_GENERATOR_H
#define _SMAGE_GENERATOR_H

// project header
#include "SmageConfig.h"
#include "SmageShapedTextBase.h"
#include "structs/TheS_DictEntries.h"
#include "../theSmage/SmageTextHolderWidget.h"
#include "../theSmage/SmageImgSnip.h"
#include <SmageImgLSBsEncoder.h>
#include <SmageImgLSBsDecoder.h>
#include <SmageImgProcessor.h>

// qt header
#include <qpixmap.h>
#include <qpointarray.h>
#include <qprogressdialog.h>
#include <qregion.h>

// c/c++ header
#include <map>
#include <vector>

struct TheS_ImgDataItem {
  TheS_ImgDataItem();
  QImage image;
  QPointArray regionPoints;
  SmageImgSnip *imgSnip;
};

struct TheS_TextDataItem {
  std::vector<char> shapedTextString;
  SmageShapedTextBase *shapedText;
};

struct TheS_DataItem {
  TheS_DataItem();
  std::vector<SmageTextHolderWidget *> textVec;
  SmageTextHolderWidget *dictItem;
  std::vector<TheS_ImgDataItem> pixVec;
  std::vector<TheS_TextDataItem> formTextVec;
};

struct TheS_Data {
  TheS_Data();  
  bool disabled;
  std::vector<QPointArray> polygonVec;
  TheS_DataItem data;
};

/** Class SmageGenerator.
  *
  * @ingroup SmageApp
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  *
  */
class SmageGenerator
{
  public:

    SmageGenerator();
    ~SmageGenerator();
   
    bool buildImg(const QImage &img, QImage &resultImg, const std::map<int, TheS_Data> *dataMap, 
		  const bool &resetImg, const int &startId);
    bool isCodedImg(const QImage &img);
    void eraseDataFromImage(const QImage &img);
    void eraseDataPartFromImage(const QImage &img, const QImage &idBuffer, const int &colorIndex);

    void initProgressDlg(QWidget *parent, const int &totalSteps);

  protected:
    void convertToVec(const char *src, const uint &size, std::vector<char> &dest);
    bool getDictEntry(const SmageTextHolderWidget *textHolder, std::vector<char> &dest);
    bool getPreparedImgData(const QColor &fillColor, const QColor &outlineColor, 
			    const std::vector<TheS_ImgDataItem>::const_iterator &pixIter, 
			    std::vector<SmageImageObjs> &imgObjects);

    static void showProgressCB(void *userData);

    QProgressDialog *_progressDlg;
};

#endif
