#ifndef _SMAGE_IMG_WIDGET_H

#define _SMAGE_IMG_WIDGET_H



// project header

#include "SmageConfig.h"

#include "SmageGenerator.h"



// qt header

#include <qpainter.h>

#include <qwidget.h>



typedef void TheS_ShowMsgCB(void *, const QString &, const int &);

typedef void TheS_SimpleCB(void *);

typedef void TheS_AllowActionCB(void *, const bool &);



/** Class SmageImgWidget is the base class for a widget that contains the image a user can interact with.

  *

  * @ingroup SmageApp

  * @ingroup NPSmagePlugin

  * @ingroup SmageAXPlugin

  *

  */

class SmageImgWidget : public QWidget

{

  Q_OBJECT



 public:



    typedef void TheS_ForceOpenNewWidgetType(void *, SmageImgWidget *, const QString &);

    SmageImgWidget(QWidget *parent, const char* name, WFlags f);

    ~SmageImgWidget();



    // Setting a number of callback functions

    void setShowMsgCB(void *userData, TheS_ShowMsgCB *f)

      { _showMsgCBFunc = f; _showMsgUserData = userData; }

    void setCloseCB(void *userData, TheS_SimpleCB *f)

      { _closeCBFunc = f; _closeUserData = userData; }

    void setSaveCB(void *userData, TheS_SimpleCB *f)

      { _saveCBFunc = f; _saveUserData = userData; }

    void setForceOpenNewWidgetTypeCB(void *userData, TheS_ForceOpenNewWidgetType *f)

      { _forceOpenNewCBFunc = f; _forceOpenNewUserData = userData; }



    /** Load an image (its location is specified by "fileName") and possibly decode

      * it if encoded with data.

      */

    bool loadImg(const char *fileName);



    /** Save active image.

      */

    void saveImg(const QString &name);



    /**  Verify specified file name.

      * if file has ending, return it

      */

    static QString getFileEnding(const QString &string, bool &validEnding);



    void enableImgDecoding(const bool& enable)

      { _imgShouldBeDecoded = enable; }



    virtual void assignWidgetData(const SmageImgWidget *smageWidget);



    bool isCodedImg() { return _isCodedImg; }



//    QImage getImg();



    QImage* getImgPointer();



    QPoint getImgStartPos() { return _imgStartPos; }



    virtual QString getWidgetType() const { return QString("SMAGE_IMG_WIDGET"); }



    /** Extracts only the file name of a string (no path).

      * e.g., /home/user/img.png => img.png

      */

    static void extractFileName(const QString &entireFileName, QString &fileName);

    QString getFileName() { return _imgFileName; }



 protected:

    /** What should be done when refreshing the widget?

      */

    virtual void paintEvent(QPaintEvent *event);

    virtual void closeEvent(QCloseEvent *event);

    

    /** When resizing the widget:

      * - the image start position has to be modified (to be centered)

      */

    virtual void resizeEvent(QResizeEvent *event);



    virtual void doWhenSuccesImgLoading(){};



    // Set callback functions and class instances

    TheS_ShowMsgCB *_showMsgCBFunc;

    void *_showMsgUserData;

    TheS_SimpleCB *_closeCBFunc;

    void *_closeUserData;

    TheS_SimpleCB *_saveCBFunc;

    void *_saveUserData;

    TheS_ForceOpenNewWidgetType *_forceOpenNewCBFunc;

    void *_forceOpenNewUserData;



    /** Get a position within the image (eg., if position is in widget but not in image)

      */

    void getImgLikePos(const QPoint &curPos, QPoint &newPos);



    /** Get position related to the image position.

      */

    void getLocalImgPos(const QPoint &curPos, QPoint &newPos);



    // The image and its properties

    QImage _img;

    QString _imgFileName;

    QImage *_imgPointer;

    bool _isRgbImg;

    bool _isCodedImg, _allowRegionSelection;



    // Drawing tools 

    QPainter *_painter;

    

    // Some booleans and positions that need to be stored

    bool _imgLoaded, _imgShouldBeDecoded, _imgHasBeenModified, _img32BitsNeeded;

    QPoint _imgStartPos;

    QSize _resizeDiff;



    // Coding images

    SmageGenerator *_smageGenerator;



    // For more efficiency

    bool _sceneChanged;

};



#endif

