// project header
#include "SmagePluginImagePropertiesDialog.h"
#include "SmagePluginViewerWidget.h"

// qt header
#include <qlayout.h>
#include <qpushbutton.h>

SmagePluginImagePropertiesDialog::SmagePluginImagePropertiesDialog( QWidget *parent, const char *name, bool modal, WFlags f ) : QDialog( parent, name, modal, f )
{
  setCaption( tr("Image Properties") );
  QGridLayout *layout = new QGridLayout( this, 13, 2, 10, 1 );
    _titleF = new QLabel( this );
  layout->addMultiCellWidget( _titleF, 0, 0, 0, 1 );

  QLabel *line = new QLabel( this );
  line->setFrameStyle( QFrame::HLine | QFrame::Plain );
  layout->addMultiCellWidget( line, 1, 1, 0, 1 );

  layout->addWidget( new QLabel( tr("File Location:"), this ), 2, 0 );
    _fileLocationF = new QLineEdit( this );
    _fileLocationF->setReadOnly( TRUE );
    _fileLocationF->setFrameStyle( QFrame::Plain );
    _fileLocationF->setBackgroundMode( backgroundMode() );
    _fileLocationF->setMinimumWidth( 200 );
  layout->addWidget( _fileLocationF, 2, 1 );

  layout->addWidget( new QLabel( tr("Width:"), this ), 3, 0 );
    _widthF = new QLabel( this );
  layout->addWidget( _widthF, 3, 1 );

  layout->addWidget( new QLabel( tr("Height:"), this ), 4, 0 );
    _heightF = new QLabel( this );
  layout->addWidget( _heightF, 4, 1 );

  layout->addWidget( new QLabel( tr("File Size:"), this ), 5, 0 );
    _filesizeF = new QLabel( this );
  layout->addWidget( _filesizeF, 5, 1 );

  layout->addWidget( new QLabel( tr("Annotated Areas:"), this ), 6, 0 );
    _annotAreasF = new QLabel( this );
  layout->addWidget( _annotAreasF, 6, 1 );

  layout->addWidget( new QLabel( tr("  Unformatted Texts:"), this ), 7, 0 );
    _unformattedTextsF = new QLabel( this );
  layout->addWidget( _unformattedTextsF, 7, 1 );

  layout->addWidget( new QLabel( tr("  Formatted Texts:"), this ), 8, 0 );
    _formattedTextsF = new QLabel( this );
  layout->addWidget( _formattedTextsF, 8, 1 );

  layout->addWidget( new QLabel( tr("  Translateable Texts:  "), this ), 9, 0 );
    _dictEntriesF = new QLabel( this );
  layout->addWidget( _dictEntriesF, 9, 1 );


  layout->addWidget( new QLabel( tr("  Encoded Images:"), this ), 10, 0 );
    _imagesF = new QLabel( this );
  layout->addWidget( _imagesF, 10, 1 );

  layout->addWidget( new QLabel( " ", this ), 11, 0 );

  QPushButton *cb = new QPushButton( tr("Close"), this );
  connect( cb, SIGNAL( clicked() ), this, SLOT( hide() ) );
  layout->addWidget( cb, 12, 0 );
  cb->setFocus();
}

SmagePluginImagePropertiesDialog::~SmagePluginImagePropertiesDialog()
{
}

void
SmagePluginImagePropertiesDialog::setProperties( SmageStructImageProperties properties )
{
  _titleF->setText( QString(tr("Properties for \"%1\"")).arg(properties._fname) );
  _fileLocationF->setText( QString(properties._flocation) );
  _fileLocationF->home( FALSE );
  _widthF->setText( QString(tr("%1 px (original) / %2 px (displayed)")).arg(properties._imageWidth).arg(properties._widgetWidth) );
  _heightF->setText( QString(tr("%1 px (original) / %2 px (displayed)")).arg(properties._imageHeight).arg(properties._widgetHeight) );

  float s;
  QString fSize;
  if ( properties._size < 0 ) fSize = QString(tr("unknown"));
  else if ( properties._size > 1024 )
  {
    s = properties._size / 1024.0;
    if ( s > 1024 )
    {
      s = s / 1024.0;
      fSize = QString(tr("%1MB (%2 bytes)")).arg(s).arg(properties._size);
    }
    else fSize = QString(tr("%1KB (%2 bytes)")).arg(s).arg(properties._size); 
  }
  else fSize = QString(tr("%1 bytes")).arg(properties._size); 
  _filesizeF->setText( fSize );

  _annotAreasF->setNum( properties._decodeInfo._numOfEncodedAreas );
  _unformattedTextsF->setNum( properties._decodeInfo._numOfEncodedTexts );
  _formattedTextsF->setNum( properties._decodeInfo._numOfEncodedFormattedTexts );
  _dictEntriesF->setNum( properties._decodeInfo._numOfEncodedDictEntries );
  _imagesF->setNum( properties._decodeInfo._numOfEncodedImages );

  show();
  setMaximumSize( this->size() );
}

/*
void
SmagePluginImagePropertiesDialog::closeEvent( QCloseEvent *e )
{
  e->accept();
  hide();
  SmagePluginViewerWidget *spw = (SmagePluginViewerWidget*)parentWidget();
  spw->updateWidget();
}
*/
