/** Provides a dialog to display the properties of an (encoded) image.
  *
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  *
  */
#ifndef SMAGE_PLUGIN_IMAGE_PROPERTIES_DIALOG
#define SMAGE_PLUGIN_IMAGE_PROPERTIES_DIALOG

// project header
#include "structs/SmageStructImageProperties.h"

// qt header
#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>

class SmagePluginImagePropertiesDialog : public QDialog
{
  Q_OBJECT
  
  public:
    /** Constructor.
      */
    SmagePluginImagePropertiesDialog( QWidget *parent = 0, const char *name = 0, bool modal = FALSE, WFlags f = 0 );

    /** Destructor.
      */
    ~SmagePluginImagePropertiesDialog();

    /** Sets the properties that should be displayed by the dialog.
      */
    void setProperties( SmageStructImageProperties properties );

  protected:
    /**
      */
//    void closeEvent( QCloseEvent *e );
  
  private:
    /** Displays a title like "Properties for + filename"
      */
    QLabel *_titleF;

    /** Displays the image file location.
      */
    QLineEdit *_fileLocationF;

    /** Displays the image width.
      * Both, the real and the scaled width, are displayed.
      */
    QLabel *_widthF;

    /** Displays the image height.
      * Both, the real and the scaled height, are displayed.
      */
    QLabel *_heightF;

    /** Displays the file size of the image.
      */
    QLabel *_filesizeF;

    /** Displays the number of annotated areas in the image.
      * This value is only displayed, if the image allows this.
      */
    QLabel *_annotAreasF;

    /** Displays the number of encoded texts.
      * This value is only displayed, if the image allows this.
      */
    QLabel *_unformattedTextsF;
    QLabel *_formattedTextsF;
    QLabel *_dictEntriesF;

    /** Displays the number of encoded images.
      * This value is only displayed, if the image allows this.
      */
    QLabel *_imagesF;
};

#endif // SMAGE_PLUGIN_IMAGE_PROPERTIES_DIALOG
