// project header
#include "SmageDecodeHandler.h"
#include "SmagePluginViewerWidget.h"

// qt header
#include <qapplication.h>
#include <qcursor.h>
#include <qfiledialog.h>

extern std::vector<TheS_Language> __languagesVec;

SmagePluginViewerWidget::SmagePluginViewerWidget( QWidget *parent ) : SmageViewerWidget( parent, 0, 0 )
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmagePluginViewerWidget::SmagePluginViewerWidget()\n");
#endif

  _xScale = _yScale = 1.0;
  _menu = NULL;

  _spipd = new SmagePluginImagePropertiesDialog( this, 0, TRUE );
  _spipd->hide();

  QString t = "Smage Version 0.7\nCopyright 2005 Lothar Schlesier, Henry Sonnet";
  _about = new QMessageBox(tr("Program Info"), t, QMessageBox::Information, 
    QMessageBox::NoButton, QMessageBox::NoButton, QMessageBox::NoButton, this, 0, TRUE, 0 );
  _about->hide();
  _imgStartPos = QPoint( 0, 0 );

  _fromLang = 3;
  _toLang = 0;

  setMouseTracking( TRUE );
}

SmagePluginViewerWidget::~SmagePluginViewerWidget() 
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmagePluginViewerWidget::~SmagePluginViewerWidget()\n");
#endif

  if ( _smageImageProperties._fname != NULL ) free( _smageImageProperties._fname );
  if ( _smageImageProperties._flocation != NULL ) free( _smageImageProperties._flocation );
}

void 
SmagePluginViewerWidget::setScale( QSize widgetSize, QSize imageSize )
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmagePluginViewerWidget::setScale()\n");
#endif

  _xScale = (float)widgetSize.width() / (float)imageSize.width();
  _yScale = (float)widgetSize.height() / (float)imageSize.height();
}

void
SmagePluginViewerWidget::setImage( QImage img, const char *url )
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmagePluginViewerWidget::setImage()\n");
#endif

  QPainter p( this );
  p.drawText( width()/8, 0, width()-width()/4, height(), AlignCenter|WordBreak, tr("Preparing Image") );

  // original image
  _img = img;

  _imgLoaded = TRUE;
  _imgPointer = &_img;
  
  _isCodedImg = _smageGenerator->isCodedImg( _img );
  if ( _isCodedImg )
  {
    // reference image
    _decodeIdBuffer = QImage();
    _smageImageProperties._decodeInfo = SmageDecodeHandler::decode( _img, _decodeIdBuffer, _outlineImg, _decodedDataMap, this );
  }

  // if widget is scaled, rescale the imaged to fit the widget
  std::map<unsigned int, SmageStructDecodedData>::iterator itr;
  if ( ( _xScale < 1.0 - 1.1 / _img.width() || _xScale > 1.0 + 1.1 / _img.width() ) ||
       ( _yScale < 1.0 - 1.1 / _img.height() || _yScale > 1.0 + 1.1 / _img.height() ) )
  {
    _scaledImg =  _img.smoothScale( (int)(_img.width() * _xScale), (int)(_img.height() * _yScale) );
    _scaledOutlineImg =  _outlineImg.smoothScale( (int)(_img.width() * _xScale), (int)(_img.height() * _yScale) );

    // scale encoded images
    for ( itr = _decodedDataMap.begin(); itr  != _decodedDataMap.end(); itr++ )
    {
      QImage img =  itr->second._pixmap.convertToImage();
      img = img.smoothScale( (int)(img.width() * _xScale), (int)(img.height() * _yScale) );
      itr->second._pixmap.convertFromImage( img );

      if ( itr->second._shapedTextVec.size() > 0 )
      {
        std::vector<SmageViewerShapedText *>::iterator bubbleIter;
        for ( bubbleIter = itr->second._shapedTextVec.begin(); bubbleIter != itr->second._shapedTextVec.end(); bubbleIter++ )
        {
          (*bubbleIter)->scaleBubble( _xScale, _yScale );
        }
      }
    }
  }
  else
  {
    _scaledImg = _img;
    _scaledOutlineImg =  _outlineImg;
  }

  setPaletteBackgroundPixmap( QPixmap( _scaledImg ) );

  _imgPointer = &_scaledImg;
  updateWidget();

  _smageImageProperties._imageWidth = _img.width();
  _smageImageProperties._imageHeight = _img.height();
  _smageImageProperties._widgetWidth = _scaledImg.width();
  _smageImageProperties._widgetHeight = _scaledImg.height();

  // Image url
  _smageImageProperties._flocation = (char*)malloc( (strlen(url) + 1) * sizeof(char) );
  strcpy( _smageImageProperties._flocation, url );
  char *b = (char*)malloc( sizeof(char) );
  // Image name
  _smageImageProperties._fname = strtok( (char*)url, "\\/" );
  while ( _smageImageProperties._fname )
  {
    free( b );
    b = (char*)malloc( (strlen(_smageImageProperties._fname) + 1) * sizeof(char) );
    strcpy( b, _smageImageProperties._fname );
    _smageImageProperties._fname = strtok( NULL, "\\/" );
  }
  _smageImageProperties._fname = (char*)malloc( (strlen(b) + 1) * sizeof(char) );
  strcpy( _smageImageProperties._fname, b );
  free( b );
  _imgLoaded = TRUE;

  initMenu();
  if ( hasRefURL() ) setCursor( QCursor(PointingHandCursor) );
}

void
SmagePluginViewerWidget::mousePressEvent( QMouseEvent *e )
{
  // only allow interaction, if image is completly loaded
  if ( !_imgLoaded ) return;
  
  if ( e->button() ==  RightButton )
  {
    // only try to popup the menu, if it was initialized
    // this test is redundant, as it is previously checked, if the image is loaded
    if ( _menu != NULL ) _menu->popup( e->globalPos() );
    // to avoid cases where the image is not repainted
    _sceneChanged = TRUE;
//    repaint( TRUE );
  }
  else visitLinkLocation();
}

void
SmagePluginViewerWidget::mouseMoveEvent( QMouseEvent *e )
{
  _currentMousePos = e->pos();
  _mousePos = QPoint( (int)(_currentMousePos.x() / _xScale), (int)(_currentMousePos.y()  / _yScale) );

  onMouseMove();
}

void
SmagePluginViewerWidget::resizeEvent( QResizeEvent *e )
{
  _imgStartPos = QPoint( 0, 0 );
}

void
SmagePluginViewerWidget::imagePropertiesCB()
{
  _spipd->setProperties( _smageImageProperties );
  _spipd->show();
}

void
SmagePluginViewerWidget::imageOutlinesCB()
{
  bool b = !_menu->isItemChecked( 1 );
  _menu->setItemChecked ( 1, b );
  showOutlines( b );
}

void
SmagePluginViewerWidget::showOutlines( const bool &show ) 
{ 
  _showOutlines = show; 
  if ( show )
    _imgPointer = &_scaledOutlineImg;
  else
    _imgPointer = &_scaledImg;
  setPaletteBackgroundPixmap( QPixmap( *_imgPointer ) );

  std::map<unsigned int, SmageStructDecodedData>::iterator itr;
  for ( itr = _decodedDataMap.begin(); itr  != _decodedDataMap.end(); itr++ )
  {
    if ( itr->second._shapedTextVec.size() > 0 )
    {
      std::vector<SmageViewerShapedText *>::iterator bubbleIter;
      for ( bubbleIter = itr->second._shapedTextVec.begin(); bubbleIter != itr->second._shapedTextVec.end(); bubbleIter++ )
      {
        if (show) (*bubbleIter)->setPaletteBackgroundPixmap( (*bubbleIter)->_outlined );
        else (*bubbleIter)->setPaletteBackgroundPixmap( (*bubbleIter)->_normal );
      }
    }
  }  
  updateWidget();
}

void
SmagePluginViewerWidget::infoCB()
{
  _about->show();
  _about->setMaximumSize( _about->size() );
}

void
SmagePluginViewerWidget::setFileSize( long int size )
{
  _smageImageProperties._size = size;
}

void
SmagePluginViewerWidget::initMenu()
{
  if ( _menu ==  NULL )
  {
    int menuID;
    _menu = new QPopupMenu( this );
    _menu->insertItem( tr("Image Properties"), this, SLOT( imagePropertiesCB() ) );
    menuID = _menu->insertItem( tr("Open Link Location"), this, SLOT( visitLinkLocation() ) );
    if ( !hasRefURL() )  _menu->setItemEnabled( menuID, FALSE );
    _menu->insertItem( tr("Save Image As"), this, SLOT( saveImageCB() ) );
    _menu->insertSeparator();
    if ( _isCodedImg )
    {
      _menu->setCheckable( TRUE );
      _menu->insertItem( tr("Show Image Outlines"), this, SLOT( imageOutlinesCB() ), 0, 1 );
      
      if ( _smageImageProperties._decodeInfo._numOfEncodedDictEntries > 0 )
      {
        SmageXmlLanguageFileParser languageFile;
        _dictMenu = SmageDecodeHandler::createDictMenu( &__languagesVec, this, _translFromGroup, _translToGroup );
        _menu->insertItem( tr("Image Dictionary"), _dictMenu );
      }
      else
      {
        menuID = _menu->insertItem( tr("Image Dictionary") );
        _menu->setItemEnabled( menuID, FALSE );
      }
    }
    _menu->insertSeparator();
    _menu->insertItem( tr("About"), this, SLOT( infoCB() ) );
  }
}

void
SmagePluginViewerWidget::saveImageCB()
{
  if ( !_imgLoaded ) return;
  QString name = QFileDialog::getSaveFileName( _smageImageProperties._fname, /*"Bitmap (*.bmp);;Portable Network Graphic (*.png)"*/ "*.*", this, "save file dialog" );

  if ( !name.isEmpty() )
  {
    // get file extension
    char *b1;
    char *b2 = (char*)malloc( sizeof(char) );
    b1 = strtok( (char*)name.ascii(), "." );
    while ( b1 )
    {
      free( b2 );
      b2 = (char*)malloc( (strlen( b1 ) + 1) * sizeof(char) );
      strcpy( b2, b1 );
      b1 = strtok( NULL, "." );
    }
    // convert file extension to lower case
    int i = 0;
    while ( b2[i] )
    {
      if ( isupper( b2[i] ) ) b2[i] = tolower( b2[i] );
      i++;
    }
    if ( !strcmp( b2, "bmp" ) ) _img.save( name, "BMP" );
    else if ( !strcmp( b2, "png" ) ) _img.save( name, "PNG" );
    free( b2 );
  }
  updateWidget();
}

void
SmagePluginViewerWidget::showCodedDictItem()
{
  bool b = !_dictMenu->isItemChecked( 3 );
  _dictMenu->setItemChecked ( 3, b );
  updateDictText(_fromLang, _toLang, _dictMenu->isItemChecked(4), b);
}

void
SmagePluginViewerWidget::showSecEntries()
{
  bool b = !_dictMenu->isItemChecked( 4 );
  _dictMenu->setItemChecked ( 4, b );
  updateDictText(_fromLang, _toLang, b, _dictMenu->isItemChecked(3));
}

void
SmagePluginViewerWidget::translFrom(int from)
{
  _fromLang = from;
  updateDictText(_fromLang, _toLang, _dictMenu->isItemChecked(4), _dictMenu->isItemChecked(3));
}

void
SmagePluginViewerWidget::translTo(int to)
{
  _toLang = to;
  updateDictText(_fromLang, _toLang, _dictMenu->isItemChecked(4), _dictMenu->isItemChecked(3));
}
