/** The base widget for viewing operations and user interaction for browser plugins.
  *
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  */

#ifndef SMAGE_PLUGIN_VIEWER_WIDGET
#define SMAGE_PLUGIN_VIEWER_WIDGET

// project header
#include "SmagePluginImagePropertiesDialog.h"
#include "SmageViewerWidget.h"
#include "structs/SmageStructImageProperties.h"

// qt header
#include <qbuttongroup.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>

class SmagePluginViewerWidget : public SmageViewerWidget
{
  Q_OBJECT
  
  public:
    /** Constructor.
      */
    SmagePluginViewerWidget( QWidget *parent );
    
    /** Destructor.
      */
    ~SmagePluginViewerWidget();
    
    /** Calculates scaling factors.
      * @see _xScale
      * @see _yScale
      */
    void setScale( QSize widgetSize, QSize imageSize );
    
    /** Sets and prepares the image.
      *  
      */
    void setImage( QImage img, const char *url );
    
    /**
      */
    void setFileSize( long int size );
    
    /** 
      */
    virtual void showOutlines( const bool &show );
    
    /** Returns a class specific identifier string.
      */
    virtual QString getWidgetType() const { return QString( "SMAGE_PLUGIN_VIEWER_WIDGET" ); }

  public slots:
    
    /** Switches the image that is displayed between _scaledImg and _scaledOutlineImg.
      */
    void imageOutlinesCB();
    
    /** Opens a dialog with the image properties.
      * @see _smageImageProperties
      */
    void imagePropertiesCB();
    
    /** Opens an info dialog.
      */
    void infoCB();
    
    /** Opens a file dialog, that allows the user to save the image.
      */
    void saveImageCB();

    void showCodedDictItem();
    void showSecEntries();
    void translFrom(int);
    void translTo(int);
   
  protected:
    /** Event handler for mouse press events.
      */
    virtual void mousePressEvent( QMouseEvent *e );
    
    /** Event handler for mouse move events.
      */
    virtual void mouseMoveEvent( QMouseEvent *e );
    
    /** Event handler for resize events.
      */
    virtual void resizeEvent( QResizeEvent *e );
    
    /** Returns TRUE if the plugin has a refence url; otherwise returns FALSE.
      */
    virtual bool hasRefURL() = 0;
    
    /** A context menu that pops up than a user clicks with the right mouse button on the widget.
      * The following fuctions are accessible throught this menu:
      * imagePropertiesCB(), imageOutlinesCB(), visitLinkLocation(), saveImageCB(), and infoCB().
      */
    QPopupMenu *_menu;
    
    QPopupMenu *_dictMenu;
    QButtonGroup *_translFromGroup, *_translToGroup;
    
    /** Scaled version of the original image.
      * If no scaling is applied it is the same as the original image.
      * @see setImage()
      */
    QImage _scaledImg;
    
    /** (Scaled) image with outlines around annotated areas.
      */
    QImage _scaledOutlineImg;
    
    /** Scaling factor between image and plugin in x direction.
      * @see setScale()
      */
    float _xScale;
    
    /** Scaling factor between image and plugin in y direction.
      * @see setScale()
      */
    float _yScale;

    int _fromLang, _toLang;

  protected slots:

    /** 
      * @see hasRefURL()
      */
    virtual void visitLinkLocation() = 0;

  private:
    /**
      */
    SmageStructImageProperties _smageImageProperties;
    
    /**
      */
    SmagePluginImagePropertiesDialog *_spipd;

    QMessageBox *_about;  
    /** Initializing of _menu.
      */
    void initMenu();
};

#endif // SMAGE_PLUGIN_VIEWER_WIDGET
