// project header
#include "SmageTextWidget.h"
#include "SmageTextWidgetButtonAnchored.xpm"
#include "SmageTextWidgetButtonNotAnchored.xpm"
#include "SmageViewerWidget.h"

// qt header
#include <qbitmap.h>
#include <qcursor.h>
#include <qpointarray.h>

#define S_ABD 15    // SmageTWAnchorButton diameter
#define S_ABDh 8    // SmageTWAnchorButton diameter
#define S_MAX_W 125 //100 // maximum widget width
#define S_MAX_H 70 //55  // maximum widget heigth

SmageTextWidget::SmageTextWidget( QWidget *parent, const char *name, WFlags f ) : QWidget( parent, name, f )
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageTextWidget::SmageTextWidget()\n");
#endif
  setMouseTracking( TRUE );
  setPaletteBackgroundColor( QColor(192, 192, 192) );
  _anchored = FALSE;
  _style = new SmageTWScrollStyle();
  setCursor( QCursor(ArrowCursor) );

  _layout = new QGridLayout( this, 3, 3 );
    _text = new QTextEdit( this );
    _text->setReadOnly( TRUE );
    _text->setVScrollBarMode( QScrollView::AlwaysOff );
    _text->setHScrollBarMode( QScrollView::AlwaysOff );
    //_text->setFrameStyle( QFrame::Box );
    //_text->setFrameShadow( QFrame::Plain );
    //_text->setPaletteForegroundColor( QColor(192, 192, 192) );
    _text->setLineWidth( 2 );
///    _text->setTextFormat( Qt::RichText);
  _layout->addWidget( _text, 0, 2 );

  _hasHBar = TRUE;
    _hBar = new QScrollBar( QScrollBar::Horizontal, this );
    _hBar->setStyle( _style );
    _hBar->setMaximumHeight( S_ABD / 2 );
    QScrollBar *hBar = _text->horizontalScrollBar();
    // my (visible) _hBar connects to the _text's (invisible) hBar. so if the user
    // interacts with the _hBar the _text is accordingly scrolled
    connect( _hBar, SIGNAL(  valueChanged ( int ) ), hBar, SLOT( setValue(int) ) );
    // on the other hand, if the text is scrolled using the mousewheel or something,
    // the visible _hBar should be updated accordingly
    connect( hBar, SIGNAL(  valueChanged ( int ) ), _hBar, SLOT( setValue(int) ) );
  _layout->addWidget( _hBar, 1, 2 );

  _hasVBar = TRUE;
    _vBar = new QScrollBar( QScrollBar::Vertical, this );
    _vBar->setStyle( _style );
    _vBar->setMaximumWidth( S_ABD / 2 );
    QScrollBar *vBar = _text->verticalScrollBar();
    connect( _vBar, SIGNAL(  valueChanged ( int ) ), vBar, SLOT( setValue(int) ) );
    connect( vBar, SIGNAL(  valueChanged ( int ) ), _vBar, SLOT( setValue(int) ) );
    _vBar->setMinValue( vBar->minValue() );
  _layout->addWidget( _vBar, 0, 1 );

    _anchor = new SmageTWAnchorButton( this );
    _anchor->setMaximumSize( S_ABD, S_ABD );
    _anchor->setToggleButton( TRUE );
  _layout->addMultiCellWidget( _anchor, 1, 2, 0, 1 );
  connect( _anchor, SIGNAL( toggled(bool)), this, SLOT(anchored(bool)) );

  _dictEntries.first = QString::null;
  _dictEntries.second.clear();
  _codedDictTerm = QString::null;
}

SmageTextWidget::~SmageTextWidget()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageTextWidget::~SmageTextWidget()\n");
#endif
  delete _style;
}

bool
SmageTextWidget::hasDictText()
{
  if (_codedDictTerm != QString::null) return TRUE;
  else return FALSE;
}

QSize 
SmageTextWidget::calcWidgetSize( const QString &text )
{
  QPixmap pm(S_MAX_W-S_ABD+1-12, S_MAX_H-S_ABD+1-12);

  QPainter p( &pm );
  p.setFont( _text->font() );
  QRect r(0,0,S_MAX_W-S_ABD+1-12, S_MAX_H-S_ABD+1-12);
  QRect br = p.boundingRect(r, Qt::WordBreak | Qt::AlignLeft, text);
  pm.fill( QColor(255, 255, 255));
  p.drawText(br, Qt::WordBreak | Qt::AlignLeft, text, -1);
#ifdef THE_S_PRINT_DEBUG
  p.drawRect( br );
  pm.save("SmageTextWidget-calcWidgetSize-test.png", "PNG" );
#endif

  int w,h;
  if (br.width()+12 > S_MAX_W-S_ABD)
  {
    w = S_MAX_W-S_ABD;
    _hasHBar = TRUE;
    _layout->addWidget( _hBar, 1, 2 );
    _hBar->show();
  }
  else
  {
    if ( _hasHBar ) _layout->remove( _hBar );
    _hBar->hide();
    w = br.width()+12;
    _hasHBar = FALSE;
  }
  if (br.height()+8 > S_MAX_H-S_ABD)
  {
    h = S_MAX_H-S_ABD;
    _hasVBar = TRUE;
    _layout->addWidget( _vBar, 0, 1 );
    _vBar->show();
  }
  else
  {
    if ( _hasVBar ) _layout->remove( _vBar );
    _vBar->hide();
    h = br.height()+8;
    _hasVBar = FALSE;
  }

  _text->setMaximumSize( w, h );
  _text->setMinimumSize( w, h );
  setMaximumSize( w+S_ABD, h+S_ABD );
  setMinimumSize( w+S_ABD, h+S_ABD );
  return QSize( w+S_ABD, h+S_ABD ); 
}

void
SmageTextWidget::calcWidgetMask( QSize size )
{
  QPointArray ll, lr, ur, ul;
  int w=size.width();
  int h=size.height();

  QPen pen( Qt::color1, 1 );
  QBrush brush( Qt::color1 );

  _ll = _lr = _ur = _ul = QBitmap( w, h, TRUE );
  QPainter pll( &_ll );
  pll.setPen(pen);
  pll.setBrush(brush);
  pll.drawRect(S_ABD,0,w-S_ABD,h-S_ABD);
  pll.drawEllipse(0,h-S_ABD,S_ABD,S_ABD);

  QPainter plr( &_lr );
  plr.setPen(pen);
  plr.setBrush(brush);
  plr.drawRect(0,0,w-S_ABD,h-S_ABD);
  plr.drawEllipse(w-S_ABD,h-S_ABD,S_ABD,S_ABD);

  QPainter pur( &_ur );
  pur.setPen(pen);
  pur.setBrush(brush);
  pur.drawRect(0,S_ABD,w-S_ABD,h-S_ABD);
  pur.drawEllipse(w-S_ABD,0,S_ABD,S_ABD);

  QPainter pul( &_ul );
  pul.setPen(pen);
  pul.setBrush(brush);
  pul.drawRect(S_ABD,S_ABD,w-S_ABD,h-S_ABD);
  pul.drawEllipse(0,0,S_ABD,S_ABD);

  if ( _hasVBar )
  {
    pll.drawRect(S_ABDh,0,S_ABDh,h-S_ABDh);
    plr.drawRect(w-S_ABD,0,S_ABDh-1,h-S_ABDh);
    pur.drawRect(w-S_ABD,S_ABDh,S_ABDh-1,h-S_ABDh);
    pul.drawRect(S_ABDh,S_ABDh,S_ABDh,h-S_ABDh);
  }
  if ( _hasHBar )
  {
    pll.drawRect(S_ABDh,h-S_ABD,w-S_ABDh+1,S_ABDh-1);
    plr.drawRect(0,h-S_ABD,w-S_ABDh,S_ABDh-1);
    pur.drawRect(0,S_ABDh,w-S_ABDh,S_ABDh);
    pul.drawRect(S_ABDh,S_ABDh,w-S_ABDh+1,S_ABDh);
  }
  pll.setPen( QPen(Qt::color1, 7));
  pll.drawLine(S_ABDh, h-S_ABDh,S_ABD+S_ABDh,h-S_ABD-S_ABDh);
  plr.setPen( QPen(Qt::color1, 7));
  plr.drawLine(w-S_ABD-S_ABDh,h-S_ABD-S_ABDh,w-S_ABDh,h-S_ABDh);

  pur.setPen( QPen(Qt::color1, 7));
  pur.drawLine(w-S_ABD-S_ABDh,S_ABD+S_ABDh,w-S_ABDh,S_ABDh);
  pul.setPen( QPen(Qt::color1, 7));
  pul.drawLine(S_ABDh, S_ABDh,S_ABD+S_ABDh,S_ABD+S_ABDh);


  setMask( _ll );

  _upperBound = h - S_ABDh;
  _rightBound = parentWidget()->width() - ( w - S_ABDh );

  _xOffset = S_ABDh;
  _yOffset = h - S_ABDh;
  _left = _bottom = TRUE;
}

void
SmageTextWidget::setText( const QString &text )
{
  QSize s = calcWidgetSize( text );
  calcWidgetMask( s );
  _text->setText( text );
}

void
SmageTextWidget::anchored( bool state )
{
  _anchored = state;
  if ( _anchored )
  {
    _anchor->releaseMouse();
    // thus mouse wheel event will scroll the text (if there is enought text to scroll)
    _text->setActiveWindow();
  }
  else _anchor->grabMouse();
  _anchor->repaint();
}

void
SmageTextWidget::updateBounds()
{
  // in the app widget size can change. that means bounds and offsets needs to be updated
  _upperBound = height() - S_ABDh;
  _rightBound = parentWidget()->width() - ( width() - S_ABDh );
}

void
SmageTextWidget::paintEvent( QPaintEvent* )
{
  // proper initialization of the scrollbars is only possible, if the widget is visible
  if ( _hasHBar )
  {
    QScrollBar *hBar = _text->horizontalScrollBar();
    _hBar->setMinValue( hBar->minValue() );
    _hBar->setMaxValue( hBar->maxValue() );
    _hBar->setLineStep( hBar->lineStep() );
    _hBar->setPageStep( hBar->pageStep() );
  }
  if ( _hasVBar )
  {
    QScrollBar *vBar = _text->verticalScrollBar();
    _vBar->setMinValue( vBar->minValue() );
    _vBar->setMaxValue( vBar->maxValue() );
    _vBar->setLineStep( vBar->lineStep() );
    _vBar->setPageStep( vBar->pageStep() );
  }
}

void
SmageTextWidget::reshape( QPoint mousePos, bool anyway )
{
  bool left;
  bool bottom;

  if ( mousePos.x() > _rightBound ) left = FALSE;
  else left = TRUE;
  if ( mousePos.y() < _upperBound ) bottom = FALSE;
  else bottom = TRUE;

  if ( !anyway && left == _left && bottom == _bottom ) return;

  setUpdatesEnabled( FALSE );

  _layout->remove( _text );
  if ( _hasHBar ) _layout->remove( _hBar );
  if ( _hasVBar ) _layout->remove( _vBar );
  _layout->remove( _anchor );
 
  if ( left )
  {
    if ( bottom )
    {
      _layout->addWidget( _text, 0, 2 );
      if ( _hasHBar ) _layout->addWidget( _hBar, 1, 2 );
      if ( _hasVBar ) _layout->addWidget( _vBar, 0, 1 );
      _layout->addMultiCellWidget( _anchor, 1, 2, 0, 1 );
      setMask( _ll );
      _xOffset = S_ABDh;
      _yOffset = height()-S_ABDh;
    }
    else
    {
      _layout->addWidget( _text, 2, 2 );
      if ( _hasHBar ) _layout->addWidget( _hBar, 1, 2 );
      if ( _hasVBar ) _layout->addWidget( _vBar, 2, 1 );
      _layout->addMultiCellWidget( _anchor, 0, 1, 0, 1 );
      setMask( _ul );
      _xOffset = S_ABDh;
      _yOffset = S_ABDh;
    }
  }
  else 
  {
    if ( bottom )
    {
      _layout->addWidget( _text, 0, 0 );
      if ( _hasHBar ) _layout->addWidget( _hBar, 1, 0 );
      if ( _hasVBar ) _layout->addWidget( _vBar, 0, 1 );
      _layout->addMultiCellWidget( _anchor, 1, 2, 1, 2 );
      setMask( _lr );
      _xOffset = width()-S_ABDh;
      _yOffset = height()-S_ABDh;
    }
    else
    {
      _layout->addWidget( _text, 2, 0 );
      if ( _hasHBar ) _layout->addWidget( _hBar, 1, 0 );
      if ( _hasVBar ) _layout->addWidget( _vBar, 2, 1 );
      _layout->addMultiCellWidget( _anchor, 0, 1, 1, 2 );
      setMask( _ur );
      _xOffset = width()-S_ABDh;
      _yOffset = S_ABDh;
    }
  }
  setUpdatesEnabled( TRUE );
  _left = left;
  _bottom = bottom;
}

void SmageTextWidget::setDictText(const int &from, const int &to, 
				  const bool &showAllEntries, const bool &showCodedTerm)
{
  bool sCodedTerm = showCodedTerm;
  if (_dictEntries.first.isEmpty() || _dictEntries.second.empty())
  {
    setText("error");  
    return;
  }
  QString textToShow, dictText, dictFrom, dictTo;


  DictIdEntriesMap entriesMap = _dictEntries.second;
  DictIdEntriesMap::const_iterator mapIter;

  mapIter = entriesMap.find(from);
  if (mapIter != entriesMap.end())
    dictFrom = mapIter->second.primeWord;
  mapIter = entriesMap.find(to);
  if (mapIter != entriesMap.end())
    dictTo = mapIter->second.primeWord;

  if (!dictFrom.isEmpty() && !dictTo.isEmpty())
  {
    // http://unicode.e-workers.de/unicode3.php
    ushort uc = 8226;
    QString bc;
    bc = bc.setUnicodeCodes( &uc, 1 );
    dictText = QString( bc+" %1\n" + bc+ " %2").arg(dictFrom).arg(dictTo);
    textToShow += dictText;
  }
  if ( textToShow.isEmpty())
  {
    sCodedTerm = FALSE;
    textToShow += QString("[%1]").arg(_codedDictTerm);
  }
  if (showAllEntries )
  {
    int vs = mapIter->second.secWords.size();
    if (vs > 0)
    {
      --vs;
      textToShow += QString(" {");
      int i;
      for (i=0; i<vs;i++) textToShow += QString(mapIter->second.secWords[i] + ", ");
      textToShow += QString(mapIter->second.secWords[i] + "}");
    }
  }
  if (sCodedTerm && !_codedDictTerm.isEmpty())
    textToShow += QString("\n\n(%1)").arg(_codedDictTerm);

  // need this, to keep the widget at its position
  QPoint oldPos = pos();
  QSize oldSize = size();
  int xOffset = _xOffset;
  int yOffset = _yOffset;
  bool bottom = _bottom;
  bool left = _left;

  hide();
  // set the text
  setText(textToShow);
  // keep widget at its position
  if ( _anchored )
  {
    show();
    reshape( QPoint(oldPos.x()+xOffset,oldPos.y()+yOffset), TRUE );
    int xn = oldPos.x();
    int yn = oldPos.y();
    if (_bottom == bottom)
    {
      if (_bottom) yn += (oldSize.height()-height());
    }
    else
    {
      if (bottom) yn += (oldSize.height()-S_ABD);
      else yn -= (height()-S_ABD);
    }

    if (_left == left)
    {
      if (!_left) xn += (oldSize.width()-width());
    }
    else
    {
      if (!left) xn += (oldSize.width()-S_ABD);
      else xn -= (width()-S_ABD);
    }

    move( xn, yn );
  }
}


//////////////////////////////////////////////////////////////////////////////////////////

SmageTWAnchorButton::SmageTWAnchorButton( QWidget *parent, const char *name ) : QPushButton( parent, name )
{
  setMouseTracking( TRUE );
  _notAnchored = QPixmap( STW_Button_NotAnchored );
  _anchored = QPixmap( STW_Button_Anchored );

  QBitmap mask = _anchored.createHeuristicMask();
  setMask( mask );

   _painter = new QPainter;
}

SmageTWAnchorButton::~SmageTWAnchorButton()
{
  delete _painter;
}

void
SmageTWAnchorButton::paintEvent(QPaintEvent *)
{
  SmageTextWidget *stw = (SmageTextWidget*)parentWidget();
  _painter->begin( this );
  if ( stw->_anchored ) _painter->drawPixmap( QPoint( 0, 0 ), _anchored );
  else _painter->drawPixmap( QPoint( 0, 0 ), _notAnchored );
  _painter->end();
}

void
SmageTWAnchorButton::mouseMoveEvent( QMouseEvent *e )
{
  SmageTextWidget *stw = (SmageTextWidget*)parentWidget();
  SmageViewerWidget *svw = (SmageViewerWidget*)stw->parentWidget();
  // mouse position on the SmageViewerWidget
  QPoint pos = svw->mapFromGlobal( QCursor::pos() );
  // manual handling if the mouse gets outside the SmageViewerWidget
  if ( pos.x() < 0 || pos.x() >= svw->width() || pos.y() < 0 || pos.y() >= svw->height() )
  {
    releaseMouse();
    stw->hide();
    svw->updateWidget();
    return;
  }

  if ( stw->_anchored )
  {
    QPushButton::mouseMoveEvent( e );
  }
  else
  {
    QMouseEvent *ne = new QMouseEvent( QEvent::MouseMove, pos, e->button(), e->state() );
    svw->passMouseEvent( ne );
  }
}

void
SmageTWAnchorButton::mousePressEvent( QMouseEvent *e )
{
  if ( e->button() == LeftButton ) QPushButton::mousePressEvent( e );
  else
  {
    SmageTextWidget *stw = (SmageTextWidget*)parentWidget();
    SmageViewerWidget *svw = (SmageViewerWidget*)stw->parentWidget();
    QMouseEvent *ne = new QMouseEvent( QEvent::MouseButtonPress, e->pos(), e->button(), e->state() );
    svw->passMouseEvent( ne );
  }
}

//////////////////////////////////////////////////////////////////////////////////////////

SmageTWScrollStyle::SmageTWScrollStyle() : QWindowsStyle()
{
}

SmageTWScrollStyle::~SmageTWScrollStyle()
{
}

// file://localhost/C:/Qt/3.2.0Educational/doc/html/customstyles.html
int
SmageTWScrollStyle::pixelMetric( PixelMetric, const QWidget * ) const
{
  return S_ABDh;
}
