#ifndef SMAGE_TEXT_WIDGET
#define SMAGE_TEXT_WIDGET

// project header
#include "structs/TheS_DictEntries.h"

// qt header
#include <qbitmap.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qtextedit.h>
#include <qwidget.h>
#include <qwindowsstyle.h>


class SmageTWAnchorButton;
class SmageTWScrollStyle;

/** The SmageTextWidget class is a container.
  *
  * @ingroup SmageApp
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  */
class SmageTextWidget : public QWidget
{
  Q_OBJECT
  
  public:
    SmageTextWidget( QWidget *parent, const char *name = 0, WFlags f = 0 );
    ~SmageTextWidget();
    void setText( const QString &text );
    void setDictText(const int &from, const int &to, 
		     const bool &showAllEntries=FALSE, const bool &showCodedTerm=FALSE);

    void setDictEntries(const DictEntriesPair &entries, const QString &codeTerm)
      { _dictEntries = entries; _codedDictTerm = codeTerm.copy(); }
    bool isDictEntry() { return !_codedDictTerm.isEmpty(); }

    void updateWidget();
    
    void updateBounds();
    void reshape( QPoint mousePos, bool anyway=FALSE );
    bool _anchored;
    SmageTWAnchorButton *_anchor;
    int _xOffset;
    int _yOffset;
    bool hasDictText();

  public slots:
    void anchored( bool state );

  protected:
    /** Just do nothing.
      * Without this reimplementation mouse clicks would be send to the parent
      * causing unexpected/undesired behaviour. 
      * (Like opening the link of a plugin). 
      */
    virtual void mousePressEvent ( QMouseEvent *) {}
    virtual void paintEvent( QPaintEvent *e );

  private:
    QSize calcWidgetSize( const QString &text );
    void calcWidgetMask( QSize size );
  
    QTextEdit *_text;
    QScrollBar *_hBar;
    QScrollBar *_vBar;
    QGridLayout *_layout;

    bool _hasHBar;
    bool _hasVBar;

    int _upperBound;
    int _rightBound;
    bool _left;
    bool _bottom;
    QBitmap _ll, _lr, _ur, _ul;


    SmageTWScrollStyle *_style;
    
    DictEntriesPair _dictEntries;
    QString _codedDictTerm;
};

//////////////////////////////////////////////////////////////////////////////////////////

/** The SmageTWAnchorButton class provides a round push button for the SmageTextWidget.
  *
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  */
class SmageTWAnchorButton : public QPushButton
{
  Q_OBJECT
  
  public:
    SmageTWAnchorButton( QWidget *parent, const char *name = 0 );
    ~SmageTWAnchorButton();
  
  protected:
    virtual void mouseMoveEvent( QMouseEvent *e );
    virtual void mousePressEvent( QMouseEvent *e );
    virtual void paintEvent( QPaintEvent *e );

  private:
    QPixmap _notAnchored;
    QPixmap _anchored;
    QPainter *_painter;
};

//////////////////////////////////////////////////////////////////////////////////////////

/** The SmageTWScrollStyle 
  *
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  */
class SmageTWScrollStyle : public QWindowsStyle
{
  Q_OBJECT

  public:
    SmageTWScrollStyle();
    ~SmageTWScrollStyle();
    int pixelMetric( PixelMetric m, const QWidget *widget = 0 ) const;
};

#endif // SMAGE_TEXT_WIDGET
