// project header
#include "SmageViewerShapedText.h"
#include "SmageViewerWidget.h"

// qt header
#include <qcursor.h>

SmageViewerShapedText::SmageViewerShapedText(QWidget *parent, const char* name)
  : SmageShapedTextBase(parent, name)
{
  setMouseTracking( TRUE );
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerShapedText::SmageViewerShapedText()\n");
#endif  

}

SmageViewerShapedText::~SmageViewerShapedText()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerShapedText::~SmageViewerShapedText()\n");
#endif  
}

void
SmageViewerShapedText::mouseMoveEvent( QMouseEvent *e )
{
  SmageViewerWidget *svw = (SmageViewerWidget*)parentWidget();
  QPoint pos = svw->mapFromGlobal( QCursor::pos() );
  QMouseEvent *ne = new QMouseEvent( QEvent::MouseMove, pos, e->button(), e->state() );
  svw->passMouseEvent( ne );
}

void
SmageViewerShapedText::mousePressEvent( QMouseEvent *e )
{
  SmageViewerWidget *svw = (SmageViewerWidget*)parentWidget();
  QPoint pos = svw->mapFromGlobal( QCursor::pos() );
  QMouseEvent *ne = new QMouseEvent( QEvent::MouseButtonPress, pos, e->button(), e->state() );
  svw->passMouseEvent( ne );
}

void
SmageViewerShapedText::initBubble()
{
  SmageViewerWidget *svw = (SmageViewerWidget*)parentWidget();
  _normal = svw->getNormalImagePointer()->copy(_position.x(), _position.y(), width(), height());
  _outlined = svw->getOutlinedImagePointer()->copy(_position.x(), _position.y(), width(), height());
  preparePixmaps();
  
  setPaletteBackgroundPixmap( _normal );
}

void
SmageViewerShapedText::scaleBubble( float xScale, float yScale )
{
  _position = QPoint( (int)(_position.x() * xScale), (int)(_position.y() * yScale) );
  QImage img =  _normal.convertToImage();
  img = img.smoothScale( (int)(img.width() * xScale), (int)(img.height() * yScale) );
  _normal = QPixmap( img );
  img =  _outlined.convertToImage();
  img = img.smoothScale( (int)(img.width() * xScale), (int)(img.height() * yScale) );
  _outlined = QPixmap( img );
  img =  _mask.convertToImage();
  img = img.smoothScale( (int)(img.width() * xScale), (int)(img.height() * yScale) );
  _mask = QPixmap( img );
  clearMask();
  setMask( _mask );
  setGeometry(_position.x(), _position.y(), img.width(), img.height() );
  setPaletteBackgroundPixmap( _normal );
}
