// project header
#include "SmageViewerSimpleBubble.h"

// qt header
#include <qcursor.h>
#include <qpainter.h>
#include <qpopupmenu.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageViewerSimpleBubble::SmageViewerSimpleBubble(QWidget *parent, const char* name, const SmageEllipseObj &ellipseObj)
  : SmageViewerShapedText(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerSimpleBubble::SmageViewerSimpleBubble()\n");
#endif

  _position = ellipseObj.textObjData.position;
  _textPosition = ellipseObj.textObjData.textPosition;
  _bubbleType = ellipseObj.textObjData.shapeType;

  _bubbleData.textData.bubbleText = ellipseObj.textObjData.textData.text;
  _bubbleData.textData.bubbleTextFont = ellipseObj.textObjData.textData.textFont;
  _bubbleData.textData.bubbleTextCol = ellipseObj.textObjData.colors.textColor;

  if (ellipseObj.textObjData.textData.textAlignment == 1)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignLeft;
  else if (ellipseObj.textObjData.textData.textAlignment == 2)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignHCenter;
  else if (ellipseObj.textObjData.textData.textAlignment == 3)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignRight;

  _bubbleData.shapeData.bubbleBackCol = ellipseObj.textObjData.colors.backgroundColor;
  _bubbleData.shapeData.bubbleOutlineCol = ellipseObj.textObjData.colors.outlineColor;
  _bubbleData.shapeData.bubbleOutlineWidth = ellipseObj.textObjData.lineWidth;

  _bubbleData.shapeData.bubbleBackgroundAlpha = ellipseObj.textObjData.colors.backgroundAlpha;
  _bubbleData.shapeData.bubbleRect = ellipseObj.ellipseShapeRect;

  int top = geometry().top();
  int left = geometry().left();
 
  QRegion region(_bubbleData.shapeData.bubbleRect);
  setGeometry(left, top, region.boundingRect().width(), region.boundingRect().height()); 
  
  initBubble();
}


SmageViewerSimpleBubble::~SmageViewerSimpleBubble()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerSimpleBubble::~SmageViewerSimpleBubble()\n");
#endif
}

void SmageViewerSimpleBubble::preparePixmaps()
{
  QRect br = _bubbleData.shapeData.bubbleRect;
  QPen pen;
  QBrush brush;
  int width = _bubbleData.shapeData.bubbleOutlineWidth;

  if (width > 0)
    pen = QPen(_bubbleData.shapeData.bubbleOutlineCol, 2*width - width%2);
  else
    pen = QPen( Qt::NoPen );
  if ( _bubbleData.shapeData.bubbleBackgroundAlpha != 0 )
    brush = QBrush(_bubbleData.shapeData.bubbleBackCol);
  else
    brush = QBrush( Qt::NoBrush );

  QPainter pn( &_normal );
  QPainter po( &_outlined );
  pn.setFont(_bubbleData.textData.bubbleTextFont);
  po.setFont(_bubbleData.textData.bubbleTextFont);

  QRect boundingRect = pn.boundingRect(br, Qt::WordBreak | Qt::AlignLeft, _bubbleData.textData.bubbleText);
  boundingRect = QRect( _textPosition, boundingRect.size() );

  pn.setBrush( brush );
  pn.setPen(pen);
  po.setBrush( brush );
  po.setPen(pen);

  if (_bubbleType == SMAGE_ELLIPSE )
  {
    pn.drawEllipse(br);
    po.drawEllipse(br);
    _mask = QBitmap(this->width(), height(), TRUE);
    QPainter pm( &_mask );
    pm.setPen( QPen(Qt::color1,1) );
    pm.setBrush( QBrush(Qt::color1) );
    pm.drawEllipse(br);
    setMask( _mask );
  }
  else
  {
/*
    pen = QPen(_bubbleData.shapeData.bubbleOutlineCol, 2*width);
    pn.setPen(pen);
    po.setPen(pen);
    pn.drawRect(br);
    po.drawRect(br);
*/
    if (_bubbleData.shapeData.bubbleOutlineWidth == 0 )
    {
      pn.drawRect(br);
      po.drawRect(br);
    }
    else
    {
      pn.setPen(QPen(_bubbleData.shapeData.bubbleOutlineCol, 1));
      pn.setBrush(Qt::NoBrush);
      po.setPen(QPen(_bubbleData.shapeData.bubbleOutlineCol, 1));
      po.setBrush(Qt::NoBrush);
      uint i;
      for (i=0; i<_bubbleData.shapeData.bubbleOutlineWidth-1;i++ )
      {
        pn.drawRect(br.x()+i,br.y()+i, br.width()-2*i, br.height()-2*i );
        po.drawRect(br.x()+i,br.y()+i, br.width()-2*i, br.height()-2*i );
      }
      pn.setBrush(brush);
      pn.drawRect(br.x()+i,br.y()+i, br.width()-2*i, br.height()-2*i );
      po.setBrush(brush);
      po.drawRect(br.x()+i,br.y()+i, br.width()-2*i, br.height()-2*i );
    }
  }
  pn.setPen(_bubbleData.textData.bubbleTextCol);
  pn.drawText(boundingRect, _bubbleData.textData.bubbleTextFlags, _bubbleData.textData.bubbleText);

  po.setPen(_bubbleData.textData.bubbleTextCol);
  po.drawText(boundingRect, _bubbleData.textData.bubbleTextFlags, _bubbleData.textData.bubbleText);
}

