// project header
#include "SmageViewerSpeechBubble.h"

// qt header
#include <qcursor.h>
#include <qpainter.h>

// c/c++ header
#include <assert.h>
#include <iostream>
#include <math.h>

using namespace std;

SmageViewerSpeechBubble::SmageViewerSpeechBubble(QWidget *parent, const char* name,const SmageComicEllipseObj &ellipseObj)
  : SmageViewerShapedText(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerSpeechBubble::SmageViewerSpeechBubble()\n");
#endif
  
  _position = ellipseObj.ellipseObjData.textObjData.position;
  _textPosition = ellipseObj.ellipseObjData.textObjData.textPosition;

  _bubblePeakPos = ellipseObj.bubblePeakPos; 
  _ellipseRect = ellipseObj.ellipseObjData.ellipseShapeRect;

  _openAngle = double(ellipseObj.openAngle);
  _closeAngle = double(ellipseObj.closeAngle);

  _bubbleData.shapeData.bubbleOutlineWidth = ellipseObj.ellipseObjData.textObjData.lineWidth;


  QRect boundingBox = QRect( _ellipseRect.x(), _ellipseRect.y(), _ellipseRect.width(), _ellipseRect.height());
  boundingBox = boundingBox.unite( QRect(_bubblePeakPos, QSize(1,1)));
  int top = geometry().top();
  int left = geometry().left();
  setGeometry(left, top, boundingBox.width(), boundingBox.height() );

  _bubbleData.textData.bubbleText = ellipseObj.ellipseObjData.textObjData.textData.text;
  _bubbleData.textData.bubbleTextFont = ellipseObj.ellipseObjData.textObjData.textData.textFont;
  _bubbleData.textData.bubbleTextCol = ellipseObj.ellipseObjData.textObjData.colors.textColor;
  _bubbleData.shapeData.bubbleBackCol = ellipseObj.ellipseObjData.textObjData.colors.backgroundColor;
  _bubbleData.shapeData.bubbleOutlineCol = ellipseObj.ellipseObjData.textObjData.colors.outlineColor;

  if (ellipseObj.ellipseObjData.textObjData.textData.textAlignment == 1)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignLeft;
  else if (ellipseObj.ellipseObjData.textObjData.textData.textAlignment == 2)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignHCenter;
  else if (ellipseObj.ellipseObjData.textObjData.textData.textAlignment == 3)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignRight;

  _bubbleData.shapeData.bubbleBackgroundAlpha = ellipseObj.ellipseObjData.textObjData.colors.backgroundAlpha;
  _bubbleData.shapeData.bubbleRect = ellipseObj.ellipseObjData.ellipseShapeRect;

  initBubble();
}


SmageViewerSpeechBubble::~SmageViewerSpeechBubble()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerSpeechBubble::~SmageViewerSpeechBubble()\n");
#endif
}


void
SmageViewerSpeechBubble::preparePixmaps()
{
  QPointArray bubblePoints;
  int closeAngle;
  if (_closeAngle <= _openAngle)
    closeAngle = int(360 - _openAngle + _closeAngle);
  else
    closeAngle = int(_closeAngle - _openAngle);
  
  bubblePoints.makeArc(_ellipseRect.x(), _ellipseRect.y(), _ellipseRect.width(), _ellipseRect.height(), 
			int(_openAngle)*16, closeAngle*16);

  if (bubblePoints.resize(bubblePoints.count()+1))
    bubblePoints.setPoint(bubblePoints.count()-1, _bubblePeakPos);

  QPen pen;
  QBrush brush;
  int width = _bubbleData.shapeData.bubbleOutlineWidth;

  if (width > 0)
    pen = QPen(_bubbleData.shapeData.bubbleOutlineCol, 2*width - width%2);
  else
    pen = QPen( Qt::NoPen );
  if ( _bubbleData.shapeData.bubbleBackgroundAlpha != 0 )
    brush = QBrush(_bubbleData.shapeData.bubbleBackCol);
  else
    brush = QBrush( Qt::NoBrush );


  QPainter pn( &_normal );
  QPainter po( &_outlined );
  pn.setFont(_bubbleData.textData.bubbleTextFont);
  po.setFont(_bubbleData.textData.bubbleTextFont);

  QRect boundingRect = pn.boundingRect(_ellipseRect, Qt::WordBreak | Qt::AlignLeft, _bubbleData.textData.bubbleText);
  boundingRect = QRect( _textPosition, boundingRect.size() );

  pn.setBrush( brush );
  pn.setPen(pen);
  pn.drawPolygon(bubblePoints);
  pn.setPen(_bubbleData.textData.bubbleTextCol);
  pn.drawText(boundingRect, _bubbleData.textData.bubbleTextFlags, _bubbleData.textData.bubbleText);

  po.setBrush( brush );
  po.setPen(pen);
  po.drawPolygon(bubblePoints);
  po.setPen(_bubbleData.textData.bubbleTextCol);
  po.drawText(boundingRect, _bubbleData.textData.bubbleTextFlags, _bubbleData.textData.bubbleText);

  _mask = QBitmap(this->width(), height(), TRUE);
  QPainter pm( &_mask );
  pm.setPen( QPen(Qt::color1,1) );
  pm.setBrush( QBrush(Qt::color1) );
  pm.drawPolygon(bubblePoints);
  setMask( _mask );
}
