// project header
#include "SmageViewerThinkBubble.h"

// qt header
#include <qpainter.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

OtherBubble::OtherBubble() {}
OtherBubble::OtherBubble( QRect r, QPoint p )
{
  rect = r;
  pos = p; 
}

SmageViewerThinkBubble::SmageViewerThinkBubble(QWidget *parent, const char* name, const SmageComicEllipseObj &ellipseObj)
  : SmageViewerShapedText(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerThinkBubble::SmageViewerThinkBubble()\n");
#endif  

  _position = ellipseObj.ellipseObjData.textObjData.position;
  _textPosition = ellipseObj.ellipseObjData.textObjData.textPosition;

  _bubbleData.textData.bubbleText = ellipseObj.ellipseObjData.textObjData.textData.text;
  _bubbleData.textData.bubbleTextFont = ellipseObj.ellipseObjData.textObjData.textData.textFont;
  _bubbleData.textData.bubbleTextCol = ellipseObj.ellipseObjData.textObjData.colors.textColor;

  if (ellipseObj.ellipseObjData.textObjData.textData.textAlignment == 1)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignLeft;
  else if (ellipseObj.ellipseObjData.textObjData.textData.textAlignment == 2)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignHCenter;
  else if (ellipseObj.ellipseObjData.textObjData.textData.textAlignment == 3)
    _bubbleData.textData.bubbleTextFlags |= Qt::AlignRight;

  _bubbleData.shapeData.bubbleBackCol = ellipseObj.ellipseObjData.textObjData.colors.backgroundColor;
  _bubbleData.shapeData.bubbleOutlineCol = ellipseObj.ellipseObjData.textObjData.colors.outlineColor;
  _bubbleData.shapeData.bubbleOutlineWidth = ellipseObj.ellipseObjData.textObjData.lineWidth;

  _bubbleData.shapeData.bubbleBackgroundAlpha = ellipseObj.ellipseObjData.textObjData.colors.backgroundAlpha;
  _bubbleData.shapeData.bubbleRect = ellipseObj.ellipseObjData.ellipseShapeRect;

  unsigned int i;
  for (i=0; i<ellipseObj.thinkBubbleCircles.size(); i++)
  {
    QRect r(QPoint(0,0), ellipseObj.thinkBubbleCircles[i].size);
    QPoint p(ellipseObj.thinkBubbleCircles[i].pos);
    OtherBubble ob( r, p );
    _otherBubblesVec.push_back(ob);
  }

  int top = geometry().top();
  int left = geometry().left();
  QRegion region(_bubbleData.shapeData.bubbleRect);

  for (i=0; i<_otherBubblesVec.size(); i++)
  {
    QRegion reg( _otherBubblesVec[i].pos.x(), _otherBubblesVec[i].pos.y(), _otherBubblesVec[i].rect.width(),_otherBubblesVec[i].rect.height());
    region += reg;
  }
  setGeometry(left, top, region.boundingRect().width()+2, region.boundingRect().height()+2);
  
  initBubble();
}

SmageViewerThinkBubble::~SmageViewerThinkBubble()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerThinkBubble::~SmageViewerThinkBubble()\n");
#endif
}

void SmageViewerThinkBubble::preparePixmaps()
{
  _mask = QBitmap(this->width(), height(), TRUE);
  QPainter pm( &_mask );
  pm.setPen( QPen(Qt::color1,1) );
  pm.setBrush( QBrush(Qt::color1) );

  QPen pen;
  QBrush brush;
  int width = _bubbleData.shapeData.bubbleOutlineWidth;

  if (width > 0)
    pen = QPen(_bubbleData.shapeData.bubbleOutlineCol, 2*width - width%2);
  else
    pen = QPen( Qt::NoPen );
  if ( _bubbleData.shapeData.bubbleBackgroundAlpha != 0 )
    brush = QBrush(_bubbleData.shapeData.bubbleBackCol);
  else
    brush = QBrush( Qt::NoBrush );

  QPainter pn( &_normal );
  QPainter po( &_outlined );
  pn.setFont(_bubbleData.textData.bubbleTextFont);
  po.setFont(_bubbleData.textData.bubbleTextFont);
  pn.setBrush( brush );
  pn.setPen(pen);
  po.setBrush( brush );
  po.setPen(pen);


  pn.drawEllipse(_bubbleData.shapeData.bubbleRect);
  po.drawEllipse(_bubbleData.shapeData.bubbleRect);
  pm.drawEllipse(_bubbleData.shapeData.bubbleRect);
 

  for (unsigned int i=0; i<_otherBubblesVec.size(); i++)
  {
    QRect obr = QRect( _otherBubblesVec[i].pos.x(), _otherBubblesVec[i].pos.y(), _otherBubblesVec[i].rect.width()-1,_otherBubblesVec[i].rect.height()-1);
    pn.drawEllipse(obr);
    po.drawEllipse(obr);
    pm.drawEllipse(obr);
  }
  QRect boundingRect = pn.boundingRect(_bubbleData.shapeData.bubbleRect, Qt::WordBreak | Qt::AlignLeft, _bubbleData.textData.bubbleText);
  boundingRect = QRect( _textPosition, boundingRect.size() );

  pn.setPen(_bubbleData.textData.bubbleTextCol);
  pn.drawText(boundingRect, _bubbleData.textData.bubbleTextFlags, _bubbleData.textData.bubbleText);

  po.setPen(_bubbleData.textData.bubbleTextCol);
  po.drawText(boundingRect, _bubbleData.textData.bubbleTextFlags, _bubbleData.textData.bubbleText);
  setMask( _mask );
}
