// project header
#include "SmageViewerWidget.h"

// qt header
#include <qbitmap.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

TheS_DictProps::TheS_DictProps()
{
  fromLang = 3;
  toLang = 0;
  showCodedDictItem = FALSE;
  showSecEntries = FALSE;
}

SmageViewerWidget::SmageViewerWidget( QWidget *parent, const char* name, WFlags f ) : SmageImgWidget( parent, name, f )
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageViewerWidget::SmageViewerWidget()\n");
#endif
  _decodedDataMap.clear();
  setMouseTracking(FALSE);

  _showOutlines = FALSE;
  _imgShouldBeDecoded = TRUE;

  _currentPixmapID = _previousPixmapID = -1;
  _currentTextID = _previousTextID = -1;
  _currentBubbleID = _previousBubbleID = -1;
  
  _sceneChanged = TRUE;
  _infoChanged = FALSE;
  _oldMousePos = QPoint( 0, 0 );
  _oldLabelSize = QSize( 100, 50 );

  _textLabel = NULL;
  _bubbleVec = NULL;

  _decodeIdBuffer = QImage();
  _outlineImg = QPixmap();
}


SmageViewerWidget::~SmageViewerWidget()
{
  _decodedDataMap.clear();  //?
}

void
SmageViewerWidget::paintEvent( QPaintEvent * )
{
  if ( !_imgLoaded ) return;
//  if ( _currentMousePos == _oldMousePos ) return;

  if ( _sceneChanged )
  {
    _painter->begin( this );
    _painter->drawImage( _imgStartPos, *_imgPointer );
    if ( _currentPixmapID != -1 )
    {
      std::map<unsigned int, SmageStructDecodedData>::iterator itr = _decodedDataMap.find( _currentPixmapID );
      itr = _decodedDataMap.find( (unsigned int)(_currentPixmapID) );
      if ( itr != _decodedDataMap.end() ) _painter->drawPixmap( _imgStartPos, itr->second._pixmap );
    }
    _painter->end();
  }
}

void
SmageViewerWidget::mouseMoveEvent( QMouseEvent *e )
{
  _mousePos = _currentMousePos = e->pos();
  onMouseMove();
}

void
SmageViewerWidget::onMouseMove()
{
  // test position and image
  if ( _decodeIdBuffer.isNull() ) return;
  // leaving the image but still be on the widget with the mouse
  if ( !_decodeIdBuffer.valid( _mousePos.x(), _mousePos.y() ) )
  {
    if ( _textLabel != NULL && !_textLabel->_anchored )
    {
      _textLabel->_anchor->releaseMouse();
      _textLabel->hide();
    }
    _textLabel = NULL;
    _currentPixmapID = _previousPixmapID = -1;
    _currentTextID = _previousTextID = -1;

    // In case that there is still a bubble visible although the cursor is outside the image
    if ( _bubbleVec != NULL ) 
      {
	std::vector<SmageViewerShapedText *>::iterator bubbleIter;
	for ( bubbleIter = _bubbleVec->begin(); bubbleIter != _bubbleVec->end(); bubbleIter++ )
	  (*bubbleIter)->hide();
	_bubbleVec = NULL;
	_previousBubbleID = _currentBubbleID = -1; 
      }

    _sceneChanged = TRUE;
    repaint( FALSE );
    //_oldMousePos = _currentMousePos;
    return;
  }
  // Look at same position in reference image to decide if and 
  // which information to show
  int colorIndex = _decodeIdBuffer.pixelIndex( _mousePos.x(), _mousePos.y() );
  int id = qBlue( _decodeIdBuffer.color( colorIndex ) );

  if ( id < 255 ) // valid annotation id
  {
    std::map<unsigned int, SmageStructDecodedData>::iterator itr = _decodedDataMap.find( id );
    if ( itr == _decodedDataMap.end() ) return;
    
    // () -> (text annotation)
    if ( itr->second._tw != NULL )
    {
      _currentTextID = id;
      // new text
      if ( _currentTextID != _previousTextID )
      {
        if ( _textLabel != NULL ) _oldLabelSize = _textLabel->size();
        if ( _textLabel != NULL && !_textLabel->_anchored )
        {
          _textLabel->_anchor->releaseMouse();
          _textLabel->hide();
        }
        _textLabel = itr->second._tw;
        _previousTextID = _currentTextID;
        _infoChanged = TRUE;
      }
      _sceneChanged = FALSE;
      // (image annotation) -> (text annotation)
      if ( _previousPixmapID != -1 )
      {
        _currentPixmapID = _previousPixmapID = -1;
        _sceneChanged = TRUE;
      }
      // only call move, if position really changed
      if ( _textLabel != NULL && !_textLabel->_anchored && _oldMousePos != _currentMousePos )
      {
        _textLabel->reshape( _currentMousePos );
        _textLabel->move( _currentMousePos.x()-_textLabel->_xOffset, _currentMousePos.y()-_textLabel->_yOffset );
        _infoChanged = TRUE;
        _sceneChanged = TRUE;
        _textLabel->raise();
        _textLabel->show();
        _textLabel->_anchor->grabMouse();
      }
    }
    // () -> (image annotation)
    if ( !itr->second._pixmap.isNull() )
    {
      _currentPixmapID = id;
      // (text annotation) -> (image annotation)
      if ( _previousTextID != -1 )
      {
        _previousTextID = _currentTextID = -1;
        _sceneChanged = TRUE;
        _infoChanged = FALSE; 
        _previousPixmapID = _currentPixmapID;
        if ( _textLabel != NULL && !_textLabel->_anchored ) _textLabel->hide();
        _textLabel = NULL;
      }
      // (no/image annotation) -> ( [new]image annotation)
      else if ( _currentPixmapID != _previousPixmapID )
      {
        _sceneChanged = TRUE;
        _infoChanged = FALSE;
        _previousPixmapID = _currentPixmapID;
      }
      // (image annotation) -> ([same]image annotation)
      else
      {
        _sceneChanged = FALSE;
        _infoChanged = FALSE; //? ntig
      }
    }
   
    // () -> bubble
    if ( itr->second._shapedTextVec.size() > 0 )
    {
      _currentBubbleID = id;
      if ( _currentBubbleID != _previousBubbleID )
      {
        std::vector<SmageViewerShapedText *>::iterator bubbleIter;
        if ( _bubbleVec != NULL  ) 
        {
          for ( bubbleIter = _bubbleVec->begin(); bubbleIter != _bubbleVec->end(); bubbleIter++ )
          {
            (*bubbleIter)->hide();
          }
        }
        _bubbleVec = &(itr->second._shapedTextVec);
        for ( bubbleIter = _bubbleVec->begin(); bubbleIter != _bubbleVec->end(); bubbleIter++ )
        {
          (*bubbleIter)->move((*bubbleIter)->_position + _imgStartPos);
          (*bubbleIter)->show();
          (*bubbleIter)->repaint(FALSE);
        }
        _previousBubbleID = _currentBubbleID;
      }
    }
  }
  // () -> (no annotation)
  else
  {
    if ( _bubbleVec != NULL ) 
    {
      std::vector<SmageViewerShapedText *>::iterator bubbleIter;
      for ( bubbleIter = _bubbleVec->begin(); bubbleIter != _bubbleVec->end(); bubbleIter++ )
        (*bubbleIter)->hide();
      _bubbleVec = NULL;
      _sceneChanged = TRUE;
      _previousBubbleID = _currentBubbleID = -1; 
    } else
    
    // (image annotation) -> (no annotation)
    if ( _previousPixmapID != -1 )
    {
      _sceneChanged = TRUE;
      _previousPixmapID = _currentPixmapID = -1;
    }
    // (text annotation) -> (no annotation)
    else if ( _previousTextID != -1 )
    {
      if ( _textLabel != NULL && !_textLabel->_anchored )
      {
        _textLabel->_anchor->releaseMouse();
        _textLabel->hide();
        _infoChanged = TRUE;
      }
      _textLabel = NULL;
      _currentTextID = _previousTextID = -1;
    }

    else
    {
      _sceneChanged = FALSE;
      _infoChanged = FALSE;
    }
  }

  // to avoid flickering: repaint( FALSE )
  repaint( FALSE );
  _oldMousePos = _currentMousePos;
}

void
SmageViewerWidget::enterEvent( QEvent * )
{
  _currentPixmapID = _previousPixmapID = -1;
  _currentTextID = _previousTextID = -1;
  updateWidget();
}

void
SmageViewerWidget::leaveEvent( QEvent * )
{
  if ( _textLabel != NULL && !_textLabel->_anchored )
  {
    // the leave event is called for example than a popup/context menu is accessed
    // (clicking right mouse button in the plugin)
    // to be able to access anything in the menu (without getting loose of the mouse)
    // the mouse focus needs to be removed from the button
    _textLabel->_anchor->releaseMouse(); 
    
  }
  _currentPixmapID = _previousPixmapID = -1;
  _currentTextID = _previousTextID = -1;
  updateWidget();
}

void SmageViewerWidget::focusInEvent(QFocusEvent *)
{
  _sceneChanged = TRUE;
  _currentPixmapID = _previousPixmapID = -1;
  _currentTextID = _previousTextID = -1;

  repaint(FALSE);
}

void SmageViewerWidget::focusOutEvent(QFocusEvent *)
{
  _sceneChanged = TRUE;
  _currentPixmapID = _previousPixmapID = -1;
  _currentTextID = _previousTextID = -1;

  repaint(FALSE);
}


void
SmageViewerWidget::passMouseEvent( QMouseEvent *e )
{
  if ( e->type() == QEvent::MouseMove ) mouseMoveEvent( e );
  else if ( e->type() == QEvent::MouseButtonPress ) mousePressEvent( e );
}


void 
SmageViewerWidget::updateWidget()
{
  // to avoid cases where the image is not repainted
  _sceneChanged = TRUE;
  repaint( FALSE );
}

void
SmageViewerWidget::showOutlines(const bool &show) 
{ 
  _showOutlines = show; 
  if (show)
    _imgPointer = &_outlineImg;
  else
    _imgPointer = &_img;
    
    
  std::map<unsigned int, SmageStructDecodedData>::iterator itr;
  for ( itr = _decodedDataMap.begin(); itr  != _decodedDataMap.end(); itr++ )
  {
    if ( itr->second._shapedTextVec.size() > 0 )
    {
      std::vector<SmageViewerShapedText *>::iterator bubbleIter;
      for ( bubbleIter = itr->second._shapedTextVec.begin(); bubbleIter != itr->second._shapedTextVec.end(); bubbleIter++ )
      {
        if (show) (*bubbleIter)->setPaletteBackgroundPixmap( (*bubbleIter)->_outlined );
        else (*bubbleIter)->setPaletteBackgroundPixmap( (*bubbleIter)->_normal );
      }
    }
  }  
  updateWidget();
}

QImage*
SmageViewerWidget::getNormalImagePointer()
{
  return &_img; 
}

QImage*
SmageViewerWidget::getOutlinedImagePointer()
{
  return &_outlineImg;
}


void SmageViewerWidget::updateDictText(const int &fromLang, const int &toLang,
				       const bool &showSecEntries, const bool &showCodedDictItem)
{
  if (_decodedDataMap.empty())
    return;

  _dictProps.fromLang = fromLang;
  _dictProps.toLang = toLang;
  _dictProps.showCodedDictItem = showCodedDictItem;
  _dictProps.showSecEntries = showSecEntries;

  map<unsigned int, SmageStructDecodedData>::iterator decodedDataMapIter;
  for (decodedDataMapIter=_decodedDataMap.begin(); decodedDataMapIter!=_decodedDataMap.end(); decodedDataMapIter++)
  {
    if (decodedDataMapIter->second._tw != NULL)
      if (decodedDataMapIter->second._tw->hasDictText() )
    	  decodedDataMapIter->second._tw->setDictText(fromLang, toLang, showSecEntries, showCodedDictItem);
  }
}
