#ifndef SMAGE_VIEWER_WIDGET_H
#define SMAGE_VIEWER_WIDGET_H

// project header
#include "SmageImgWidget.h"
#include "SmageTextWidget.h"
#include "SmageViewerShapedText.h"
#include "structs/SmageStructDecodedData.h"

// qt header
#include <qwidget.h>
#include <qpainter.h>

// c/c++ header
#include <map>
#include <vector>

struct TheS_DictProps {
  TheS_DictProps();
  int fromLang;
  int toLang;
  bool showCodedDictItem;
  bool showSecEntries;
};

/** The base widget for viewing operations and user interaction on an encoded image.
  *
  * @ingroup SmageApp
  * @ingroup NPSmagePlugin
  * @ingroup SmageAXPlugin
  *
  */
class SmageViewerWidget : public SmageImgWidget
{
  Q_OBJECT

    /** The base widget for viewing interactions.
      */
    typedef SmageImgWidget inherited;

 public:
    /** Constructor.
      */
    SmageViewerWidget( QWidget *parent, const char* name, WFlags f );
    
    /** Destructor.
      */
    ~SmageViewerWidget();
    
    /** Pass a mouse event of a child widget to the SmageViewerWidget.
      * Used by SmageTextWidget.
      */
    void passMouseEvent( QMouseEvent *e );

    /** Returns a class specific identifier string.
      */
    virtual QString getWidgetType() const { return QString( "SMAGE_VIEWER_WIDGET" ); }
    
    /** 
      */
    virtual void showOutlines( const bool &show );

    QImage getOutlineImg() { return _outlineImg; }
    QImage getIdBuffer() { return _decodeIdBuffer; }
    int getNumCodedDataItems() { return _decodedDataMap.size(); }

    void updateDictText(const int &fromLang, const int &toLang,
			const bool &showSecEntries, const bool &showCodedDictItem);

    /**
      */
    QImage* getNormalImagePointer();
    QImage* getOutlinedImagePointer();
    
  public slots:
    /** 
      */
    void updateWidget();

 protected:
    /** 
      */
    virtual void paintEvent(QPaintEvent *event);

    /** Event handler for mouse move events.
      * This function sets _mousePos to the current event->pos() and then calls
      * onMouseMove(), which uses _mousePos.
      * If any transformations (translation, scalings, ...) should be applied to 
      * that position, reimplement the funtion:
      * <ol>
      * <li>get mouse position with event->pos()</li>
      * <li>transform it and store the result in _mousePos</li>
      * <li>call onMouseMove()</li>
      * </ol>
      */
    virtual void mouseMoveEvent(QMouseEvent *event);
    
    /**
      */
    virtual void focusInEvent(QFocusEvent *event);
    
    /**
      */
    virtual void focusOutEvent(QFocusEvent *event);
    
    /**
      */
    virtual void enterEvent( QEvent* );
    
    /**
      */
    virtual void leaveEvent( QEvent* );

    /** Implementation of the behaviour, than an mouse move event occurs.
      * @see mouseMoveEvent()
      */
    void onMouseMove();

    /** The active (if any) widget for displaying encoded text.
      *
      */
    SmageTextWidget *_textLabel;
    
    /**
      * @see showOutlines()
      */
    bool _showOutlines;

    /** ID-Buffer that holds coded regions.
      *
      */
    QImage _decodeIdBuffer;

    /** Buffer that holds outlines of coded regions in addition to original image
      * This image is displayed than _showOutlines is TRUE.
      */
    QImage _outlineImg;
    
    /**  Map holding ID and associated data.
      */
    std::map<unsigned int, SmageStructDecodedData> _decodedDataMap;

    /** 
      */
    int _currentPixmapID;

    /** 
      */
    int _previousPixmapID;

    /** 
      */
    int _currentTextID;

    /** 
      */
    int _previousTextID;

    /**
      */
    int _currentBubbleID;

    /**
      */
    int _previousBubbleID;

    /** 
      */
    bool _sceneChanged;

    /** 
      */
    bool _infoChanged;

    /** 
      */
    QPoint _oldMousePos;

    /** 
      */
    QPoint _currentMousePos;

    /** 
      */
    QPoint _mousePos;

    /** 
      */
    QSize _oldLabelSize;

    std::vector<SmageViewerShapedText *> *_bubbleVec;

    TheS_DictProps _dictProps;
};

#endif
