// project header
#include "SmageConfig.h"
#include "SmageXmlParser.h"
#include "SmageXmlLanguageFileParser.h"

// qt header
#include <qfile.h>
#include <qstring.h>

// c/c++ header
#include <iostream>

extern std::vector<TheS_Language> __languagesVec;

SmageXmlParser::SmageXmlParser()
  : QXmlDefaultHandler()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageXmlParser::SmageXmlParser()\n");
#endif
  _shouldStopParsing = FALSE;
  _getData = FALSE;
  _curIDNum = -1;
  _curIDString = QString::null;
  _searchedIDNum = -1;
  _searchedIDString = QString::null;
  _searchLang = -1;
  _searchedEntry = QString::null;
  _searchTagName = QString::null;
  _dictEntriesPair = NULL;

  _curDictIdEntry.first = -1;
  _curDictIdEntry.second.primeWord = QString::null;
  _curDictIdEntry.second.secWords.clear();
  
  char *d = getenv("SMAGE_DIR");
  if ( d == NULL ) _dictDir = "";
  else _dictDir = QString( QString(d) + "dictionary/");
  // inits the language list
  SmageXmlLanguageFileParser languageFile;
}
 
SmageXmlParser::~SmageXmlParser()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageXmlParser::~SmageXmlParser()\n");
#endif
}


bool SmageXmlParser::searchForEntries(const char *fileName, std::pair<QString, DictIdEntriesMap> &dictEntriesPair,
				      const QString &idString, const int &idNum)
{
  _searchedIDNum = idNum;
  _searchedIDString = idString;
  _dictEntriesPair = &dictEntriesPair;
  dictEntriesPair.first = QString::null;

  QFile xmlFile( _dictDir + fileName);
  QXmlInputSource source(&xmlFile);
  QXmlSimpleReader reader;
  reader.setContentHandler(this);
  reader.parse(source);

  if (dictEntriesPair.first.isEmpty())
    return FALSE;
  else
    return TRUE;
}


bool SmageXmlParser::searchForId(const char *fileName, QString &idString, int &idNum, 
				 const int &lan, const QString &entry)
{
  _searchLang = lan;
  _searchedEntry = entry.lower();
  _foundEntry = FALSE;
  _curIDNum = -1;
  _curIDString = QString::null;

  if (!getLangTagName(lan, _searchTagName))
    return FALSE;

  QFile xmlFile(_dictDir + fileName);
  QXmlInputSource source(&xmlFile);
  QXmlSimpleReader reader;
  reader.setContentHandler(this);
  reader.parse(source);

  if (_foundEntry)
    {
      idString = _curIDString;
      idNum = _curIDNum;
      return TRUE;
    }
  
  return FALSE;
}

bool SmageXmlParser::getLanguage(const QString &word, int &language)
{
  std::vector<TheS_Language>::iterator langIter;
  int i;
  for ( langIter=__languagesVec.begin(), i=0; langIter != __languagesVec.end(); ++langIter, ++i)
  {
    if ( (*langIter)._tag == word) 
    {
      language = i;
      return TRUE;
    }
  }
  language = -1;
  return FALSE;
}

bool SmageXmlParser::getLangTagName(const int &language, QString &tagName)
{
  if ( language >= __languagesVec.size() || language < 0 ) return FALSE;
  tagName = __languagesVec[language]._tag;
  return TRUE;
}

bool SmageXmlParser::startDocument()
{
  return TRUE;
}

bool SmageXmlParser::startElement(const QString &, const QString &, const QString &tagName, 
				  const QXmlAttributes &atts)
{
  if (!_shouldStopParsing)
    {
      _curTagName = tagName;

      if (!_searchedIDString.isEmpty())
	getEntryStartElement(atts);
      else if (!_searchedEntry.isEmpty())
	getIDStartElement(atts);
    }

  return TRUE;
}

bool SmageXmlParser::endElement(const QString&, const QString&, const QString &tagName)
{
  if (!_shouldStopParsing)
    {
      if (!_searchedIDString.isEmpty())
	getEntryEndElement(tagName);
      else if (!_searchedEntry.isEmpty())
	getIDEndElement(tagName);
    }

  return TRUE;
}

bool SmageXmlParser::characters(const QString &data)
{
  if (!_shouldStopParsing)
    {
      if (!_searchedIDString.isEmpty())
	getEntryChars(data);
      else if (!_searchedEntry.isEmpty())
	getIDChars(data);
    }

  return TRUE;
}

void SmageXmlParser::getEntryStartElement(const QXmlAttributes &atts)
{
  if (_getData)
    {
      if (!_curTagName.contains('_'))
	getLanguage(_curTagName, _curDictIdEntry.first);
    }
  
  if (_curTagName == "DictItem")
    {
      if (_searchedIDNum != -1)
	{
	  if (atts.value(0).toInt() == _searchedIDNum)
	    _getData = TRUE;
	}
    }
}

void SmageXmlParser::getIDStartElement(const QXmlAttributes &atts)
{
  if (_curTagName == "DictItem")
    {
      if (atts.length() > 0)
	_curIDNum = atts.value(0).toInt();
    }

  else if (_curTagName == _searchTagName)
    _getData = TRUE;
}

void SmageXmlParser::getEntryEndElement(const QString &tagName)
{
  if (_getData)
    {
      if (tagName == "DictItem")
	{      
	  _getData = FALSE;
	  _shouldStopParsing = TRUE;
	  
	  _dictEntriesPair->first = _searchedIDString;
	}
      else
	{
	  int tmpLang;
	  if (getLanguage(tagName, tmpLang))
	    {
	      if (tmpLang == _curDictIdEntry.first)
		{
		  _dictEntriesPair->second.insert(_curDictIdEntry);
		  _curDictIdEntry.second.primeWord = QString::null;
		  _curDictIdEntry.second.secWords.clear();
		}
	    }
	}
    }
}

void SmageXmlParser::getIDEndElement(const QString &tagName)
{
  if (_getData)
    {
      if (tagName == _searchTagName)
	_getData = FALSE;
    }
}

void SmageXmlParser::getEntryChars(const QString &data)
{
  if (_getData)
    {
      if (!data.isEmpty() && data != "\n")
	{
	  if (_curTagName != "DictItem")
	    {
	      if (_curTagName.right(2) == "_P")
		_curDictIdEntry.second.primeWord = data;
	      else if (_curTagName.right(2) == "_S")
		_curDictIdEntry.second.secWords.push_back(data);
	    }
	}
    }
  else if (_searchedIDNum == -1)
    {  
      QString tmpData = data;
      if (tmpData.findRev('\n') != -1)
	tmpData.truncate(data.length()-1);

      if (_curTagName == "DictItem")
	if (tmpData == _searchedIDString)
	  _getData = TRUE;
    }
}

void SmageXmlParser::getIDChars(const QString &data)
{
  if (_curTagName == "DictItem")
    {
      _curIDString = data;

      if (_curIDString.findRev('\n') != -1)
	_curIDString.truncate(data.length()-1);
    }

  else if (_getData)
    if (data.lower() == _searchedEntry)
      {
	_foundEntry = TRUE;
	_shouldStopParsing = TRUE;
      }
}
