#ifndef SMAGE_XML_PARSER_H
#define SMAGE_XML_PARSER_H

// project header
#include "structs/TheS_DictEntries.h"

// qt header
#include <qxml.h>

class SmageXmlParser : public QXmlDefaultHandler
{
 public:
    
  SmageXmlParser();
  ~SmageXmlParser();

  bool startDocument();
  bool startElement(const QString&, const QString&, const QString& , const QXmlAttributes&);
  bool endElement(const QString&, const QString&, const QString&);
  bool characters(const QString &data);

  bool searchForEntries(const char *fileName, std::pair<QString, DictIdEntriesMap> &dictEntriesPair,
			const QString &idString, const int &idNum=-1);
  bool searchForId(const char *fileName, QString &idString, int &idNum, const int &lan, const QString &entry);

  bool getLanguage(const QString &word, int &language);
  bool getLangTagName(const int &language, QString &tagName);

 private:
  
  void getEntryStartElement(const QXmlAttributes &atts);
  void getIDStartElement(const QXmlAttributes &atts);
  void getEntryEndElement(const QString &tagName);
  void getIDEndElement(const QString &tagName);
  void getEntryChars(const QString &data);
  void getIDChars(const QString &data);

  bool _shouldStopParsing, _getData;
  QString _searchedIDString, _searchedEntry, _searchTagName;
  QString _curTagName, _curIDString;
  int _searchedIDNum, _curIDNum;
  int _searchLang;
  bool _foundEntry;

  /** Directory where the dictionaries are stored.
    */
  QString _dictDir;

  std::pair<int, TheS_DictEntries> _curDictIdEntry;
  std::pair<QString, DictIdEntriesMap> *_dictEntriesPair;
};

#endif
