/** Class SmageApp derived from "SmageAppBase".
  *
  * @ingroup SmageApp
  *
  * all actions are carried out here (pressing a button, selecting a menu item, ...)
  */

#ifndef _SMAGE_APP_H
#define _SMAGE_APP_H

// project header
#include "SmageAppBase.h"
#include "../smageSharedFiles/structs/TheS_DictEntries.h"

class SmageApp : public SmageAppBase
{
  Q_OBJECT

 public:
    /** Constructor.
      */
    SmageApp(QWidget *topLevel, int argc, char *argv[]);
    
    /** Destructor.
      */
    ~SmageApp();
   
 protected:  
    /** Opens an image file.
      */
    virtual void doOpen();
    
    /** Saves current image.
      */
    virtual void doSaveP();
    
    /** Current selection type: rectangular region.
      */
    virtual void doSelectRect();
    
    /**   Specify text data to be embedded in selected region and correlate specified data with image region.
      */
    virtual void doInsertTInfo();
    
    /** Specify image data to be embedded in selected region.
      */
    virtual void doInsertBInfo();

    /** Encode data into current image.
      */
    virtual void doBuildImg();
    
    /** Delete data from image.
      */
    virtual void doErase();
    
    /** Show outlines of coded image regions.
      */
    virtual void doShowOutlines();

    virtual void doShowDataManager(const bool &);
    virtual void doSwitchToEncoder();
    virtual void doInsertDictEntry();
    virtual void doShowSecEntries(bool);
    virtual void doShowCodedDictItem(bool);
    virtual void doTranslFrom(int);
    virtual void doTranslTo(int);
    virtual void doGenerateHtml();

#ifdef SMAGE_FULL
    /** Current selection type: any shape.
      */
    virtual void doSelectPoly();
#endif

 private:

    /** A region has been selected, data can now be inserted.
      */
    static void regionSelectedCB(void *userData, const bool &regionSelected, const bool &allowBuildingImg);

    /** Shows a message in the status bar.
      */
    static void showMsgCB(void *userData, const QString &msg, const int &ms);

    /** Show coded image regions.
      */
    static void showRegionsCB(void *userData, const bool &setEnabled, const bool &showRegions);

    /** New image (SmageAppViewerWidget class).
      */
    static void openViewerImgCB(void *userData, const bool &allowErasing, const bool &hasDictEntriesEncoded, const TheS_DictProps &props);

    /** New image (SmageImgEncodeWidget class).
      */
    static void openSelectorImgCB(void *userData, const SmageSelectionType &selectionType, 
				  const bool &allowDataInsertion, const bool &allowBuildingImg);

    /** Image region (from "SmageImgGrabber") has been selected, copied, and can can now be inserted
      * into the active app's widget
      */
    static void selCopyImgCB(void *userData, const QImage* img, const QPointArray &region);

    /** A new image has been opened or a different workspace widget is activated.
      * is image coded with data ("isCodedImg"); which selection type was set ("selectionType")
      */
    void doOpenImg(const bool &enableDataEncoding, const SmageSelectionType &selectionType, const bool &allowErasing, 
       const bool &allowDataInsertion, const bool &allowBuildingImg, const bool &enableDictMenu, const TheS_DictProps &dictProps);

    /** A workspace widget is about to get closed.
      */
    static void closeCB(void *userData);
    
    /** Saves current child's picture.
      */
    static void saveCB(void *userData);

    /** Should erasing be allowed.
      */
    static void eraseCB(void *userData, const bool &enableErasing);

    /** Should building image be allowed.
      */
    static void buildAllowedCB(void *userData, const bool &enableBuildingImg);

    /** Sets a new main window caption.
      */
    static void captionCB(void *userData, const QString &caption);

    /** Forcing to open new image widget type.
      */
    static void forceOpenNewCB(void *userData, SmageImgWidget *imgWidget, const QString &fileName);

    /** Ok button was pressed in the "_formatTextWidget"
     */
    static void formatTextOkCB(void *userData);

    /** Initialize a new workspace widget (a workspace widget only contains the (un)coded image).
      */
    void initializeNewImgWidget(const char *name, const SmageImgWidget *tmpImgWidget = NULL);

    /** Assign a new type of image region selection.
      */
    void assignNewSelectionType(const SmageSelectionType &selectionType);

    void updateDictText();

    /** "_imgWidgetCounter" is included in each specific workspace widget name.
      */
    unsigned int _imgWidgetCounter;

    bool _showCodedDictItem, _showSecEntries;
    int _fromLang, _toLang;
};

#endif
