// project header
#include "SmageAppBase.h"
#include "SmageHelpBrowser.h"
#include "SmageInfoWidget.h"
#include "../smageSharedFiles/SmageDecodeHandler.h"

// qt header
#include <qapplication.h>
#include <qbuttongroup.h>
#include <qlist.h>
#include <qmenubar.h>
#include <qpixmap.h>
#include <qradiobutton.h>
#include <qtimer.h>
#include <qvbox.h>

// c/c++ header
#include <assert.h>
#include <iostream>
#include <stdlib.h>

// project icons
#ifndef SMAGE_COMMON_ICONS
#define SMAGE_COMMON_ICONS
#include "./pixmaps/open.xpm"
#include "./pixmaps/exit.xpm"
#include "./pixmaps/selectRect.xpm"
#include "./pixmaps/selectRect_dis.xpm"
#ifdef SMAGE_FULL
#include "./pixmaps/selectPoly.xpm"
#include "./pixmaps/selectPoly_dis.xpm"
#endif
#endif // SMAGE_COMMON_ICONS

#include "./pixmaps/save.xpm"
#include "./pixmaps/save_dis.xpm"
#include "./pixmaps/erase.xpm"
#include "./pixmaps/erase_dis.xpm"
#include "./pixmaps/spritzeT.xpm"
#include "./pixmaps/spritzeT_dis.xpm"
#include "./pixmaps/spritzeD.xpm"
#include "./pixmaps/spritzeD_dis.xpm"
#include "./pixmaps/build.xpm"
#include "./pixmaps/build_dis.xpm"
#include "./pixmaps/view.xpm"
#include "./pixmaps/view_dis.xpm"

#ifdef SMAGE_FULL
#include "./pixmaps/spritzeI.xpm"
#include "./pixmaps/spritzeI_dis.xpm"
#endif

using namespace std;

extern std::vector<TheS_Language> __languagesVec;

SmageAppBase::SmageAppBase(QWidget *topLevel, int, char **) : QMainWindow(topLevel, 0)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgWidget::SmageAppBase()\n");
#endif
  setCaption( tr("Smage - Smart Images") );

  _translFromGroup = NULL;
  _translToGroup = NULL;

  char *d = getenv("SMAGE_DIR");
  QString smageDir ="";
  if ( d != NULL ) smageDir = QString( d );

  QStringList paths;
  paths << smageDir << QString(smageDir + "help") << QString(smageDir + "help/pixs");
  _helpBrowser = new SmageHelpBrowser(0, "help/helpIndex.html", paths, "helpBrowser");

  if (QApplication::desktop()->width() > 700)
  {
    setGeometry(200,150,600,400);
    _helpBrowser->setGeometry(250,130,750,550);
  }
  else
  {
    setGeometry(100,50,400,300);
    _helpBrowser->setGeometry(150,30,500,450);
  }

  initialize();

  QTimer *timer = new QTimer(this);
  connect(timer, SIGNAL(timeout()), this, SLOT(timerDone()));
  timer->start(5000, TRUE);
  //timer->start(60, TRUE);

  _startWidget = new SmageInfoWidget(this, "startWidget", WStyle_NoBorder | WStyle_StaysOnTop | 
             WStyle_Tool | WX11BypassWM | WWinOwnDC);

  QString imgPathStart;

  imgPathStart = QString(smageDir + "pixmaps/startImg.png");

  _startWidget->setImg(imgPathStart);
  _startWidget->show();

  _aboutWidget = new SmageInfoWidget(this, "aboutWidget", WStyle_NoBorder | WStyle_StaysOnTop | 
             WStyle_Tool | WX11BypassWM | WWinOwnDC);
  
  QString imgPathAbout;
  imgPathAbout = QString(smageDir + "pixmaps/smageInfo.png");
  _aboutWidget->setImg(imgPathAbout, TRUE);
  _aboutWidget->hide();

  _imgGrabber = new SmageImgGrabber(0, "imgGrabber");

  _htmlGenerator = new SmageHTMLGenerator(this);
  _htmlGenerator->hide();
}


SmageAppBase::~SmageAppBase() 
{
  delete _toggleButtonGroup;
  delete _helpBrowser;
  delete _imgGrabber;
}


void SmageAppBase::initialize()
{  
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageAppBase::initialize()\n");
#endif

  // ############ File Menu #############################

  _fileMenu = new QPopupMenu(this);
  CHECK_PTR(_fileMenu);

  _fileMenu->insertItem(QPixmap(open_xpm), tr("&Open"), this, SLOT(open()), CTRL+Key_O, 1);

  QIconSet saveIconSet = QIconSet(QPixmap(save_xpm));
  saveIconSet.setPixmap(QPixmap(save_xpm), QIconSet::Small, QIconSet::Normal);
  saveIconSet.setPixmap(QPixmap(save_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _saveItemId = _fileMenu->insertItem(saveIconSet, tr("&Save as"), this, SLOT(saveP()), CTRL+Key_S, 2);

  _fileMenu->setItemEnabled(_saveItemId, FALSE);

  _fileMenu->insertSeparator();
  _fileMenu->insertItem(QPixmap(exit_xpm), tr("E&xit"), qApp, SLOT(quit()), CTRL+Key_Q);
  menuBar()->insertItem(tr("&File"), _fileMenu);
  // ############ View Menu #############################

  _viewMenu = new QPopupMenu(this);
  Q_CHECK_PTR(_viewMenu);

  _showDataManagerId = _viewMenu->insertItem( tr("Overview Window"), this, SLOT( showDataManager() ), 0, 1 );
  _viewMenu->setItemEnabled( _showDataManagerId, FALSE );
  menuBar()->insertItem(tr("&View"), _viewMenu);

  _switchToEncoderId = _viewMenu->insertItem(tr("Add Data"), this, SLOT(switchToEncoder()), 0, 3);
  _viewMenu->setItemEnabled(_switchToEncoderId, FALSE);

  _viewMenu->insertSeparator();
  _htmlGeneratorId = _viewMenu->insertItem(tr("&Generate HTML code"), this, SLOT(generateHtml()));
  _viewMenu->setItemEnabled(_htmlGeneratorId, FALSE);

  // ############ Dictionary Menu #############################
  // inits the language list
  SmageXmlLanguageFileParser languageFile;
  _dictMenu = SmageDecodeHandler::createDictMenu( &__languagesVec, this, _translFromGroup, _translToGroup );
  _dictMenuId = menuBar()->insertItem(tr("&Image Dictionary"), _dictMenu);
  menuBar()->setItemEnabled(_dictMenuId, FALSE);

  // ############ Help Menu #############################

  QPopupMenu *helpMenu = new QPopupMenu(this);
  Q_CHECK_PTR(helpMenu);
  helpMenu->insertItem( tr("Help"), this, SLOT(help()), Qt::Key_F1);
  helpMenu->insertSeparator();
  helpMenu->insertItem( tr("About Smage"), this, SLOT(about()), CTRL+Key_A);
  menuBar()->insertItem( tr("&Help"), helpMenu);

  // ### Initializing the toolbar ###

  _toolBar = new QToolBar(this, "tools");
  
  _toggleButtonGroup = new QButtonGroup();
  _toggleButtonGroup->setExclusive(TRUE);

  QIconSet eraseIconSet = QIconSet(QPixmap(erase_xpm));
  eraseIconSet.setPixmap(QPixmap(erase_xpm), QIconSet::Small, QIconSet::Normal);
  eraseIconSet.setPixmap(QPixmap(erase_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _erase = new QToolButton(eraseIconSet, tr("Erase encoded data from image"), 
         QString::null, this, SLOT(erase()), _toolBar, "erase");
  _erase->setToggleButton(FALSE);
  _erase->setEnabled(FALSE);

  QIconSet viewIconSet = QIconSet(QPixmap(view_xpm));
  viewIconSet.setPixmap(QPixmap(view_xpm), QIconSet::Small, QIconSet::Normal);
  viewIconSet.setPixmap(QPixmap(view_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _showOutlines = new QToolButton(viewIconSet, tr("Show coded image regions"), 
          QString::null, this, SLOT(showOutlines()), _toolBar, "showOutlines");
  _showOutlines->setToggleButton(TRUE);
  _showOutlines->setEnabled(FALSE);

  _toolBar->addSeparator();

  QIconSet selectRectIconSet = QIconSet(QPixmap(selectRect_xpm));
  selectRectIconSet.setPixmap(QPixmap(selectRect_xpm), QIconSet::Small, QIconSet::Normal);
  selectRectIconSet.setPixmap(QPixmap(selectRect_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _selectRect = new QToolButton(selectRectIconSet, tr("Select a rectangular image region"), 
        QString::null, this, SLOT(selectRect()), _toolBar, "selectRect");
  _selectRect->setToggleButton(TRUE);
  _selectRect->setEnabled(FALSE);
  _toggleButtonGroup->insert(_selectRect);

#ifndef SMAGE_DEMO
  QIconSet selectPolyIconSet = QIconSet(QPixmap(selectPoly_xpm));
  selectPolyIconSet.setPixmap(QPixmap(selectPoly_xpm), QIconSet::Small, QIconSet::Normal);
  selectPolyIconSet.setPixmap(QPixmap(selectPoly_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _selectPoly = new QToolButton(selectPolyIconSet, tr("Select any image region"), 
        QString::null, this, SLOT(selectPoly()), _toolBar, "selectPoly");
  _selectPoly->setToggleButton(TRUE);
  _selectPoly->setEnabled(FALSE);
  _toggleButtonGroup->insert(_selectPoly);
#endif

  _toolBar->addSeparator();

  QIconSet insertTInfoIconSet = QIconSet(QPixmap(spritzeT_xpm));
  insertTInfoIconSet.setPixmap(QPixmap(spritzeT_xpm), QIconSet::Small, QIconSet::Normal);
  insertTInfoIconSet.setPixmap(QPixmap(spritzeT_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _insertTInfo = new QToolButton(insertTInfoIconSet, tr("Insert text into selected image region"), 
         QString::null, this, SLOT(insertTInfo()), _toolBar, "insertTInfo");
  _insertTInfo->setEnabled(FALSE);

  QIconSet insertDictEntryIconSet = QIconSet(QPixmap(spritzeD_xpm));
  insertDictEntryIconSet.setPixmap(QPixmap(spritzeD_xpm), QIconSet::Small, QIconSet::Normal);
  insertDictEntryIconSet.setPixmap(QPixmap(spritzeD_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _insertDictEntry = new QToolButton(insertDictEntryIconSet, tr("Insert a dictionary entry into selected image region"), 
         QString::null, this, SLOT(insertDictEntry()), _toolBar, "insertDictEntry");

  _insertDictEntry->setEnabled(FALSE);

#ifndef SMAGE_DEMO
  QIconSet insertBInfoIconSet = QIconSet(QPixmap(spritzeI_xpm));
  insertBInfoIconSet.setPixmap(QPixmap(spritzeI_xpm), QIconSet::Small, QIconSet::Normal);
  insertBInfoIconSet.setPixmap(QPixmap(spritzeI_dis_xpm), QIconSet::Small, QIconSet::Disabled);

  _insertBInfo = new QToolButton(insertBInfoIconSet, tr("Insert image into selected image region"), 
				 QString::null, this, SLOT(insertBInfo()), _toolBar, "insertBInfo");
  _insertBInfo->setEnabled(FALSE);
#endif

  _toolBar->addSeparator();

  QIconSet buildImgIconSet = QIconSet(QPixmap(build_xpm));
  buildImgIconSet.setPixmap(QPixmap(build_xpm), QIconSet::Small, QIconSet::Normal);
  buildImgIconSet.setPixmap(QPixmap(build_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _buildImg = new QToolButton(buildImgIconSet, tr("Generate the coded image"), 
            QString::null, this, SLOT(buildImg()), _toolBar, "buildImg");

  _buildImg->setEnabled(FALSE);

  _vBox = new QVBox(this);
  _vBox->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
  
  _wSpace = new QWorkspace(_vBox);
  _wSpace->setScrollBarsEnabled(TRUE);

  setCentralWidget(_vBox);
}

// Open an image
void SmageAppBase::open()
{    
  doOpen();
}

// Save current image
void SmageAppBase::saveP()
{
  doSaveP();
}

// Toggle to rectangular selection
void SmageAppBase::selectRect()
{
  doSelectRect();
}

// Insert text into image
void SmageAppBase::insertTInfo()
{
  doInsertTInfo();
}

// Insert image elements into image
void SmageAppBase::insertBInfo()
{
  doInsertBInfo();
}

void SmageAppBase::insertDictEntry()
{
  doInsertDictEntry();
}


// Build coded image
void SmageAppBase::buildImg()
{
  doBuildImg();
}

//   Erase data
void SmageAppBase::erase()
{
  doErase();
}

// Show the outlines of coded image regions
void SmageAppBase::showOutlines()
{
  doShowOutlines();
}

// Select any region
void SmageAppBase::selectPoly()
{
#ifdef SMAGE_FULL
  doSelectPoly();
#endif
}

// Timer for showing start image ended
void SmageAppBase::timerDone()
{
  _startWidget->hide();
}


void SmageAppBase::about()
{
  _aboutWidget->show();
}


void SmageAppBase::showDataManager()
{
  bool b = !_viewMenu->isItemChecked( _showDataManagerId );
  _viewMenu->setItemChecked ( _showDataManagerId, b );
  doShowDataManager(b);
}

void SmageAppBase::help()
{
  _helpBrowser->show();
  _helpBrowser->setFocus();
}


void SmageAppBase::resizeEvent(QResizeEvent *)
{
  _startWidget->updateGeometry();
  _aboutWidget->updateGeometry();
}


void SmageAppBase::switchToEncoder()
{
  doSwitchToEncoder();
}


void SmageAppBase::showSecEntries()
{
  bool b = !_dictMenu->isItemChecked( 4 );
  _dictMenu->setItemChecked ( 4, b );
  doShowSecEntries(b);
}


void SmageAppBase::showCodedDictItem()
{
  bool b = !_dictMenu->isItemChecked( 3 );
  _dictMenu->setItemChecked ( 3, b );
  doShowCodedDictItem(b);
}


void SmageAppBase::translFrom(int from)
{
  doTranslFrom(from);
}


void SmageAppBase::translTo(int to)
{
  doTranslTo(to);
}

void SmageAppBase::generateHtml()
{
  doGenerateHtml();
}
