/** @defgroup SmageApp Smage Application
  *
  */

#ifndef _SMAGE_APP_BASE_H
#define _SMAGE_APP_BASE_H

// project header
#include "SmageAppViewerWidget.h"
#include "SmageImgEncodeWidget.h"
#include "SmageImgGrabber.h"
#include "SmageHTMLGenerator.h"

#include "../smageSharedFiles/SmageConfig.h"
class SmageHelpBrowser;
class SmageInfoWidget;

// qt header
#include <qcheckbox.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qworkspace.h> 
class QButtonGroup;
class QVBox;

#define THE_S_NAME "Smage"

/** Class SmageAppBase.
  *
  * @ingroup SmageApp
  *
  * Base class for the main application;
  * layout of application is specified here (menus, tool buttons, backgrounds, ...)
  */
class SmageAppBase : public QMainWindow
{
  Q_OBJECT
   
 public:

    /** Constructor.
      */
    SmageAppBase(QWidget *topLevel, int argc, char *argv[]);
    
    /** Destructor.
      */
    ~SmageAppBase();

  public slots:
    /** Opens an image.
      */
    void open();
    
    /** Saves current image.
      */
    void saveP();
    
    /** Erase data
      */
    void erase();
    
    /** Toggle to rectangular selection.
      */
    void selectRect();
    
    /** Insert text into image.
      */
    void insertTInfo();
    
    /** Insert image elements into image.
      */
    void insertBInfo();

    /** Build coded image.
      */
    void buildImg();
    
    /** Show the outlines of coded image regions
      */
    void showOutlines();
    
    /** Select any region
      */
    void selectPoly();

    void showDataManager();
    void switchToEncoder();
    void showSecEntries();
    void showCodedDictItem();
    void translFrom(int);
    void translTo(int);
    void insertDictEntry();
    void generateHtml();

    /** Timer for showing start image ended
      */
    void timerDone();
    void about();
    void help();

 protected:    
    void initialize();

    virtual void doOpen() = 0;
    virtual void doSaveP() = 0;
    virtual void doSelectRect() = 0;
    virtual void doInsertBInfo() = 0;
    virtual void doInsertTInfo() = 0;
    virtual void doInsertDictEntry() = 0;
    virtual void doBuildImg() = 0;
    virtual void doErase() = 0;
    virtual void doShowOutlines() = 0;
    virtual void doShowDataManager(const bool &) = 0;
    virtual void doSwitchToEncoder() = 0;
    virtual void doShowSecEntries(bool) = 0;
    virtual void doShowCodedDictItem(bool) = 0;
    virtual void doTranslFrom(int) = 0;
    virtual void doTranslTo(int) = 0;
    virtual void doGenerateHtml() = 0;

    virtual void resizeEvent(QResizeEvent *event);

#ifdef SMAGE_FULL
    virtual void doSelectPoly() = 0;
#endif

    QVBox *_vBox;
    QPopupMenu *_fileMenu, *_viewMenu, *_dictMenu;
    QToolBar *_toolBar;
    QWorkspace *_wSpace;
    QButtonGroup *_toggleButtonGroup, *_translFromGroup, *_translToGroup;
    QToolButton *_insertTInfo, *_insertDictEntry, *_buildImg, *_showOutlines, *_selectRect, *_erase;    

#ifdef SMAGE_FULL
    QToolButton *_selectPoly, *_insertBInfo;
#endif

    int _saveItemId, _dictMenuId, _switchToEncoderId, _showDataManagerId, _htmlGeneratorId;

    SmageInfoWidget *_startWidget;
    SmageInfoWidget *_aboutWidget;
    SmageHelpBrowser *_helpBrowser;
    SmageImgGrabber *_imgGrabber;

    SmageHTMLGenerator *_htmlGenerator;
};

#endif
