#ifndef _SMAGE_APP_VIEWER_WIDGET_H
#define _SMAGE_APP_VIEWER_WIDGET_H

// project header
#include "../smageSharedFiles/SmageViewerWidget.h"

typedef void TheS_ShowRegionsCB(void *, const bool &, const bool &);
typedef void TheS_Viewer_OpenImgCB(void *, const bool &, const bool &, const TheS_DictProps &);

/** Class SmageAppViewerWidget.
  *
  * @ingroup SmageApp
  *
  * Widget that enables a user to explore a coded image
  */
class SmageAppViewerWidget : public SmageViewerWidget
{
  Q_OBJECT

    typedef SmageViewerWidget inherited;

 public:
  
    SmageAppViewerWidget(QWidget *parent, const char* name, WFlags f);
    ~SmageAppViewerWidget();

    // Setting callback functions
    void setShowRegionsCB(void *userData, TheS_ShowRegionsCB *f)
      { _showRegionsCBFunc = f; _showRegionsUserData = userData; }
    void setOpenImgCB(void *userData, TheS_Viewer_OpenImgCB *f)
      { _openImgCBFunc = f; _openImgUserData = userData; }
    void setEraseCB(void *userData, TheS_AllowActionCB *f)
      { _eraseCBFunc = f; _eraseUserData = userData; }

    // Delete data from image
    void eraseDataFromImage();

    virtual void assignWidgetData(const SmageImgWidget *smageWidget);

    virtual QString getWidgetType() const { return QString("SMAGE_APP_VIEWER_WIDGET"); }

 protected:
    // A couple of events
    virtual void paintEvent(QPaintEvent *event);
    virtual void closeEvent(QCloseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void resizeEvent(QResizeEvent *event);

    virtual void doWhenSuccesImgLoading();

    void eraseDataPartFromImage(const QPoint &pos);

    // Set callback functions and class instances
    TheS_ShowRegionsCB *_showRegionsCBFunc;
    void *_showRegionsUserData;
    TheS_Viewer_OpenImgCB *_openImgCBFunc;
    void *_openImgUserData;
    TheS_AllowActionCB *_eraseCBFunc;
    void *_eraseUserData;

    // Reset the image widgets data
    void resetImgWidget();
       
    // Some booleans
    bool _allowErasing, _waitForClick, _hasDictEntriesEncoded; 
};

#endif
