#ifndef _SMAGE_BUBBLE_H
#define _SMAGE_BUBBLE_H

// project header
#include "SmageButton.h"
#include "../smageSharedFiles/structs/TheS_BubbleShapeData.h"

// qt header
#include <qbitmap.h>
#include <qwidget.h>

typedef void TheS_SendSignalSenderCB(void *, const char *);
typedef void TheS_SendSignalCB(void *);
typedef void TheS_BubbleMousePressCB(void *, const QMouseEvent *);
typedef void TheS_BubbleUpdateMaskCB(void *, const bool &clearMask);

/** Widget class that shows a simple editable ellipse
  *
  * @ingroup SmageApp
  *
  */
class SmageBubble : public QWidget
{
  Q_OBJECT

 public:

  SmageBubble(QWidget *parent, const char* name, const QRect &rect, const bool &hasAutoMovement,
	      const bool &isEllipse=TRUE);
  ~SmageBubble();

  void setMouseMoveCB(void *userData, TheS_SendSignalCB *f)
    { _mouseMoveCBFunc = f; _mouseMoveUserData = userData; }
  void setMouseReleaseCB(void *userData, TheS_SendSignalCB *f)
    { _mouseReleaseCBFunc = f; _mouseReleaseUserData = userData; }
  void setMousePressCB(void *userData, TheS_BubbleMousePressCB *f)
    { _mousePressCBFunc = f; _mousePressUserData = userData; }
  
  void setUpdateMaskCB(void *userData, TheS_BubbleUpdateMaskCB *f)
    { _updateMaskCBFunc = f; _updateMaskUserData = userData; }
  void setUpdateBBoxCB(void *userData, TheS_SendSignalSenderCB *f)
    { _updateBBoxCBFunc = f; _updateBBoxUserData = userData; }

  void setBubbleData(const TheS_BubbleShapeData &data);

  QBitmap getMask() { return _bubbleMask; }

  QRect getRect() { return _ellipseRect; }


  
 protected:

  void paintEvent(QPaintEvent *);
  void mousePressEvent(QMouseEvent *);
  void mouseReleaseEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);

  static void updateBBoxCB(void *userData, const QPoint &pos);
  static void buttonReleaseCB(void *userData);
  static void buttonPressCB(void *userData);

  void calcWidgetMask();
  void assignWidgetSize();
  void updateBBox(const QPoint &pos);

  TheS_SendSignalCB *_mouseMoveCBFunc;
  void *_mouseMoveUserData;
  TheS_SendSignalCB *_mouseReleaseCBFunc;
  void *_mouseReleaseUserData;
  TheS_BubbleMousePressCB *_mousePressCBFunc;
  void *_mousePressUserData;
  TheS_BubbleUpdateMaskCB *_updateMaskCBFunc;
  void *_updateMaskUserData;
  TheS_SendSignalSenderCB *_updateBBoxCBFunc;
  void *_updateBBoxUserData;

  QPoint _pressedPos;
  SmageButton *_buttonBoundingBox;

  bool _hasAutoMovement;
  
  QRect _ellipseRect;
  QBitmap _bubbleMask;

  TheS_BubbleShapeData _shapeData;

  bool _isEllipse;

};

#endif
