// project header
#include "SmageBubbleText.h"

// qt header
#include <qpainter.h>
#include <qcursor.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageBubbleText::SmageBubbleText(QWidget *parent, const char* name, 
				 const QRect &clipRect, const QString &text, const QFont &font)
  : QWidget(parent, name)
{  
  _clipRect = clipRect;

  _mouseMoveCBFunc = NULL;
  _mouseMoveUserData = NULL;
  _mouseReleaseCBFunc = NULL;
  _mouseReleaseUserData = NULL;

  _pressedPos = QPoint(0,0);
  _textData.bubbleText = QString(text.ascii());
  _textData.bubbleTextFont = QFont(font);
  _textData.bubbleTextCol = QColor(0,0,0);
  _textData.bubbleTextFlags = 0; 
  _textData.bubbleTextFlags = Qt::WordBreak | Qt::AlignHCenter;

  _textPos = QPoint(0, _textData.bubbleTextFont.pointSize());

  assignWidgetSize();

  _buttonText = new SmageButton(this, "ButtonPeak");
  _buttonText->setMouseMoveCB(this, moveTextCB);
  _buttonText->setMousePressCB(this, buttonPressCB);  
  _buttonText->setMouseReleaseCB(this, buttonReleaseCB);
  _buttonText->setAutoMovement(FALSE);
  _buttonText->setGeometry(rect().center().x() - int(_buttonText->width()/2.0), 
			       rect().center().y() - int(_buttonText->height()/2.0), 
			       _buttonText->width(), _buttonText->height());

  calcWidgetMask();
  setMask(_bubbleTextMask);
}

void SmageBubbleText::assignWidgetSize()
{
  QPainter painter(this);
  painter.setFont(_textData.bubbleTextFont);
  QRect boundingRect = painter.boundingRect(_clipRect, Qt::WordBreak | Qt::AlignLeft, _textData.bubbleText);

  resize(boundingRect.width(), boundingRect.height());
}

void SmageBubbleText::calcWidgetMask()
{
  _bubbleTextMask = QBitmap(width(), height(), TRUE);
  
  QPainter painter(&_bubbleTextMask);
  painter.setFont(_textData.bubbleTextFont);
  painter.setPen(Qt::color1);
  painter.setBrush(Qt::color1);

  QRect boundingRect = painter.boundingRect(_clipRect, Qt::WordBreak | Qt::AlignLeft, _textData.bubbleText);

  painter.drawText(boundingRect, _textData.bubbleTextFlags, _textData.bubbleText);

  painter.drawRect(_buttonText->geometry());
}

void SmageBubbleText::paintEvent(QPaintEvent *)
{
  QPainter p(this);
  p.setFont(_textData.bubbleTextFont);
  QRect boundingRect = p.boundingRect(_clipRect, Qt::WordBreak | Qt::AlignLeft, _textData.bubbleText);

  p.setPen(_textData.bubbleTextCol);
  p.drawText(boundingRect, _textData.bubbleTextFlags, _textData.bubbleText);
}

SmageBubbleText::~SmageBubbleText()
{
}

void SmageBubbleText::mousePressEvent(QMouseEvent *event)
{  
  _pressedPos = event->pos();
  clearMask();
}

void SmageBubbleText::mouseReleaseEvent(QMouseEvent *)
{ 
  calcWidgetMask();
  setMask(_bubbleTextMask);

  if (_mouseReleaseCBFunc != NULL && _mouseReleaseUserData != NULL)
	(*_mouseReleaseCBFunc)(_mouseReleaseUserData);
}

void SmageBubbleText::mouseMoveEvent(QMouseEvent *event)
{
  clearMask();

  if (_mouseMoveCBFunc != NULL && _mouseMoveUserData != NULL)
	(*_mouseMoveCBFunc)(_mouseMoveUserData);
      
  move(pos() - _pressedPos + event->pos());
      
  event->accept();
}


void SmageBubbleText::buttonReleaseCB(void *userData)
{
  SmageBubbleText *text = (SmageBubbleText *) userData;
  
  text->calcWidgetMask();
  text->setMask(text->_bubbleTextMask);  
      
  if (text->_mouseReleaseCBFunc != NULL && text->_mouseReleaseUserData != NULL)
	(*text->_mouseReleaseCBFunc)(text->_mouseReleaseUserData);

}


void SmageBubbleText::buttonPressCB(void *userData)
{
  SmageBubbleText *text = (SmageBubbleText *) userData;
  
  text->_pressedPos = text->mapFromGlobal(QCursor::pos());
  text->clearMask();
}


void SmageBubbleText::moveTextCB(void *userData, const QPoint &)
{
  SmageBubbleText *text = (SmageBubbleText *) userData;
  
  text->clearMask();
      
  text->move(text->pos() - text->_pressedPos + text->mapFromGlobal(QCursor::pos()));
      
  if (text->_mouseMoveCBFunc != NULL && text->_mouseMoveUserData != NULL)
	(*text->_mouseMoveCBFunc)(text->_mouseMoveUserData);
}


void SmageBubbleText::setTextData(const TheS_BubbleTextData &data)
{
  _textData.bubbleText = QString(data.bubbleText.ascii());
  _textData.bubbleTextFont = QFont(data.bubbleTextFont);
  _textData.bubbleTextCol = QColor(data.bubbleTextCol);
  _textData.bubbleTextFlags = data.bubbleTextFlags;

  assignWidgetSize();

  _buttonText->setGeometry(rect().center().x() - int(_buttonText->width()/2.0), 
     rect().center().y() - int(_buttonText->height()/2.0), _buttonText->width(), _buttonText->height());

  calcWidgetMask();
  setMask(_bubbleTextMask);
  repaint();
}
