#ifndef _SMAGE_BUBBLE_TEXT_H
#define _SMAGE_BUBBLE_TEXT_H

// project header
#include "SmageButton.h"
#include "SmageTextEditor.h"

// qt header
#include <qbitmap.h>
#include <qwidget.h>

typedef void TheS_BubbleTextMouseMoveCB(void *);
typedef void TheS_BubbleTextMouseReleaseCB(void *);

/** Widget class for displaying a moveable text.
  *
  * @ingroup SmageApp
  *
  */
class SmageBubbleText : public QWidget
{
  Q_OBJECT

 public:

  SmageBubbleText(QWidget *parent, const char* name, const QRect &clipRect, 
		  const QString &text, const QFont &font);
  ~SmageBubbleText();

  void setMouseMoveCB(void *userData, TheS_BubbleTextMouseMoveCB *f)
    { _mouseMoveCBFunc = f; _mouseMoveUserData = userData; }
  void setMouseReleaseCB(void *userData, TheS_BubbleTextMouseReleaseCB *f)
    { _mouseReleaseCBFunc = f; _mouseReleaseUserData = userData; }
  
  void setTextData(const TheS_BubbleTextData &data);

  void setClippingRect(const QRect &clipRect)
    { _clipRect = clipRect; }

  QBitmap getMask()
    { return _bubbleTextMask; }

 
 protected:

  void paintEvent(QPaintEvent *);
  void mousePressEvent(QMouseEvent *);
  void mouseReleaseEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);

  static void moveTextCB(void *userData, const QPoint &pos);
  static void buttonReleaseCB(void *userData);
  static void buttonPressCB(void *userData);

  void calcWidgetMask();
  void assignWidgetSize();

  TheS_BubbleTextMouseMoveCB *_mouseMoveCBFunc;
  void *_mouseMoveUserData;
  TheS_BubbleTextMouseReleaseCB *_mouseReleaseCBFunc;
  void *_mouseReleaseUserData;

  QPoint _pressedPos;
  
  SmageButton *_buttonText;
  
  QBitmap _bubbleTextMask;
  QPoint _textPos;
  
  QRect _clipRect;
  TheS_BubbleTextData _textData;

};

#endif
