// project header
#include "SmageButton.h"

// qt header
#include <qcursor.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

#include "./pixmaps/button.xpm"

SmageButton::SmageButton(QWidget *parent, const char* name,
			 const char* pixXpm[])
  : QWidget(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageButton::SmageButton()\n");
#endif
  
  _pressedPos = QPoint(0,0);
  _autoMovement = TRUE;

  _mouseMoveCBFunc = NULL;
  _mouseMoveUserData = NULL;
  _mouseReleaseCBFunc = NULL;
  _mouseReleaseUserData = NULL;
  _mousePressCBFunc = NULL;
  _mousePressUserData = NULL;

  if (pixXpm == NULL)
    _backgroundPixmap = QPixmap(button_xpm);
  else
    _backgroundPixmap = QPixmap(pixXpm);

  if (!_backgroundPixmap.isNull())
    {
      resize(_backgroundPixmap.width(), _backgroundPixmap.height());
      setPaletteBackgroundPixmap(_backgroundPixmap);
    }
}


SmageButton::~SmageButton()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageButton::~SmageButton()\n");
#endif
}


void SmageButton::mousePressEvent(QMouseEvent *event)
{
  _pressedPos = mapFromGlobal(QCursor::pos());

  if (_mousePressCBFunc != NULL && _mousePressUserData != NULL)
    (*_mousePressCBFunc)(_mousePressUserData);

  event->accept();
}

void SmageButton::mouseMoveEvent(QMouseEvent *event)
{
  QPoint newPos(pos() - _pressedPos + mapFromGlobal(QCursor::pos()));
  
  if (_autoMovement)
    move(newPos);

  if (_mouseMoveCBFunc != NULL && _mouseMoveUserData != NULL)
    (*_mouseMoveCBFunc)(_mouseMoveUserData, newPos + QPoint(int(width()/2.0), int(height()/2.0)));

  event->accept();
}


void SmageButton::mouseReleaseEvent(QMouseEvent *)
{
  if (_mouseReleaseCBFunc != NULL && _mouseReleaseUserData != NULL)
    (*_mouseReleaseCBFunc)(_mouseReleaseUserData);
}
