#ifndef _SMAGE_BUTTON_H
#define _SMAGE_BUTTON_H

// qt header
#include <qpixmap.h>
#include <qwidget.h>

typedef void TheS_ButtonMouseMoveCB(void *, const QPoint &);
typedef void TheS_ButtonMouseReleaseCB(void *);
typedef void TheS_ButtonMousePressCB(void *);

/** Widget class that provides a button that can be used for interaction.
  *
  * @ingroup SmageApp
  *
  * Callback functions can be defined for feedback actions.
  */
class SmageButton : public QWidget
{
  Q_OBJECT

 public:

  SmageButton(QWidget *parent, const char* name, const char* pixXpm[] = NULL);
  ~SmageButton();

  void setMouseMoveCB(void *userData, TheS_ButtonMouseMoveCB *f)
    { _mouseMoveCBFunc = f; _mouseMoveUserData = userData; }
  void setMouseReleaseCB(void *userData, TheS_ButtonMouseReleaseCB *f)
    { _mouseReleaseCBFunc = f; _mouseReleaseUserData = userData; }
  void setMousePressCB(void *userData, TheS_ButtonMousePressCB *f)
    { _mousePressCBFunc = f; _mousePressUserData = userData; }

  void setAutoMovement(const bool &autoM)
    { _autoMovement = autoM; }

 protected:

  void mousePressEvent(QMouseEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);

 private:

  TheS_ButtonMouseMoveCB *_mouseMoveCBFunc;
  void *_mouseMoveUserData;
  TheS_ButtonMouseReleaseCB *_mouseReleaseCBFunc;
  void *_mouseReleaseUserData;
  TheS_ButtonMousePressCB *_mousePressCBFunc;
  void *_mousePressUserData;
  
  QPixmap _backgroundPixmap;
  QPoint _pressedPos;
  bool _autoMovement;
};

#endif
