#ifndef _SMAGE_DATA_MANAGE_WIDGET_H
#define _SMAGE_DATA_MANAGE_WIDGET_H

// project header
#include "../smageSharedFiles/SmageConfig.h"

// qt header
#include <qwidget.h>
class QCheckBox;
class QGridLayout;
class QLabel;
class QPushButton;

// c/c++ header
#include <map>

// int (1): view (0: disabled, 1: enabled) or delete (2) button pressed
// int (2): the "SmageDataManageWidgetItem"'s ID
typedef void TheS_DataManagerItemCB(void *, const int &, const int &);

typedef void TheS_DataManagerFocusedItemCB(void *, const int &);

/**
  *
  * @ingroup SmageApp
  *
  */

class SmageDataManageWidgetItem : public QWidget
{ 
  Q_OBJECT

  public:
  
    SmageDataManageWidgetItem(QWidget *parent, const char *name, const int &widgetItemID, const QString &itemName);
    ~SmageDataManageWidgetItem();

    void setDataManagerItemCB(void *userData, TheS_DataManagerItemCB *f)
      { _dataManagerItemCBFunc = f; _dataManagerItemUserData = userData; }

    void setDataManagerFocusItemCB(void *userData, TheS_DataManagerFocusedItemCB *f)
      { _dataManagerFocusItemCBFunc = f; _dataManagerFocusItemUserData = userData; }

    void resetItem();
    
 public slots:
    void viewPressed();
    void delPressed();

 protected:

    void mousePressEvent(QMouseEvent *event);

 private:
    
    TheS_DataManagerItemCB *_dataManagerItemCBFunc;
    TheS_DataManagerFocusedItemCB *_dataManagerFocusItemCBFunc;
    void *_dataManagerItemUserData, *_dataManagerFocusItemUserData;

    QGridLayout *_widgetItemGridLayout;
    QCheckBox *_widgetItemCheckBox;
    QLabel *_widgetItemLabel;
    QPushButton *_widgetItemDelButton;

    int _widgetItemID;
};

/** Class SmageDataManageWidget: widget for managing the data to be coded.
  *
  * @ingroup SmageApp
  *
  */
class SmageDataManageWidget : public QWidget
{ 
  Q_OBJECT

  public:
  
    SmageDataManageWidget(QWidget *parent, const char* name);
    ~SmageDataManageWidget();

    void addItem(const QString &name, const int &id);
    void removeItem(const int &id);

    void setDataManagerWidgetCB(void *userData, TheS_DataManagerItemCB *f)
      { _dataManagerWidgetCBFunc = f; _dataManagerWidgetUserData = userData; }

    void setDataManagerFocusCB(void *userData, TheS_DataManagerFocusedItemCB *f)
      { _dataManagerFocusCBFunc = f; _dataManagerFocusUserData = userData; }

    void itemFocused(const int &whichItem);

 protected:

    void closeEvent(QCloseEvent *event);

 private:

    static void itemButtonPressedCB(void *userData, const int &whichButton, const int &itemID);
    static void itemFocusedCB(void *userData, const int &whichItem);

    TheS_DataManagerItemCB *_dataManagerWidgetCBFunc;
    TheS_DataManagerFocusedItemCB *_dataManagerFocusCBFunc;
    void *_dataManagerWidgetUserData, *_dataManagerFocusUserData;

    QGridLayout *_manageWidgetGridLayout;

    std::map<int, SmageDataManageWidgetItem *> _itemMap;

    int _focusedItemID;
};

#endif
