// project header
#include "SmageDictEditor.h"
#include "../smageSharedFiles/SmageXmlLanguageFileParser.h"

// project header
#include <qgroupbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qpushbutton.h>

extern std::vector<TheS_Language> __languagesVec;


SmageDictEditor::SmageDictEditor(QWidget *parent, const char* name, const char *xmlFileName)
  : QDialog(parent, name)
{
  setCaption(tr("Dictionary Entries"));

  _xmlHandler = new SmageXmlHandler(xmlFileName);

  initSubWidgets();
}

SmageDictEditor::~SmageDictEditor()
{
  delete _xmlHandler;
}

void SmageDictEditor::initSubWidgets()
{
  int lineWidth = 200;

  QGroupBox *group = new QGroupBox(2, Qt::Horizontal, this);
  
  QLabel *eng = new QLabel(group);
  eng->setText(tr("English:"));
  _lineEng = new QLineEdit(group);
  _lineEng->setFixedWidth(lineWidth);
  _lineEng->setText(tr("obligatory"));
  _lineEng->selectAll();

  std::vector<TheS_Language>::iterator langIter=__languagesVec.begin();
  for ( langIter++; langIter != __languagesVec.end(); ++langIter)
  {
    QLabel *label = new QLabel(group);
    label->setText((*langIter)._name + ":");
    QLineEdit *lineEdit = new QLineEdit(group);
    lineEdit->setFixedWidth(lineWidth);
    _lineEdits.push_back( lineEdit );
  }
  
  QPushButton *okButton = new QPushButton(QObject::tr("OK"), this);
  connect(okButton, SIGNAL(clicked()), this, SLOT(okPressed()));
  okButton->setFixedSize(100, 40);
  okButton->setDefault(TRUE);

  QPushButton *cancelButton = new QPushButton(QObject::tr("Cancel"), this);
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelPressed()));
  cancelButton->setFixedSize(100, 40);

  QGridLayout *gridLayout = new QGridLayout(this, 3, 4, 20);
  gridLayout->addMultiCellWidget(group, 0, 1, 0, 3, Qt::AlignHCenter);
  gridLayout->addWidget(okButton, 2, 1, Qt::AlignRight);
  gridLayout->addWidget(cancelButton, 2, 2, Qt::AlignLeft);

}

void SmageDictEditor::okPressed()
{
  if (appendEntry())
    {
      _xmlHandler->saveCurDoc("smageDictPriv.xml");
      accept();
    }
  else
    QMessageBox::warning(this, QObject::tr("Smage - Message"), tr("The English term is obligatory ..."),
			 QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
}

void SmageDictEditor::cancelPressed()
{
  reject();
}


bool SmageDictEditor::appendEntry()
{
  if (!_lineEng->text().isEmpty() && _lineEng->text() != tr("obligatory"))
  {
    TheS_DictEntries entry;
    DictIdEntriesMap map;

    entry.primeWord = _lineEng->text();
    DictIdEntryPair pair(0, entry);
    map.insert(pair);

    std::vector<QLineEdit*>::iterator lineIter;
    int i;
    for( lineIter=_lineEdits.begin(), i=1; lineIter != _lineEdits.end(); lineIter++, i++ )
    {
      if( !((*lineIter)->text().isEmpty()))
      {
        entry.primeWord = (*lineIter)->text();
        pair = DictIdEntryPair (i, entry);
        map.insert(pair);
      }
    }
    if (_xmlHandler->appendEntry(map, _lineEng->text(), -2) != NULL)
      return TRUE;
  }
  return FALSE;
}


bool SmageDictEditor::getNewDictEntries(const char *xmlFileName, QString &idString, QString &entry, 
					int &language, QWidget *parent)
{
  bool returnValue = FALSE;
  SmageDictEditor *dlg = new SmageDictEditor(parent, "SmageDictEditor", xmlFileName);
  
  if (dlg->exec() == QDialog::Accepted)
  {
    idString = dlg->_lineEng->text().copy();
    if (!dlg->getFirstEntry(entry, language))
	  {
	    language = 0;
	    entry = dlg->_lineEng->text().copy();
	  }
    returnValue = TRUE;
  }
  
  delete dlg;
  return returnValue;
}


bool SmageDictEditor::getFirstEntry(QString &entry, int &language)
{
  std::vector<QLineEdit*>::iterator lineIter;
  int i;
  for( lineIter=_lineEdits.begin(), i=1; lineIter != _lineEdits.end(); lineIter++, i++ )
  {
    if( !((*lineIter)->text().isEmpty()))
    {
      entry = (*lineIter)->text().copy();
      language = i;
      return TRUE;
    }
  }
  return FALSE;
}
