// project header
#include "SmageDictEntryDlg.h"
#include "../smageSharedFiles/SmageXmlParser.h"
#include "../smageSharedFiles/SmageXmlLanguageFileParser.h"

// qt header
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>

// c/c++ header
#include <iostream>
#include <stdlib.h>

extern std::vector<TheS_Language> __languagesVec;

using namespace std;

SmageDictEntryDlg::SmageDictEntryDlg(QWidget *topLevel)
  : QDialog(topLevel, 0)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageDictEntryDlg::SmageDictEntryDlg()\n");
#endif

  //setPaletteBackgroundColor(QColor(232,227,227));
  _language = 0;

  setCaption( tr("Image Dictionary Entry"));
  
  QGroupBox *group = new QGroupBox(1, Qt::Horizontal, this);
  group->setInsideSpacing(20);

  QLabel *taskLabel = new QLabel(group);
  taskLabel->setText( tr("Choose a language and enter the\n term in the selected language ..."));

  _langsCombo = new QComboBox(group);
  
  std::vector<TheS_Language>::iterator langIter;
  int i;
  for ( langIter=__languagesVec.begin(), i=0; langIter != __languagesVec.end(); ++langIter, ++i)
  {
    _langsCombo->insertItem((*langIter)._name, i);
  }
  _langsCombo->setCurrentItem(0);
  connect(_langsCombo, SIGNAL(activated(int)), SLOT(chosenLang(int)));
  _langsCombo->setFixedWidth(200);

  _enterTerm = new QLineEdit(group);
  _enterTerm->setFixedWidth(200);

  // OK / Cancel button
  
  QPushButton *okButton = new QPushButton( QObject::tr("OK"), this);
  connect(okButton, SIGNAL(clicked()), this, SLOT(okPressed()));
  okButton->setFixedSize(100, 40);
  okButton->setDefault(TRUE);

  QPushButton *cancelButton = new QPushButton(QObject::tr("Cancel"), this);
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelPressed()));
  cancelButton->setFixedSize(100, 40);

  QGridLayout *gridLayout = new QGridLayout(this, 2, 2, 20);
  gridLayout->addMultiCellWidget(group, 0, 0, 0, 1, Qt::AlignHCenter);
  gridLayout->addWidget(okButton, 1, 0, Qt::AlignRight);
  gridLayout->addWidget(cancelButton, 1, 1, Qt::AlignLeft);
}


SmageDictEntryDlg::~SmageDictEntryDlg() 
{   
}


void SmageDictEntryDlg::okPressed()
{
  accept();
}


void SmageDictEntryDlg::cancelPressed()
{
  reject();
}

bool SmageDictEntryDlg::getDictEntry(int &language, QString &entry, QWidget *parent)
{
  bool returnValue = FALSE;

  SmageDictEntryDlg *dlg = new SmageDictEntryDlg(parent);

  if (!entry.isEmpty())
    dlg->_enterTerm->setText(entry);

  if (language != -1)
    {
      dlg->_language = language;
      dlg->setLang(language);
    }

  if (dlg->exec() == QDialog::Accepted)
  {
    language = dlg->_language;
    entry = dlg->_enterTerm->text();
    returnValue = TRUE;
  }
  
  delete dlg;
  return returnValue;
}

void SmageDictEntryDlg::chosenLang(int lang)
{
  _language = lang;
}

void SmageDictEntryDlg::setLang(const int &lang)
{
  _langsCombo->setCurrentItem(lang);
}
