// project header
#include "SmageEditorShapedText.h"
#include "SmageImgEncodeWidget.h"

SmageEditorShapedText::SmageEditorShapedText(QWidget *parent, const char* name)
  : SmageShapedTextBase(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorShapedText::SmageEditorShapedText()\n");
#endif  

  _bubbleEditor = NULL;
}

SmageEditorShapedText::~SmageEditorShapedText()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorShapedText::~SmageEditorShapedText()\n");
#endif  

  if (_bubbleEditor != NULL)
    delete _bubbleEditor;
}

void
SmageEditorShapedText::mousePressEvent(QMouseEvent *event)
{  
  if (event->button() == Qt::RightButton)
  {
     _contextMenu->exec(QCursor::pos());
  }
  else
   _pressedPos = event->pos();
}

void
SmageEditorShapedText::mouseReleaseEvent(QMouseEvent *)
{  
  setMask(_bubbleMask);
}

void
SmageEditorShapedText::mouseMoveEvent(QMouseEvent *event)
{
  clearMask();      
  move(pos() - _pressedPos + mapFromGlobal(QCursor::pos()));

  event->accept();
}

void
SmageEditorShapedText::updateText(const TheS_BubbleData &bubbleData)
{
  _bubbleText->setTextData(bubbleData.textData);
  moveTextToCenter();

  calcWidgetMask();
  setMask(_bubbleMask);
}

void
SmageEditorShapedText::initContextMenu()
{
  _contextMenu = new QPopupMenu(this);
  Q_CHECK_PTR(_contextMenu);

  _contextMenu->insertItem(tr("Change properties"), this, SLOT(changeProps()));
  _contextMenu->insertItem(tr("Center text"), this, SLOT(moveTextToCenter()));
  _contextMenu->insertItem(tr("Remove this element"), this, SLOT(deleteElement()));
}

void
SmageEditorShapedText::changeProps()
{
  if (_bubbleEditor == NULL) return;

  _bubbleEditor->assignNewProps(_newBubbleData);
  _bubbleEditor->show();
}

void
SmageEditorShapedText::deleteElement()
{  
  if (_bubbleEditor != NULL)
    {
      delete _bubbleEditor;
      _bubbleEditor = NULL;

      //_bubbleEditor->close();
    }

  SmageImgEncodeWidget* siew = (SmageImgEncodeWidget*)parentWidget();
  siew->deleteDataItem( this );
}
