// project header
#include "SmageBubble.h"
#include "SmageEditorSimpleBubble.h"
#include "../smageSharedFiles/SmageImgWidget.h"

// qt header
#include <qcursor.h>
#include <qpainter.h>
#include <qpopupmenu.h>

// c/c++ header
#include <iostream>
#include <assert.h>

using namespace std;

SmageEditorSimpleBubble::SmageEditorSimpleBubble(QWidget *parent, const char* name, const TextShapeType &type)
  : SmageEditorShapedText(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorSimpleBubble::SmageEditorSimpleBubble()\n");
#endif

  _bubbleType = type;

  _prevBubbleData.textData.bubbleText = QString(tr("Click right mouse\nbutton to edit"));
  _prevBubbleData.textData.bubbleTextFont = QFont("Helvetica", 12, -1);
  _prevBubbleData.textData.bubbleTextCol = QColor(0,0,0);
  _prevBubbleData.shapeData.bubbleBackCol = QColor(255,255,255);
  _prevBubbleData.shapeData.bubbleOutlineCol = QColor(0,0,0);
  _prevBubbleData.shapeData.bubbleOutlineWidth = 1;
  _prevBubbleData.textData.bubbleTextFlags |= Qt::AlignCenter;
  _newBubbleData = _prevBubbleData;

  initClassData();
  initBubble(QRect(0,0,150,70));
  assignWidgetSize();

  // Init the main bubble's text  
  _bubbleText = new SmageBubbleText(this, "BubbleText", rect(), _newBubbleData.textData.bubbleText, 
				    _newBubbleData.textData.bubbleTextFont);

  _bubbleText->setMouseMoveCB(this, bubbleTextMoveCB);
  _bubbleText->setMouseReleaseCB(this, bubbleTextMouseReleaseCB);
  moveTextToCenter();
  _bubbleText->show();

  calcWidgetMask();
  setMask(_bubbleMask);
}


void SmageEditorSimpleBubble::initBubble(const QRect &rect)
{
  _bubble = new SmageBubble(this, "MainBubble", rect, FALSE, 
			    (_bubbleType == SMAGE_ELLIPSE));
  _bubble->setMouseMoveCB(this, bubbleMoveCB);
  _bubble->setMouseReleaseCB(this, bubbleReleaseCB);
  _bubble->setMousePressCB(this, bubblePressCB);
  _bubble->setUpdateMaskCB(this, updateMaskCB);  
  _bubble->setUpdateBBoxCB(this, updateBBoxCB);

  updateBubble(_newBubbleData);

  _bubble->show();
}

void SmageEditorSimpleBubble::initClassData()
{
  _pressedPos = QPoint(0,0);

  _bubble = NULL;
  _bubbleText = NULL;

  initContextMenu();

  _bubbleEditor = new SmageTextEditor(0, "Editor", _newBubbleData.textData.bubbleText, _newBubbleData.textData.bubbleTextFont);
  _bubbleEditor->setPrevCB(this, previewCB);
  _bubbleEditor->setOkCB(this, okCB);
  _bubbleEditor->setCancelCB(this, cancelCB);
  _bubbleEditor->hide();
}

SmageEditorSimpleBubble::~SmageEditorSimpleBubble()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorSimpleBubble::~SmageEditorSimpleBubble()\n");
#endif
}

void SmageEditorSimpleBubble::assignWidgetSize()
{
  int top = geometry().top();
  int left = geometry().left();
 
  QRegion region(_bubble->rect());
  setGeometry(left, top, region.boundingRect().width(), region.boundingRect().height());
}

void SmageEditorSimpleBubble::calcWidgetMask()
{
  _bubbleMask = QBitmap(width(), height(), TRUE);
  
  QPainter painter(&_bubbleMask);
  painter.setPen(Qt::color1);
  painter.setBrush(Qt::color1);
  QBitmap srcBitmap = _bubbleText->getMask();

  bitBlt(&_bubbleMask, _bubbleText->geometry().topLeft().x(), _bubbleText->geometry().topLeft().y(), 
	 &srcBitmap, 0, 0, _bubbleText->width(), _bubbleText->height(), Qt::OrROP, TRUE);

  if (_bubble != NULL)
    {
      QBitmap srcBitmap = _bubble->getMask();
      bitBlt(&_bubbleMask, _bubble->geometry().topLeft().x(), _bubble->geometry().topLeft().y(), 
	     &srcBitmap, 0, 0, _bubble->width(), _bubble->height(), Qt::OrROP, TRUE);
    }
}



void SmageEditorSimpleBubble::bubbleTextMoveCB(void *userData)
{
  SmageEditorSimpleBubble *db = (SmageEditorSimpleBubble *) userData;

  db->clearMask();
}

void SmageEditorSimpleBubble::bubbleTextMouseReleaseCB(void *userData)
{
  SmageEditorSimpleBubble *db = (SmageEditorSimpleBubble *) userData;

  db->calcWidgetMask();
  db->setMask(db->_bubbleMask);
}

void SmageEditorSimpleBubble::updateBubble(const TheS_BubbleData &bubbleData)
{
  _bubble->setBubbleData(bubbleData.shapeData);
}

std::vector<char>  SmageEditorSimpleBubble::getBubbleData(const QPoint &posOffset)
{
  SmageEllipseShape ellipseShape;

  // The comic bubble object
  SmageEllipseObj ellipseObj;

  ellipseObj.textObjData.position = pos() - posOffset;
  SmageObjShape::intToChar(_newBubbleData.shapeData.bubbleOutlineWidth, ellipseObj.textObjData.lineWidth);
  ellipseObj.textObjData.textPosition = _bubbleText->pos();
  ellipseObj.textObjData.shapeType = _bubbleType;
  ellipseObj.textObjData.textData.text = _newBubbleData.textData.bubbleText;
  ellipseObj.textObjData.textData.textFont = _newBubbleData.textData.bubbleTextFont;

  if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignLeft) == Qt::AlignLeft)
    ellipseObj.textObjData.textData.textAlignment = 1;
  else if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignHCenter) == Qt::AlignHCenter)
    ellipseObj.textObjData.textData.textAlignment = 2;
  else if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignRight) == Qt::AlignRight)
    ellipseObj.textObjData.textData.textAlignment = 3;

  ellipseObj.textObjData.colors.textColor = _newBubbleData.textData.bubbleTextCol;
  ellipseObj.textObjData.colors.outlineColor = _newBubbleData.shapeData.bubbleOutlineCol;
  ellipseObj.textObjData.colors.backgroundColor = _newBubbleData.shapeData.bubbleBackCol;
  ellipseObj.textObjData.colors.backgroundAlpha = _newBubbleData.shapeData.bubbleBackgroundAlpha;
  ellipseObj.ellipseShapeRect = _bubble->getRect();

  ellipseShape.setEllipseObject(&ellipseObj);  
  ellipseShape.generateEllipseShapeString();

  return ellipseShape.getCharVector();
}

void SmageEditorSimpleBubble::bubbleMoveCB(void *userData)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  if (userData == NULL)
    return;

  bubble->clearMask();      
  bubble->move(bubble->pos() - bubble->_pressedPos + bubble->mapFromGlobal(QCursor::pos()));
}

void SmageEditorSimpleBubble::bubbleReleaseCB(void *userData)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  if (userData == NULL)
    return;

  bubble->setMask(bubble->_bubbleMask);
}

void SmageEditorSimpleBubble::bubblePressCB(void *userData, const QMouseEvent *event)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  if (userData == NULL)
    return;

  if (event->button() == Qt::RightButton)
	{
	    bubble->_contextMenu->exec(QCursor::pos());
	}
  else
  	bubble->_pressedPos = bubble->mapFromGlobal(QCursor::pos());
}


void SmageEditorSimpleBubble::updateMaskCB(void *userData, const bool &clearMask)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  if (userData == NULL)
    return;

  if (clearMask)
    bubble->clearMask();
  else
    {
      bubble->calcWidgetMask();
      bubble->setMask(bubble->_bubbleMask); 
    }
}

void SmageEditorSimpleBubble::updateBBoxCB(void *userData, const char *)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  if (userData == NULL)
    return;

  bubble->handleCrossingBorder();
  bubble->clearMask();
  bubble->moveTextToCenter();
}

void SmageEditorSimpleBubble::handleCrossingBorder()
{
  int left = geometry().left();
  int top = geometry().top();
 
  QRegion region(_bubble->geometry());
  setGeometry(left, top, region.boundingRect().width(), region.boundingRect().height());
}


void SmageEditorSimpleBubble::previewCB(void *userData, const TheS_BubbleData &bubbleData)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  bubble->_newBubbleData = bubbleData;

  bubble->updateText(bubbleData);
  bubble->updateBubble(bubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorSimpleBubble::okCB(void *userData, const TheS_BubbleData &bubbleData)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  bubble->_prevBubbleData = bubbleData;
  bubble->_newBubbleData = bubbleData;

  bubble->updateText(bubbleData);
  bubble->updateBubble(bubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorSimpleBubble::cancelCB(void *userData)
{
  SmageEditorSimpleBubble *bubble = (SmageEditorSimpleBubble *) userData;
  bubble->_newBubbleData = bubble->_prevBubbleData;

  bubble->updateText(bubble->_prevBubbleData);
  bubble->updateBubble(bubble->_prevBubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorSimpleBubble::moveTextToCenter()
{
  QRect textRect = _bubbleText->geometry();
  textRect.moveCenter(_bubble->geometry().center());
  _bubbleText->move(textRect.topLeft());  
}
