// project header
#include "SmageEditorSpeechBubble.h"

// qt header
#include <qcursor.h>
#include <qpainter.h>
#include <qpopupmenu.h>

// c/c++ header
#include <assert.h>
#include <iostream>
#include <math.h>

#ifndef M_PI
 #define M_PI 3.14159265358979323846
#endif

using namespace std;

#include "./pixmaps/buttonBig.xpm"

SmageEditorSpeechBubble::SmageEditorSpeechBubble(QWidget *parent, const char* name)
  : SmageEditorShapedText(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorSpeechBubble::SmageEditorSpeechBubble()\n");
#endif
  
  _pressedPos = QPoint(0,0);
  _bubblePeakPos = QPoint(60,90); 

  _ellipseRect = QRect(0,0, 150,70);
  calcRefAngles(_ellipseRect);

  calcAngle(QPoint(90, 70), _openAngle, FALSE);
  calcAngle(QPoint(65, 70), _closeAngle, FALSE);
  calcAngle(_bubblePeakPos, _peakAngle, FALSE);

  _buttonPeak = NULL;
  _buttonAngle1 = NULL;
  _buttonAngle2 = NULL;
  _buttonBoundingBox = NULL;

  calcBubblePoints();

  _vecBBox = _bubblePoints.boundingRect().bottomRight() - _bubblePoints.boundingRect().topLeft();
  _vecPeak = _bubblePeakPos - _bubblePoints.boundingRect().topLeft();
  _bBoxDiff = QPoint(0,0);

  _contextMenu = NULL;

  initEditButtons();
  initContextMenu();

  assignWidgetSize();

  _prevBubbleData.textData.bubbleText = QString(tr("Click right mouse\nbutton to edit"));
  _prevBubbleData.textData.bubbleTextFont = QFont("Helvetica", 12, -1);
  _prevBubbleData.textData.bubbleTextCol = QColor(0,0,0);
  _prevBubbleData.shapeData.bubbleBackCol = QColor(255,255,255);
  _prevBubbleData.shapeData.bubbleOutlineCol = QColor(0,0,0);
  _prevBubbleData.shapeData.bubbleOutlineWidth = 1;
  _prevBubbleData.textData.bubbleTextFlags |= Qt::AlignCenter;
  _newBubbleData = _prevBubbleData;

  _bubbleText = new SmageBubbleText(this, "BubbleText", rect(), _newBubbleData.textData.bubbleText, 
				    _newBubbleData.textData.bubbleTextFont);

  _bubbleText->setMouseMoveCB(this, bubbleTextMoveCB);
  _bubbleText->setMouseReleaseCB(this, bubbleTextMouseReleaseCB);

  moveTextToCenter();
  _bubbleText->show();

  _bubbleEditor = new SmageTextEditor(0, "Editor", _newBubbleData.textData.bubbleText, _newBubbleData.textData.bubbleTextFont);
  _bubbleEditor->setPrevCB(this, previewCB);
  _bubbleEditor->setOkCB(this, okCB);
  _bubbleEditor->setCancelCB(this, cancelCB);
  _bubbleEditor->hide();

  calcWidgetMask();
  setMask(_bubbleMask);
}

SmageEditorSpeechBubble::~SmageEditorSpeechBubble()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorSpeechBubble::~SmageEditorSpeechBubble()\n");
#endif
}

void SmageEditorSpeechBubble::initEditButtons()
{
  _buttonPeak = new SmageButton(this, "ButtonPeak", buttonBig_xpm);
  _buttonPeak->setMouseMoveCB(this, moveBubblePeakCB);
  _buttonPeak->setMouseReleaseCB(this, buttonReleaseCB);
  _buttonPeak->setMousePressCB(this, buttonPressCB);
  _buttonPeak->setGeometry(_bubblePeakPos.x() - int(_buttonPeak->width()/2.0), _bubblePeakPos.y() - int(_buttonPeak->height()/2.0), 
			   _buttonPeak->width(), _buttonPeak->height());

  if (_bubblePoints.size() > 2)
    {
      QPoint p1 = _bubblePoints.point(0);
      QPoint p2 = _bubblePoints.point(_bubblePoints.count()-2);
	
      _buttonAngle1 = new SmageButton(this, "ButtonAngle1");
      _buttonAngle1->setAutoMovement(FALSE);
      _buttonAngle1->setMouseMoveCB(this, moveBubbleAngle1CB);
      _buttonAngle1->setMouseReleaseCB(this, buttonReleaseCB);
      _buttonAngle1->setMousePressCB(this, buttonPressCB);
      _buttonAngle1->setGeometry(p1.x() - int(_buttonAngle1->width()/2.0), p1.y() - int(_buttonAngle1->height()/2.0), 
				 _buttonAngle1->width(), _buttonAngle1->height());

      _buttonAngle2 = new SmageButton(this, "ButtonAngle2");
      _buttonAngle2->setAutoMovement(FALSE);
      _buttonAngle2->setMouseMoveCB(this, moveBubbleAngle2CB);
      _buttonAngle2->setMouseReleaseCB(this, buttonReleaseCB);
      _buttonAngle2->setMousePressCB(this, buttonPressCB);
      _buttonAngle2->setGeometry(p2.x() - int(_buttonAngle2->width()/2.0), p2.y() - int(_buttonAngle2->height()/2.0), 
				 _buttonAngle2->width(), _buttonAngle2->height());

      QPoint p3 = _bubblePoints.boundingRect().bottomRight();
      
      _buttonBoundingBox = new SmageButton(this, "ButtonBBox");
      _buttonBoundingBox->setAutoMovement(FALSE);
      _buttonBoundingBox->setMouseReleaseCB(this, buttonReleaseCB);
      _buttonBoundingBox->setMousePressCB(this, buttonPressCB);
      _buttonBoundingBox->setMouseMoveCB(this, updateBBoxCB);
      _buttonBoundingBox->setGeometry(p3.x() - int(_buttonBoundingBox->width()/2.0), p3.y() - int(_buttonBoundingBox->height()/2.0), 
				      _buttonBoundingBox->width(), _buttonBoundingBox->height());
    }
}


void SmageEditorSpeechBubble::assignWidgetSize()
{
  QRect boundingBox = _bubblePoints.boundingRect();
  
  int top = geometry().top();
  int left = geometry().left();
 
  int buttonWidth = int(ceil(double(_buttonBoundingBox->width()/2.0)));
  int buttonHeight = int(ceil(double(_buttonBoundingBox->height()/2.0)));

  setGeometry(left, top, boundingBox.width() + buttonWidth, boundingBox.height() + buttonHeight);
}

void SmageEditorSpeechBubble::calcWidgetMask()
{
  _bubbleMask = QBitmap(width(), height(), TRUE);
  
  QPainter painter(&_bubbleMask);
  painter.setPen(Qt::color1);
  painter.setBrush(Qt::color1);
  painter.drawPolygon(_bubblePoints);
  
  painter.drawRect(_buttonPeak->geometry());
  painter.drawRect(_buttonAngle1->geometry());
  painter.drawRect(_buttonAngle2->geometry());
  painter.drawRect(_buttonBoundingBox->geometry());
  
  painter.setBrush(Qt::NoBrush);
  painter.drawRect(_bubblePoints.boundingRect());

  QBitmap srcBitmap = _bubbleText->getMask();
  bitBlt(&_bubbleMask, _bubbleText->geometry().topLeft().x(), _bubbleText->geometry().topLeft().y(), 
	 &srcBitmap, 0, 0, _bubbleText->width(), _bubbleText->height(), Qt::OrROP, TRUE);
}

void SmageEditorSpeechBubble::getNormVec(const QPoint &vec, double &vecNormX, double &vecNormY)
{
  double length = sqrt(double(vec.x()*vec.x()) + double(vec.y()*vec.y()));
  vecNormX = double(vec.x()) / length;
  vecNormY = double(vec.y()) / length; 
}


bool SmageEditorSpeechBubble::getRectPoint(const double &angle, QPoint &rectPoint)
{
  // Return TRUE: to get a square, change rectangle's height
  // Return FALSE: to get a square, change rectangle's width

  int height, width;

  if (angle > _refAngle_1 && angle <= _refAngle_2)
    {  
      rectPoint.setY(_ellipseRect.top());

      if (angle <= 90)
	{
	  width = int(_ellipseRect.height()/2.0 * tan((90 - angle) * M_PI / 180));
	  rectPoint.setX(_ellipseRect.center().x() + width);
	}
      else
	{
	  width = int(_ellipseRect.height()/2.0 * tan((angle - 90) * M_PI / 180));
	  rectPoint.setX(_ellipseRect.center().x() - width);
	}

      return FALSE;
    }
  else if (angle > _refAngle_2 && angle <= _refAngle_3)
    {
      rectPoint.setX(_ellipseRect.left());

      if (angle <= 180)
	{
	  height = int(_ellipseRect.width()/2.0 * tan((180 - angle) * M_PI / 180));
	  rectPoint.setY(_ellipseRect.center().y() - height);
	}
      else
	{
	  height = int(_ellipseRect.width()/2.0 * tan((angle - 180) * M_PI / 180));
	  rectPoint.setY(_ellipseRect.center().y() + height);
	}

      return TRUE;
    }
  else if (angle > _refAngle_3 && angle <= _refAngle_4)
    {
      rectPoint.setY(_ellipseRect.bottom());

      if (angle <= 270)
	{
	  width = int(_ellipseRect.height()/2.0 * tan((270 - angle) * M_PI / 180));
	  rectPoint.setX(_ellipseRect.center().x() - width);
	}
      else
	{
	  width = int(_ellipseRect.height()/2.0 * tan((angle - 270) * M_PI / 180));
	  rectPoint.setX(_ellipseRect.center().x() + width);
	}

      return FALSE;
    }
  else
    {
      rectPoint.setX(_ellipseRect.right());

      if (angle <= 90)
	{
	  height = int(_ellipseRect.width()/2.0 * tan(angle * M_PI / 180));
	  rectPoint.setY(_ellipseRect.center().y() - height);
	}
      else
	{
	  height = int(_ellipseRect.width()/2.0 * tan((360 - angle) * M_PI / 180));
	  rectPoint.setY(_ellipseRect.center().y() + height);
	}

      return TRUE;
    }
}


void SmageEditorSpeechBubble::getRect2SquarePoint(const bool &changeWidth, const QPoint &rectPoint,
				     QPoint &scuarePoint)
{
  scuarePoint = rectPoint;
  int diff = _ellipseRect.width() - _ellipseRect.height();
  if (changeWidth)
    {
      if (rectPoint.x() < _ellipseRect.center().x())
	scuarePoint.setX(int(rectPoint.x() + diff/2.0));
      else
	scuarePoint.setX(int(rectPoint.x() - diff/2.0));
    }
  else
    {
      if (rectPoint.y() < _ellipseRect.center().y())
	scuarePoint.setY(int(rectPoint.y() - diff/2.0));
      else
	scuarePoint.setY(int(rectPoint.y() + diff/2.0));
    }
}


void SmageEditorSpeechBubble::calcAngle(const QPoint &point, double &newAngle, const bool &updateAngles)
{
  double openAngleDiff = 0;
  double closeAngleDiff = 0;
    
  if (updateAngles)
    {
      if (_openAngle < _peakAngle)
	openAngleDiff = 360 - _peakAngle + _openAngle;
      else
	openAngleDiff = _openAngle - _peakAngle;
      
      if (_closeAngle < _peakAngle)
	closeAngleDiff = _peakAngle - _closeAngle;
      else
	closeAngleDiff = 360 - _closeAngle + _peakAngle;
    }

  QPoint center = _ellipseRect.center();
  QPoint vec = point-center;

  double vecNormX, vecNormY;
  getNormVec(vec, vecNormX, vecNormY);

  double angle = acos(vecNormX) * 180 / M_PI;
  if (vec.y() > 0)
    angle = 360 - angle;
  
  QPoint rectPoint;
  bool changeWidth = getRectPoint(angle, rectPoint);
  
  QPoint normRectPoint;
  getRect2SquarePoint(changeWidth, rectPoint, normRectPoint);
  getNormVec(normRectPoint-center, vecNormX, vecNormY);

  newAngle = acos(vecNormX) * 180 / M_PI;
  if (vec.y() > 0)
    newAngle = 360 - newAngle;

  if (updateAngles)
    {
      _openAngle = newAngle + openAngleDiff;
      if (_openAngle >= 360)
	_openAngle -= 360;
      
      if (_closeAngle > _peakAngle)
	_closeAngle = 360 - (closeAngleDiff - newAngle);
      else
	_closeAngle = newAngle - closeAngleDiff;

      if (_closeAngle >= 360)
	_closeAngle -= 360;
      else if (_closeAngle < 0)
	_closeAngle = 360 + _closeAngle;
    }
}

void SmageEditorSpeechBubble::calcRefAngles(const QRect &rect)
{
  // Calculate a rectangle's angles (from center to 4 corners)

  QPoint vec = rect.topRight() - rect.bottomLeft();
  double vecNormX, vecNormY;
  getNormVec(vec, vecNormX, vecNormY);

  _refAngle_1 = int(acos(vecNormX) * 180 / M_PI);

  _refAngle_2 = 180 - _refAngle_1;
  _refAngle_3 = 180 + _refAngle_1;
  _refAngle_4 = 360 - _refAngle_1;
}


void SmageEditorSpeechBubble::calcBubblePoints()
{
  _bubblePoints = QPointArray();

  int closeAngle;
  if (_closeAngle <= _openAngle)
    closeAngle = int(360 - _openAngle + _closeAngle);
  else
    closeAngle = int(_closeAngle - _openAngle);
  
  _bubblePoints.makeArc(_ellipseRect.x(), _ellipseRect.y(), _ellipseRect.width(), _ellipseRect.height(), 
			int(_openAngle)*16, closeAngle*16);

  if (_bubblePoints.resize(_bubblePoints.count()+1))
    _bubblePoints.setPoint(_bubblePoints.count()-1, _bubblePeakPos);
}

//
// Handles paint events for the connect widget.
//

void SmageEditorSpeechBubble::paintEvent(QPaintEvent *)
{
  QPainter p(this);
  QPen pen;
  QBrush brush;
  int width = _newBubbleData.shapeData.bubbleOutlineWidth;

  if (width > 0)
    pen = QPen(_newBubbleData.shapeData.bubbleOutlineCol, 2*width - width%2);
  else
    pen = QPen( Qt::NoPen );
  if ( _newBubbleData.shapeData.bubbleBackgroundAlpha != 0 )
    brush = QBrush(_newBubbleData.shapeData.bubbleBackCol);
  else
    brush.setPixmap( QPixmap("pixmaps/transparentBackground.xpm") );

  p.setBrush( brush );
  p.setPen(pen);
  p.drawPolygon(_bubblePoints);
  
  p.setBrush(Qt::NoBrush);
  p.setPen(Qt::SolidLine);
  p.drawRect(_bubblePoints.boundingRect());
}




void SmageEditorSpeechBubble::updateBubbleAnglePos(const bool &angle1)
{
  calcBubblePoints();
  
  if (_bubblePoints.size() > 2)
    {
      QPoint p;
      SmageButton *button;

      if (_bubblePoints.boundingRect().left() != 0 ||
	  _bubblePoints.boundingRect().top() != 0)
	{
	  handleAnglesSpecialCase();

	  if (!angle1)
	    {
	      p = _bubblePoints.point(0);
	      button = _buttonAngle1;
	    }
	  else
	    {
	      p = _bubblePoints.point(_bubblePoints.count()-2);
	      button = _buttonAngle2;
	    }

	  button->move(p - QPoint(int(button->width()/2.0), int(button->height()/2.0)));
	}

      if (angle1)
	{
	  p = _bubblePoints.point(0);
	  button = _buttonAngle1;
	}
      else
	{
	  p = _bubblePoints.point(_bubblePoints.count()-2);
	  button = _buttonAngle2;
	}

      button->move(p - QPoint(int(button->width()/2.0), int(button->height()/2.0)));

      QPoint p1;
      p1 = _bubblePoints.boundingRect().bottomRight();
      _buttonBoundingBox->move(p1.x() - int(_buttonBoundingBox->width()/2.0), p1.y() - int(_buttonBoundingBox->height()/2.0));
      assignWidgetSize();
    }

  clearMask();
}

void SmageEditorSpeechBubble::handleAnglesSpecialCase()
{
  int left = geometry().left();
  int top = geometry().top();

  if (_bubblePoints.boundingRect().left() != 0)
    {
      _ellipseRect.moveLeft(_ellipseRect.left() - _bubblePoints.boundingRect().left());
      _bubblePeakPos.setX(_bubblePeakPos.x() - _bubblePoints.boundingRect().left());
      left += _bubblePoints.boundingRect().left();
    }

  if (_bubblePoints.boundingRect().top() != 0)
    {
      _ellipseRect.moveTop(_ellipseRect.top() - _bubblePoints.boundingRect().top());
      _bubblePeakPos.setY(_bubblePeakPos.y() - _bubblePoints.boundingRect().top());
      top += _bubblePoints.boundingRect().top();
    }

  calcBubblePoints();

  QRect boundingBox = _bubblePoints.boundingRect();
  setGeometry(left, top,
	      boundingBox.width() + int(ceil(double(_buttonBoundingBox->width()/2.0))),
	      boundingBox.height() + int(ceil(double(_buttonBoundingBox->height()/2.0))));

  QPoint p1 = _bubblePoints.point(_bubblePoints.count()-1);
  _buttonPeak->move(p1.x() - int(_buttonPeak->width()/2.0), p1.y() - int(_buttonPeak->height()/2.0));

  moveTextToCenter();
}

void SmageEditorSpeechBubble::moveBubbleAngle1CB(void *userData, const QPoint &pos)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;
  db->calcAngle(pos, db->_openAngle, FALSE);
  db->updateBubbleAnglePos(TRUE);

  db->update();
}

void SmageEditorSpeechBubble::moveBubbleAngle2CB(void *userData, const QPoint &pos)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;
  db->calcAngle(pos, db->_closeAngle, FALSE);
  db->updateBubbleAnglePos(FALSE);

  db->update();
}


void SmageEditorSpeechBubble::handlePeakSpecialCase()
{
  int left = geometry().left();
  int top = geometry().top();

  QPoint newTextPos = _bubbleText->pos();

  if (_bubblePeakPos.x() <= _ellipseRect.left())
    {
      _ellipseRect.moveLeft(_ellipseRect.left() - _bubblePeakPos.x());
      newTextPos.setX(newTextPos.x() - _bubblePeakPos.x());
      left = left + _bubblePeakPos.x();
      _bubblePeakPos.setX(0);
    }

  if (_bubblePeakPos.y() <= _ellipseRect.top())
    {
      _ellipseRect.moveTop(_ellipseRect.top() - _bubblePeakPos.y());
      newTextPos.setY(newTextPos.y() - _bubblePeakPos.y());
      top = top + _bubblePeakPos.y();
      _bubblePeakPos.setY(0);
    }

  calcBubblePoints();

  _bubbleText->move(newTextPos);

  QRect boundingBox = _bubblePoints.boundingRect();
  setGeometry(left, top,
	      boundingBox.width() + int(ceil(double(_buttonBoundingBox->width()/2.0))),
	      boundingBox.height() + int(ceil(double(_buttonBoundingBox->height()/2.0))));
}


void SmageEditorSpeechBubble::moveBubblePeakCB(void *userData, const QPoint &pos)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;
  db->calcAngle(pos, db->_peakAngle, TRUE);

  db->_bubblePeakPos = pos;    

  if (db->_bubblePeakPos.x() <= db->_ellipseRect.left() ||
      db->_bubblePeakPos.y() <= db->_ellipseRect.top())
    db->handlePeakSpecialCase();
  else
    {
      db->_ellipseRect.moveTopLeft(QPoint(0,0));
      db->calcBubblePoints();
    }

  db->_vecBBox = db->_bubblePoints.boundingRect().bottomRight() - db->_bubblePoints.boundingRect().topLeft();
  db->_vecPeak = db->_bubblePeakPos - db->_bubblePoints.boundingRect().topLeft();
  db->_bBoxDiff = QPoint(0,0);

  QPoint p1, p2, p3;
  p1 = db->_bubblePoints.point(0);
  p2 = db->_bubblePoints.point(db->_bubblePoints.count()-2);
  p3 = db->_bubblePoints.boundingRect().bottomRight();
 
  db->_buttonAngle1->move(p1.x() - int(db->_buttonAngle1->width()/2.0), p1.y() - int(db->_buttonAngle1->height()/2.0));
  db->_buttonAngle2->move(p2.x() - int(db->_buttonAngle2->width()/2.0), p2.y() - int(db->_buttonAngle2->height()/2.0));
  db->_buttonBoundingBox->move(p3.x() - int(db->_buttonBoundingBox->width()/2.0), p3.y() - int(db->_buttonBoundingBox->height()/2.0));

  db->assignWidgetSize();
  db->clearMask();

  db->update();
}


void SmageEditorSpeechBubble::assignNewPeakPos()
{
  QPoint newVecBBox = _vecBBox + _bBoxDiff;
  QPoint newVecPeak;
  newVecPeak.setX(newVecBBox.x() * _vecPeak.x() / _vecBBox.x());
  newVecPeak.setY(newVecBBox.y() * _vecPeak.y() / _vecBBox.y());

  _bubblePeakPos = _bubblePoints.boundingRect().topLeft() + newVecPeak;
}


void SmageEditorSpeechBubble::updateBBox(const QPoint &pos)
{
  QPoint currentPos = _buttonBoundingBox->pos();
  QPoint newPos = pos - QPoint(int(_buttonBoundingBox->width()/2.0), int(_buttonBoundingBox->height()/2.0));
  int diffX = newPos.x() - currentPos.x();
  int diffY = newPos.y() - currentPos.y();

  int newEllipseWidth = _ellipseRect.width() + diffX;
  int newEllipseHeight = _ellipseRect.height() + diffY;

  if (newEllipseWidth <= 0 || newEllipseHeight <= 0)
    return;

  _bBoxDiff += QPoint(diffX, diffY);
  assignNewPeakPos();
  moveTextToCenter();

  _ellipseRect.setWidth(newEllipseWidth);
  _ellipseRect.setHeight(newEllipseHeight);

  calcBubblePoints();

  newPos = _bubblePoints.boundingRect().bottomRight() - 
    QPoint(int(_buttonBoundingBox->width()/2.0), int(_buttonBoundingBox->height()/2.0));
  _buttonBoundingBox->move(newPos);

  QPoint p1, p2;
  p1 = _bubblePoints.point(0);
  p2 = _bubblePoints.point(_bubblePoints.count()-2);

  _buttonAngle1->move(p1.x() - int(_buttonAngle1->width()/2.0), p1.y() - int(_buttonAngle1->height()/2.0));
  _buttonAngle2->move(p2.x() - int(_buttonAngle2->width()/2.0), p2.y() - int(_buttonAngle2->height()/2.0));
  _buttonPeak->move(_bubblePeakPos.x() - int(_buttonPeak->width()/2.0), _bubblePeakPos.y() - int(_buttonPeak->height()/2.0));

  assignWidgetSize();
  clearMask();

  update();
}

void SmageEditorSpeechBubble::updateBBoxCB(void *userData, const QPoint &pos)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;
  
  db->updateBBox(pos);
}  


void SmageEditorSpeechBubble::buttonReleaseCB(void *userData)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;

  db->calcWidgetMask();
  db->setMask(db->_bubbleMask);
}


void SmageEditorSpeechBubble::buttonPressCB(void *userData)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;

  db->clearMask();
}

void SmageEditorSpeechBubble::bubbleTextMoveCB(void *userData)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;

  db->clearMask();
}


void SmageEditorSpeechBubble::bubbleTextMouseReleaseCB(void *userData)
{
  SmageEditorSpeechBubble *db = (SmageEditorSpeechBubble *) userData;

  db->calcWidgetMask();
  db->setMask(db->_bubbleMask);
}

void SmageEditorSpeechBubble::previewCB(void *userData, const TheS_BubbleData &bubbleData)
{
  SmageEditorSpeechBubble *bubble = (SmageEditorSpeechBubble *) userData;
  bubble->_newBubbleData = bubbleData;

  bubble->updateText(bubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorSpeechBubble::okCB(void *userData, const TheS_BubbleData &bubbleData)
{
  SmageEditorSpeechBubble *bubble = (SmageEditorSpeechBubble *) userData;
  bubble->_prevBubbleData = bubbleData;
  bubble->_newBubbleData=bubbleData;

  bubble->updateText(bubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorSpeechBubble::cancelCB(void *userData)
{
  SmageEditorSpeechBubble *bubble = (SmageEditorSpeechBubble *) userData;
  bubble->_newBubbleData=bubble->_prevBubbleData;

  bubble->updateText(bubble->_prevBubbleData);
  bubble->repaint(FALSE);
}

std::vector<char> SmageEditorSpeechBubble::getBubbleData(const QPoint &posOffset)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorSpeechBubble::getBubbleData()\n");
#endif

  SmageComicEllipseShape comicEllipseShape;

  // The comic bubble object
  SmageComicEllipseObj comicObj;

  comicObj.ellipseObjData.textObjData.position = pos() - posOffset;
  SmageObjShape::intToChar(_newBubbleData.shapeData.bubbleOutlineWidth, comicObj.ellipseObjData.textObjData.lineWidth);
  comicObj.ellipseObjData.textObjData.textPosition = _bubbleText->pos();
  comicObj.ellipseObjData.textObjData.shapeType = SMAGE_SPEECH_BUBBLE;
  comicObj.ellipseObjData.textObjData.textData.text = _newBubbleData.textData.bubbleText;
  comicObj.ellipseObjData.textObjData.textData.textFont = _newBubbleData.textData.bubbleTextFont;

  if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignLeft) == Qt::AlignLeft)
    comicObj.ellipseObjData.textObjData.textData.textAlignment = 1;
  else if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignHCenter) == Qt::AlignHCenter)
    comicObj.ellipseObjData.textObjData.textData.textAlignment = 2;
  else if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignRight) == Qt::AlignRight)
    comicObj.ellipseObjData.textObjData.textData.textAlignment = 3;

  comicObj.ellipseObjData.textObjData.colors.textColor = _newBubbleData.textData.bubbleTextCol;
  comicObj.ellipseObjData.textObjData.colors.outlineColor = _newBubbleData.shapeData.bubbleOutlineCol;
  comicObj.ellipseObjData.textObjData.colors.backgroundColor = _newBubbleData.shapeData.bubbleBackCol;
  comicObj.ellipseObjData.textObjData.colors.backgroundAlpha = _newBubbleData.shapeData.bubbleBackgroundAlpha;
  comicObj.ellipseObjData.ellipseShapeRect = QRect(_ellipseRect.x(), _ellipseRect.y(), _ellipseRect.width(), _ellipseRect.height());

  comicObj.bubblePeakPos = _bubblePeakPos;
  comicObj.openAngle = int(_openAngle);
  comicObj.closeAngle = int(_closeAngle);

  comicEllipseShape.setComicEllipseObject(&comicObj);
  comicEllipseShape.generateComicEllipseShapeString();

  return comicEllipseShape.getCharVector();
}

void SmageEditorSpeechBubble::moveTextToCenter()
{
  QRect textRect = _bubbleText->geometry();
  textRect.moveCenter(_ellipseRect.center());
  _bubbleText->move(textRect.topLeft());  

  calcWidgetMask();
  setMask(_bubbleMask);
}
