#ifndef _SMAGE_EDITOR_SPEECH_BUBBLE_H
#define _SMAGE_EDITOR_SPEECH_BUBBLE_H

// project header
#include "SmageButton.h"
#include "SmageEditorShapedText.h"
#include "SmageTextEditor.h"
#include <SmageComicEllipseShape.h>

// qt header
#include <qbitmap.h>
#include <qpointarray.h>


/** Widget class that enables a user to create and edit speech bubbles.
  *
  * @ingroup SmageApp
  *
  */
class SmageEditorSpeechBubble : public SmageEditorShapedText
{
  Q_OBJECT

 public:
  SmageEditorSpeechBubble(QWidget *parent, const char* name);

  std::vector<char> getBubbleData(const QPoint &posOffset);
  virtual QString getClassName() { return QString("SmageEditorSpeechBubble"); }

  ~SmageEditorSpeechBubble();

 protected:

  void paintEvent(QPaintEvent *);

  static void moveBubblePeakCB(void *userData, const QPoint &pos);
  static void moveBubbleAngle1CB(void *userData, const QPoint &pos);
  static void moveBubbleAngle2CB(void *userData, const QPoint &pos);
  static void updateBBoxCB(void *userData, const QPoint &pos);
  static void buttonReleaseCB(void *userData);
  static void buttonPressCB(void *userData);
  static void bubbleTextMoveCB(void *userData);
  static void bubbleTextMouseReleaseCB(void *userData);

  static void previewCB(void *, const TheS_BubbleData &);
  static void okCB(void *, const TheS_BubbleData &);
  static void cancelCB(void *);

  void updateBBox(const QPoint &pos);
  void initEditButtons();

  void calcBubblePoints();
  void getNormVec(const QPoint &vec, double &vecNormX, double &vecNormY);
  void getRect2SquarePoint(const bool &changeWidth, const QPoint &rectPoint, QPoint &scuarePoint);
  bool getRectPoint(const double &angle, QPoint &rectPoint);
  void calcRefAngles(const QRect &rect);
  void updateBubbleAnglePos(const bool &angle1);
  void calcAngle(const QPoint &point, double &newAngle, const bool &updateAngles);
  void assignNewPeakPos();
  void calcWidgetMask();
  void assignWidgetSize();
  void handlePeakSpecialCase();
  void handleAnglesSpecialCase();

  QPoint _bubblePeakPos;
  QRect _ellipseRect;
  QPointArray _bubblePoints;

  QPoint _vecBBox, _vecPeak;
  QPoint _bBoxDiff;

  int _refAngle_1, _refAngle_2, _refAngle_3, _refAngle_4;
  double _openAngle, _closeAngle, _peakAngle;

  SmageButton *_buttonPeak, *_buttonAngle1, *_buttonAngle2, *_buttonBoundingBox;

 protected slots:
   void moveTextToCenter();

};

#endif
