// project header
#include "SmageEditorThinkBubble.h"

// qt header
#include <qcursor.h>
#include <qpainter.h>
#include <qpopupmenu.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageEditorThinkBubble::SmageEditorThinkBubble(QWidget *parent, const char* name)
  : SmageEditorShapedText(parent, name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorThinkBubble::SmageEditorThinkBubble()\n");
#endif

  _prevBubbleData.textData.bubbleText = QString(tr("Click right mouse\nbutton to edit"));
  _prevBubbleData.textData.bubbleTextFont = QFont("Helvetica", 12, -1);
  _prevBubbleData.textData.bubbleTextCol = QColor(0,0,0);
  _prevBubbleData.shapeData.bubbleBackCol = QColor(255,255,255);
  _prevBubbleData.shapeData.bubbleOutlineCol = QColor(0,0,0);
  _prevBubbleData.shapeData.bubbleOutlineWidth = 1;
  _prevBubbleData.textData.bubbleTextFlags |= Qt::AlignCenter;
  _newBubbleData = _prevBubbleData;

  initClassData();
  initClassWidgets();

  calcWidgetMask();
  setMask(_bubbleMask);
}

void SmageEditorThinkBubble::initClassWidgets()
{
  _mainBubble = new SmageBubble(this, "MainBubble", QRect(0,0,150,70), FALSE);
  _mainBubble->setMouseMoveCB(this, mainBubbleMoveCB);
  _mainBubble->setMouseReleaseCB(this, mainBubbleReleaseCB);
  _mainBubble->setMousePressCB(this, mainBubblePressCB);
  _mainBubble->setUpdateMaskCB(this, updateMaskCB);  
  _mainBubble->setUpdateBBoxCB(this, updateBBoxCB);
  _mainBubble->show();
  
  SmageBubble *bubble1 = new SmageBubble(this, "SubBubble1", QRect(0,0,21,18), TRUE);
  _otherBubblesVec.push_back(bubble1);
  bubble1->setUpdateMaskCB(this, updateMaskCB);
  bubble1->setMouseMoveCB(this, crossingBorderCB);
  bubble1->setUpdateBBoxCB(this, updateBBoxCB);
  bubble1->move(55,74);
  bubble1->show();
  SmageBubble *bubble2 = new SmageBubble(this, "SubBubble2", QRect(0,0,16,14), TRUE);
  _otherBubblesVec.push_back(bubble2);
  bubble2->setUpdateMaskCB(this, updateMaskCB);
  bubble2->setMouseMoveCB(this, crossingBorderCB);
  bubble2->setUpdateBBoxCB(this, updateBBoxCB);
  bubble2->move(37,85);
  bubble2->show();
  SmageBubble *bubble3 = new SmageBubble(this, "SubBubble3", QRect(0,0,14,12), TRUE);
  _otherBubblesVec.push_back(bubble3);
  bubble3->setMouseMoveCB(this, crossingBorderCB);
  bubble3->setUpdateMaskCB(this, updateMaskCB);
  bubble3->setUpdateBBoxCB(this, updateBBoxCB);
  bubble3->move(23,100);
  bubble3->show();

  updateShapes(_newBubbleData);

  assignWidgetSize();  

  // Init the main bubble's text  
  _bubbleText = new SmageBubbleText(this, "BubbleText", rect(), _newBubbleData.textData.bubbleText, 
				    _newBubbleData.textData.bubbleTextFont);

  _bubbleText->setMouseMoveCB(this, bubbleTextMoveCB);
  _bubbleText->setMouseReleaseCB(this, bubbleTextMouseReleaseCB);

  moveTextToCenter();
  _bubbleText->show();
}

void SmageEditorThinkBubble::initClassData()
{
  _pressedPos = QPoint(0,0);

  _mainBubble = NULL;
  _bubbleText = NULL;
  _otherBubblesVec.clear();

  initContextMenu();

  _bubbleEditor = new SmageTextEditor(0, "Editor", _newBubbleData.textData.bubbleText, _newBubbleData.textData.bubbleTextFont);
  _bubbleEditor->setPrevCB(this, previewCB);
  _bubbleEditor->setOkCB(this, okCB);
  _bubbleEditor->setCancelCB(this, cancelCB);
  _bubbleEditor->hide();
}

SmageEditorThinkBubble::~SmageEditorThinkBubble()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorThinkBubble::~SmageEditorThinkBubble()\n");
#endif
}

void SmageEditorThinkBubble::assignWidgetSize()
{
  int top = geometry().top();
  int left = geometry().left();
 
  QRegion region(_mainBubble->rect());

  for (unsigned int i=0; i<_otherBubblesVec.size(); i++)
    region += QRegion(_otherBubblesVec[i]->geometry());
 
  setGeometry(left, top, region.boundingRect().width(), region.boundingRect().height());
}

void SmageEditorThinkBubble::calcWidgetMask()
{
  _bubbleMask = QBitmap(width(), height(), TRUE);
  
  QPainter painter(&_bubbleMask);
  painter.setPen(Qt::color1);
  painter.setBrush(Qt::color1);
  QBitmap srcBitmap = _bubbleText->getMask();

  bitBlt(&_bubbleMask, _bubbleText->geometry().topLeft().x(), _bubbleText->geometry().topLeft().y(), 
	 &srcBitmap, 0, 0, _bubbleText->width(), _bubbleText->height(), Qt::OrROP, TRUE);

  if (_mainBubble != NULL)
    {
      QBitmap srcBitmap = _mainBubble->getMask();
      bitBlt(&_bubbleMask, _mainBubble->geometry().topLeft().x(), _mainBubble->geometry().topLeft().y(), 
	     &srcBitmap, 0, 0, _mainBubble->width(), _mainBubble->height(), Qt::OrROP, TRUE);
    }

  for (unsigned int i=0; i<_otherBubblesVec.size(); i++)
    {
      QBitmap srcBitmap = _otherBubblesVec[i]->getMask();
      bitBlt(&_bubbleMask, _otherBubblesVec[i]->geometry().topLeft().x(), _otherBubblesVec[i]->geometry().topLeft().y(), 
	     &srcBitmap, 0, 0, _otherBubblesVec[i]->width(), _otherBubblesVec[i]->height(), Qt::OrROP, TRUE);
    }
}

void SmageEditorThinkBubble::bubbleTextMoveCB(void *userData)
{
  SmageEditorThinkBubble *db = (SmageEditorThinkBubble *) userData;

  db->clearMask();
}

void SmageEditorThinkBubble::bubbleTextMouseReleaseCB(void *userData)
{
  SmageEditorThinkBubble *db = (SmageEditorThinkBubble *) userData;

  db->calcWidgetMask();
  db->setMask(db->_bubbleMask);
}

void SmageEditorThinkBubble::updateShapes(const TheS_BubbleData &bubbleData)
{
  _mainBubble->setBubbleData(bubbleData.shapeData);

  for (unsigned int i=0; i<_otherBubblesVec.size(); i++)
    _otherBubblesVec[i]->setBubbleData(bubbleData.shapeData);
}

std::vector<char>  SmageEditorThinkBubble::getBubbleData(const QPoint &posOffset)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageEditorThinkBubble::getBubbleData()\n");
#endif

  SmageComicEllipseShape comicEllipseShape;

  // The comic bubble object
  SmageComicEllipseObj comicObj;

  comicObj.ellipseObjData.textObjData.position = pos() - posOffset;
  SmageObjShape::intToChar(_newBubbleData.shapeData.bubbleOutlineWidth, comicObj.ellipseObjData.textObjData.lineWidth);
  comicObj.ellipseObjData.textObjData.textPosition = _bubbleText->pos();
  comicObj.ellipseObjData.textObjData.shapeType = SMAGE_THINK_BUBBLE;
  comicObj.ellipseObjData.textObjData.textData.text = _newBubbleData.textData.bubbleText;
  comicObj.ellipseObjData.textObjData.textData.textFont = _newBubbleData.textData.bubbleTextFont;

  if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignLeft) == Qt::AlignLeft)
    comicObj.ellipseObjData.textObjData.textData.textAlignment = 1;
  else if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignHCenter) == Qt::AlignHCenter)
    comicObj.ellipseObjData.textObjData.textData.textAlignment = 2;
  else if ((_newBubbleData.textData.bubbleTextFlags & Qt::AlignRight) == Qt::AlignRight)
    comicObj.ellipseObjData.textObjData.textData.textAlignment = 3;

  comicObj.ellipseObjData.textObjData.colors.textColor = _newBubbleData.textData.bubbleTextCol;
  comicObj.ellipseObjData.textObjData.colors.outlineColor = _newBubbleData.shapeData.bubbleOutlineCol;
  comicObj.ellipseObjData.textObjData.colors.backgroundColor = _newBubbleData.shapeData.bubbleBackCol;
  comicObj.ellipseObjData.textObjData.colors.backgroundAlpha = _newBubbleData.shapeData.bubbleBackgroundAlpha;

  comicObj.ellipseObjData.ellipseShapeRect = QRect( _mainBubble->pos(), _mainBubble->getRect().size() );

  for (unsigned int i=0; i<_otherBubblesVec.size(); i++)
    {
      SmageThinkBubbleCircle circle;
      circle.pos = _otherBubblesVec[i]->pos();
      circle.size = _otherBubblesVec[i]->getRect().size();
      comicObj.thinkBubbleCircles.push_back(circle);
    }
 
  comicEllipseShape.setComicEllipseObject(&comicObj);
  comicEllipseShape.generateComicEllipseShapeString();

  return comicEllipseShape.getCharVector();
}

void SmageEditorThinkBubble::mainBubbleMoveCB(void *userData)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  if (userData == NULL)
    return;

  bubble->clearMask();      
  bubble->move(bubble->pos() - bubble->_pressedPos + bubble->mapFromGlobal(QCursor::pos()));
}

void SmageEditorThinkBubble::mainBubbleReleaseCB(void *userData)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  if (userData == NULL)
    return;

  bubble->setMask(bubble->_bubbleMask);
}

void SmageEditorThinkBubble::mainBubblePressCB(void *userData, const QMouseEvent *event)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  if (userData == NULL)
    return;

  if (event->button() == Qt::RightButton)
	{
   bubble->_contextMenu->exec(QCursor::pos());
	}
  else
  	bubble->_pressedPos = bubble->mapFromGlobal(QCursor::pos());
}


void SmageEditorThinkBubble::updateMaskCB(void *userData, const bool &clearMask)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  if (userData == NULL)
    return;

  if (clearMask)
    bubble->clearMask();
  else
    {
      bubble->calcWidgetMask();
      bubble->setMask(bubble->_bubbleMask); 
    }
}


void SmageEditorThinkBubble::crossingBorderCB(void *userData)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  if (userData == NULL)
    return;

  bubble->handleCrossingBorder();
}

void SmageEditorThinkBubble::updateBBoxCB(void *userData, const char *name)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  if (userData == NULL)
    return;

  bubble->handleCrossingBorder();

  if (strcmp(name, "MainBubble") == 0)
    {
      bubble->clearMask();
      bubble->moveTextToCenter();
    }
}

void SmageEditorThinkBubble::handleCrossingBorder()
{
  int left = geometry().left();
  int top = geometry().top();

  QPoint newTextPos = _bubbleText->pos();
  bool moveText = FALSE;
  
  QRegion region(_mainBubble->geometry());
  unsigned int i;
  for (i=0; i<_otherBubblesVec.size(); i++)
    region += QRegion(_otherBubblesVec[i]->geometry());
 
  if (region.boundingRect().left() != 0)
    {
      _mainBubble->move(_mainBubble->geometry().left() - region.boundingRect().left(), _mainBubble->geometry().y());
      left += region.boundingRect().left();
      newTextPos.setX(newTextPos.x() - region.boundingRect().left());
      moveText = TRUE;

      for (i=0; i<_otherBubblesVec.size(); i++)
	_otherBubblesVec[i]->move(_otherBubblesVec[i]->geometry().left() - region.boundingRect().left(), 
				  _otherBubblesVec[i]->geometry().y());
    }
  if (region.boundingRect().top() != 0)
    {
      _mainBubble->move(_mainBubble->geometry().x(), _mainBubble->geometry().y() - region.boundingRect().top());
      top += region.boundingRect().top();
      newTextPos.setY(newTextPos.y() - region.boundingRect().top());
      moveText = TRUE;

      for (i=0; i<_otherBubblesVec.size(); i++)
	_otherBubblesVec[i]->move(_otherBubblesVec[i]->geometry().x(), 
				  _otherBubblesVec[i]->geometry().y() - region.boundingRect().top());
    }

  if (moveText)
    _bubbleText->move(newTextPos);

  QRegion newRegion(_mainBubble->geometry());
  
  for (i=0; i<_otherBubblesVec.size(); i++)
    newRegion += QRegion(_otherBubblesVec[i]->geometry());

  setGeometry(left, top, newRegion.boundingRect().width(), newRegion.boundingRect().height());
}


void SmageEditorThinkBubble::previewCB(void *userData, const TheS_BubbleData &bubbleData)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  bubble->_newBubbleData = bubbleData;

  bubble->updateText(bubbleData);
  bubble->updateShapes(bubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorThinkBubble::okCB(void *userData, const TheS_BubbleData &bubbleData)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  bubble->_prevBubbleData=bubbleData;
  bubble->_newBubbleData=bubbleData;

  bubble->updateText(bubbleData);
  bubble->updateShapes(bubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorThinkBubble::cancelCB(void *userData)
{
  SmageEditorThinkBubble *bubble = (SmageEditorThinkBubble *) userData;
  bubble->_newBubbleData=bubble->_prevBubbleData;

  bubble->updateText(bubble->_prevBubbleData);
  bubble->updateShapes(bubble->_prevBubbleData);
  bubble->repaint(FALSE);
}

void SmageEditorThinkBubble::moveTextToCenter()
{
  QRect textRect = _bubbleText->geometry();
  textRect.moveCenter(_mainBubble->geometry().center());
  _bubbleText->move(textRect.topLeft());  
  
  //calcWidgetMask();
  //setMask(_bubbleMask);
}
