#ifndef _SMAGE_EDITOR_THINK_BUBBLE_H
#define _SMAGE_EDITOR_THINK_BUBBLE_H

// project header
#include "SmageBubble.h"
#include "SmageEditorShapedText.h"
#include <SmageComicEllipseShape.h>

/** Widget class that enables a user to create and edit think bubbles.
  *
  * @ingroup SmageApp
  *
  * Think-Bubbles are composed of a main ellipse containing
  * the text and three smaller ellipses connecting object and main 
  * ellipse.
  */
class SmageEditorThinkBubble : public SmageEditorShapedText
{
  Q_OBJECT

 public:
  SmageEditorThinkBubble(QWidget *parent, const char* name);

  std::vector<char> getBubbleData(const QPoint &posOffset);
  virtual QString getClassName() { return QString("SmageEditorThinkBubble"); }

  ~SmageEditorThinkBubble();

 protected:

  static void bubbleTextMoveCB(void *userData);
  static void bubbleTextMouseReleaseCB(void *userData);
  static void mainBubbleReleaseCB(void *userData);
  static void mainBubblePressCB(void *userData, const QMouseEvent *event);
  static void mainBubbleMoveCB(void *userData);
  static void updateMaskCB(void *userData, const bool &clearMask);
  static void crossingBorderCB(void *userData);
  static void updateBBoxCB(void *userData, const char *name);

  static void previewCB(void *, const TheS_BubbleData &);
  static void okCB(void *, const TheS_BubbleData &);
  static void cancelCB(void *);
  void initClassData();
  void initClassWidgets();

  void calcWidgetMask();
  void assignWidgetSize();
  void updateShapes(const TheS_BubbleData &bubbleData);
  void handleCrossingBorder();

  SmageBubble *_mainBubble;
  std::vector<SmageBubble *> _otherBubblesVec;

 protected slots:
   void moveTextToCenter();

};

#endif
