// project header
#include "SmageHTMLGenerator.h" 

// qt header
#include <qapplication.h>
#include <qclipboard.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qvalidator.h>

ImageProperties::ImageProperties()
{
  _fileName = "";
  _width = -1;
  _height = -1;
}

ImageProperties::ImageProperties( QString fN, int w, int h)
{
  _fileName = fN;
  _width = w;
  _height = h;
}

SmageHTMLGenerator::SmageHTMLGenerator( QWidget *parent, const char *name, bool modal, WFlags f ) : QDialog( parent, name, modal, f )
{
  setCaption( tr("Smage - HTML Generator") );
  resize(400,300);
  QGridLayout *layout = new QGridLayout( this, 5, 5, 5, 1 );

  layout->addWidget( new QLabel( tr("File: "), this ), 0, 0 );
    _fileName = new QLineEdit( this );
    connect( _fileName, SIGNAL( textChanged(const QString&) ), this, SLOT( updateText() ) );
  layout->addMultiCellWidget( _fileName, 0, 0, 1, 4 );

  layout->addWidget( new QLabel( tr("Width: "), this ), 1, 0 );
    _width = new QLineEdit( this );
    _width->setMaximumWidth(55);
    _width->setAlignment(Qt::AlignRight);
    _width->setValidator( new QIntValidator( 1, 3600, _width));
    connect( _width, SIGNAL( textChanged(const QString&) ), this, SLOT( updateText() ) );
  layout->addWidget( _width, 1, 1 );
  layout->addWidget( new QLabel( tr("Pixels"), this ), 1, 2 );

  layout->addWidget( new QLabel( tr("Height: "), this ), 2, 0 );
    _height = new QLineEdit( this );
    _height->setMaximumWidth(55);
    _height->setAlignment(Qt::AlignRight);
    _height->setValidator( new QIntValidator( 1, 3600, _height));
    connect( _height, SIGNAL( textChanged(const QString&) ), this, SLOT( updateText() ) );
  layout->addWidget( _height, 2, 1 );
  layout->addWidget( new QLabel( tr("Pixels"), this ), 2, 2 );

    _hasLinkUrl = new QCheckBox( tr("Link URL: "), this );
    _hasLinkUrl->setChecked( FALSE );
    connect( _hasLinkUrl, SIGNAL( clicked() ), this, SLOT( activateLink() ) );
  layout->addWidget( _hasLinkUrl, 3, 0 );
    _linkUrl = new QLineEdit( this );
    _linkUrl->setReadOnly( TRUE );
    connect( _linkUrl, SIGNAL( textChanged(const QString&) ), this, SLOT( updateText() ) );
  layout->addMultiCellWidget( _linkUrl, 3, 3, 1, 4 );

    _htmlSource = new QTextEdit( this );
  layout->addMultiCellWidget( _htmlSource, 4, 4, 0, 4 );
  
    QPushButton *copyButton = new QPushButton( tr("Copy all"), this );
    connect( copyButton, SIGNAL( clicked() ), this, SLOT( copyToClipboard() ) );
  layout->addWidget( copyButton, 5, 3 );
    QPushButton *closeButton = new QPushButton( tr("Close"), this );
    connect( closeButton, SIGNAL( clicked() ), this, SLOT( hide() ) );
  layout->addWidget( closeButton, 5, 4 );
  layout->setColStretch ( 2, 100 );  
  updateText();
}

SmageHTMLGenerator::~SmageHTMLGenerator()
{
}

void
SmageHTMLGenerator::updateText()
{
  QString code;
  code = QString(
    "<script src=\"SmagePluginTest.js\" type=\"text/javascript\">\n"
    "</script>\n"
    "<script type=\"text/javascript\">\n"
    "<!--\n"
    );
  if (_hasLinkUrl->isChecked() )    
    code.append(QString("  document.writeln( smage( \"%1\", %2, %3, \"%4\" ) );\n").arg(_fileName->text()).arg(_width->text()).arg(_height->text()).arg(_linkUrl->text()));
  else
    code.append(QString("  document.writeln( smage( \"%1\", %2, %3 ) );\n").arg(_fileName->text()).arg(_width->text()).arg(_height->text()));
  code.append(QString(
    "//-->\n"
    "</script>\n"
    "<noscript>\n"
  ));
  if (_hasLinkUrl->isChecked() )
  {
    code.append(QString(
     "    <embed type=\"application/smage\" smage=\"%1\" width=\"%2\" height=\"%3\"\n"
     "           href=\"%4\">\n"
        ).arg(_fileName->text()).arg(_width->text()).arg(_height->text()).arg(_linkUrl->text()));
  }
  else 
    code.append(QString(
     "    <embed type=\"application/smage\" smage=\"%1\" width=\"%2\" height=\"%3\">\n"
                        ).arg(_fileName->text()).arg(_width->text()).arg(_height->text()));
  code.append(QString(
    "</noscript>"));
  _htmlSource->setText(code);
}

void
SmageHTMLGenerator::copyToClipboard()
{
  QClipboard *cb = QApplication::clipboard();
  cb->setText(_htmlSource->text());
}

void
SmageHTMLGenerator::activateLink()
{
  _linkUrl->setReadOnly( !_hasLinkUrl->isChecked() );
  updateText();
}

void
SmageHTMLGenerator::setValues( ImageProperties ip )
{
  _fileName->setText( ip._fileName );
  _width->setText( QString("%1").arg(ip._width) );
  _height->setText( QString("%1").arg(ip._height) );
}
