
#ifndef _SMAGE_IMG_ENCODE_WIDGET_H
#define _SMAGE_IMG_ENCODE_WIDGET_H

// project header
#include "SmageDataManageWidget.h"
#include "SmageEditorShapedText.h"
#include "SmageImgSelectionWidget.h"
#include "SmageTextTypeDlg.h"

// qt header
#include <qbitmap.h>

typedef void TheS_Encoder_OpenImgCB(void *, const SmageSelectionType &, const bool &, const bool &);
typedef void TheS_Encoder_AllowActionCB(void *, const bool &, const bool &);

/** Class SmageImgEncodeWidget Widget that contains the image a user can interact with.
  *
  * @ingroup SmageApp
  *
  * A user can select image regions and encode data into those regions.
  */
class SmageImgEncodeWidget : public SmageImgSelectionWidget
{
  Q_OBJECT

    typedef SmageImgSelectionWidget inherited;

 public:
  
    SmageImgEncodeWidget(QWidget *parent, const char* name, WFlags f);
    ~SmageImgEncodeWidget();

    void setCodedImg(const QImage &outlineImg, const QImage &idBuffer, const int &startId);

    // Setting a number of callback functions
    void setOpenImgCB(void *userData, TheS_Encoder_OpenImgCB *f)
      { _openImgCBFunc = f; _openImgUserData = userData; }
    void setBuildImgCB(void *userData, TheS_AllowActionCB *f)
      { _buildImgCBFunc = f; _buildImgUserData = userData; }
    void setRegionSelectedExtCB(void *userData, TheS_Encoder_AllowActionCB *f)
      { _regionSelectedExtCBFunc = f; _regionSelectedExtUserData = userData; }

    /** Verify the last selected image region.
      */
    bool verifyRegion();

    /** Assign data to current active image region.
      */
    void integratePlainText(const QString &data);
    void integrateImg(const QImage* img, const QPointArray &region);
    void integrateFormattedText(const TextShapeType &textType);
    bool integrateDictEntry(SmageTextHolderWidget *textHolder);

    /** Encode data in image.
      */
    void buildImg();

    void doInsertDictEntry();

    SmageDataManageWidget *getDataManageWidget(){ return _dataManageWidget; }

    virtual QString getWidgetType() const { return QString("SMAGE_ENCODER_WIDGET"); }

    void deleteDataItem(SmageEditorShapedText *shapedText);

 protected:
    // A couple of events
    virtual void paintEvent(QPaintEvent *event);
    virtual void closeEvent(QCloseEvent *event);
    virtual void resizeEvent(QResizeEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);

    /** Class specific operations when an image has been successfully loaded.
      */
    virtual void doWhenSuccesImgLoading();

    virtual void incrementRegionID();

    static void dataManageWidgetButtonPressedCB(void *userData, const int &whichButton, const int &itemID);
    static void dataManageWidgetFocusCB(void *userData, const int &whichId);

    // Set callback functions and class instances
    TheS_Encoder_OpenImgCB *_openImgCBFunc;
    void *_openImgUserData;
    TheS_AllowActionCB *_buildImgCBFunc;
    void *_buildImgUserData;
    TheS_Encoder_AllowActionCB *_regionSelectedExtCBFunc;
    void *_regionSelectedExtUserData;

    /** Init buffers that store already selected regions.
      */
    void initReservedRegionBuffers();

    // Data has been encoded in the image
    bool encodingDone(const QImage &img, QString &fileName);

    void insertDataItem(const int &id, SmageEditorShapedText *shapedText);
    void prepareDataMapForCoding();
    void addNewDataItem(const int &id);

    /** Reset the image widgets data.
      */
    virtual void resetImgWidget();

    bool verifyAllRegions();

    // Some booleans
    bool _allowBuildingImg;

    /** Buffer that holds already selected regions.
      */
    QBitmap _reservedRegionsBitmap;

    /** Buffer that holds already coded regions.
     */
    QBitmap _codedRegionsBitmap;
    
    /** Image in which the already coded regions are highlighted
     */
    QImage _outlineImg;

    int _startId;

    std::map<int, TheS_Data> _dataMap;

    SmageDataManageWidget *_dataManageWidget;
};

#endif

