// project header
#include "SmageImgGrabber.h"
#ifndef SMAGE_COMMON_ICONS
#define SMAGE_COMMON_ICONS
#include "./pixmaps/open.xpm"
#include "./pixmaps/exit.xpm"
#include "./pixmaps/selectRect.xpm"
#include "./pixmaps/selectRect_dis.xpm"
#include "./pixmaps/selectPoly.xpm"
#include "./pixmaps/selectPoly_dis.xpm"
#endif

#include "./pixmaps/hand.xpm"
#include "./pixmaps/hand_dis.xpm"
#include "./pixmaps/selectAutoPoly.xpm"
#include "./pixmaps/selectAutoPoly_dis.xpm"

// qt header
#include <qapplication.h>
#include <qbuttongroup.h>
#include <qfiledialog.h> 
#include <qmenubar.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qvbox.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageImgGrabber::SmageImgGrabber(QWidget *parent, const char* name, WFlags f)
  : QMainWindow(parent, name, f)
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgGrabber::SmageImgGrabber()\n");
#endif

  setCaption(tr("Smage - Image Grabber"));

  if (QApplication::desktop()->width() > 700)
    setGeometry(230,170,600,400);
  else
    setGeometry(120,60,400,300);

  QVBox *vBox = new QVBox(this);
  vBox->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
  
  _wSpace = new QWorkspace(vBox);
  _wSpace->setScrollBarsEnabled(TRUE);

  setCentralWidget(vBox);

  _selCopyImgCBFunc = NULL;
  _selCopyImgUserData = NULL;

  initialize();
}


SmageImgGrabber::~SmageImgGrabber()
{
  delete _toggleButtonGroup;
}


void SmageImgGrabber::initialize()
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgGrabber::initialize()\n");
#endif

  QPopupMenu *fileMenu = new QPopupMenu(this);
  CHECK_PTR(fileMenu);
  fileMenu->insertItem(QPixmap(open_xpm), tr("&Open"), this, SLOT(open()), CTRL+Key_O, 1);

  fileMenu->insertSeparator();
  fileMenu->insertItem(QPixmap(exit_xpm), tr("E&xit"), this, SLOT(hide()), CTRL+Key_Q);

  menuBar()->insertItem(tr("&File"), fileMenu);

  QToolBar *toolBar = new QToolBar(this, "imgGrabberTools");
  
  _toggleButtonGroup = new QButtonGroup();
  _toggleButtonGroup->setExclusive(TRUE);
  
  QIconSet selectRectIconSet = QIconSet(QPixmap(selectRect_xpm));
  selectRectIconSet.setPixmap(QPixmap(selectRect_xpm), QIconSet::Small, QIconSet::Normal);
  selectRectIconSet.setPixmap(QPixmap(selectRect_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _selectRect = new QToolButton(selectRectIconSet, tr("Select a rectangular image region"), 
        QString::null, this, SLOT(selectRect()), toolBar, "selectRect");
  _selectRect->setToggleButton(TRUE);
  _selectRect->setEnabled(FALSE);
  _toggleButtonGroup->insert(_selectRect);

  QIconSet selectPolyIconSet = QIconSet(QPixmap(selectPoly_xpm));
  selectPolyIconSet.setPixmap(QPixmap(selectPoly_xpm), QIconSet::Small, QIconSet::Normal);
  selectPolyIconSet.setPixmap(QPixmap(selectPoly_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _selectPoly = new QToolButton(selectPolyIconSet, tr("Select any image region"), 
        QString::null, this, SLOT(selectPoly()), toolBar, "selectPoly");
  _selectPoly->setToggleButton(TRUE);
  _selectPoly->setEnabled(FALSE);
  _toggleButtonGroup->insert(_selectPoly);

  
  _selectAutoPoly = NULL;
  /*
  QIconSet selectAutoPolyIconSet = QIconSet(QPixmap(selectAutoPoly_xpm));
  selectAutoPolyIconSet.setPixmap(QPixmap(selectAutoPoly_xpm), QIconSet::Small, QIconSet::Normal);
  selectAutoPolyIconSet.setPixmap(QPixmap(selectAutoPoly_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _selectAutoPoly = new QToolButton(selectAutoPolyIconSet, tr("Select an image region automatically"), 
						QString::null, this, SLOT(selectAutoPoly()), toolBar, "selectAutoPoly");
  _selectAutoPoly->setEnabled(FALSE);
  */

  QIconSet grabPartIconSet = QIconSet(QPixmap(hand_xpm));
  grabPartIconSet.setPixmap(QPixmap(hand_xpm), QIconSet::Small, QIconSet::Normal);
  grabPartIconSet.setPixmap(QPixmap(hand_dis_xpm), QIconSet::Small, QIconSet::Disabled);
  _grabPart = new QToolButton(grabPartIconSet, tr("Grab selected image region"), 
            QString::null, this, SLOT(grabPart()), toolBar, "grabPart");
  _grabPart->setEnabled(FALSE);
}


void SmageImgGrabber::open()
{    
  char *d = getenv("SMAGE_DIR");
  QString smageDir ="";
  if ( d != NULL ) smageDir = QString( d + QObject::tr("My Smages") );

  QString string = QFileDialog::
    getOpenFileName(smageDir, tr("Images (*.bmp *.jpg *.jpeg *.png *.ppm *.xpm)"), this,
        "OpenFileDlg", tr("Open File"));
  if (!string.isEmpty())
  {
    bool validEnding;
    SmageImgWidget::getFileEnding(string, validEnding);
      
    if (validEnding)
      initializeNewImgWidget(string);
  }
}


void SmageImgGrabber::initializeNewImgWidget(const char *name)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgGrabber::initializeNewImgWidget()\n");
#endif

  SmageImgSelectCopyWidget *imgWidget = new SmageImgSelectCopyWidget(_wSpace, "imgWigdet", Qt::WDestructiveClose);

  // Set a number of callback functions
  imgWidget->setRegionSelectedCB(this, regionSelectedCB);
  imgWidget->setOpenImgCB(this, openImgCB);
  imgWidget->setCloseCB(this, closeCB);

  if (_selCopyImgCBFunc != NULL && _selCopyImgUserData != NULL)
    imgWidget->setSelCopyImgCB(_selCopyImgUserData, _selCopyImgCBFunc);
  
  // Cause "SmageImgWidget" to load an image
  if (imgWidget->loadImg(name))
  {
    imgWidget->showMaximized();
  }
}

// Assign a new type of image region selection
void SmageImgGrabber::assignNewSelectionType(const SmageSelectionType &selectionType)
{
  QWidget *widget = _wSpace->activeWindow();
  if (widget != NULL)
  {
    SmageImgWidget *tmpWidget = (SmageImgWidget *) widget;

    if (tmpWidget->getWidgetType() == "SMAGE_SELECT_COPY_WIDGET")
    {
      SmageImgSelectCopyWidget *smageWidget = (SmageImgSelectCopyWidget *) widget;
      smageWidget->setSelectionType(selectionType);
    }
  }
}

void SmageImgGrabber::selectPoly()
{
  assignNewSelectionType(SELECT_POLY);
}

void SmageImgGrabber::selectAutoPoly()
{   
  QWidget *widget = _wSpace->activeWindow();
  if (widget != NULL)
    {
      SmageImgWidget *tmpWidget = (SmageImgWidget *) widget;
      
      if (tmpWidget->getWidgetType() == "SMAGE_SELECT_COPY_WIDGET")
	{
	  SmageImgSelectCopyWidget *smageWidget = (SmageImgSelectCopyWidget *) widget;
	  smageWidget->selectAutoPoly();
	}
    }
}

void SmageImgGrabber::selectRect()
{
  assignNewSelectionType(SELECT_RECT);
}

// A new image has been opened or a different workspace widget is activated; 
//  which selection type was set ("selectionType")
void SmageImgGrabber::openImgCB(void *userData, const SmageSelectionType &selectionType,
        const bool &regionSelected)
{
  SmageImgGrabber *imgGrabber = (SmageImgGrabber *) userData;
  if (imgGrabber == 0)
    return;

  imgGrabber->_selectRect->setEnabled(TRUE);
  imgGrabber->_selectPoly->setEnabled(TRUE);
  //imgGrabber->_selectAutoPoly->setEnabled(TRUE);
  imgGrabber->_grabPart->setEnabled(regionSelected);

  if (selectionType == SELECT_RECT)
    imgGrabber->_selectRect->setOn(TRUE);
  else if (selectionType == SELECT_POLY)
    imgGrabber->_selectPoly->setOn(TRUE);
}

// A workspace widget is about to get closed
void SmageImgGrabber::closeCB(void *userData)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgGrabber::closeCB()\n");
#endif

  SmageImgGrabber *app = (SmageImgGrabber *) userData;
  if (app == NULL)
    return;

  app->_selectRect->setEnabled(FALSE);
  app->_selectRect->setOn(FALSE);
  app->_selectPoly->setEnabled(FALSE);
  app->_selectPoly->setOn(FALSE);
  //app->_selectAutoPoly->setEnabled(FALSE);
  app->_grabPart->setEnabled(FALSE);
}

//  A region has been selected
void SmageImgGrabber::regionSelectedCB(void *userData, const bool &regionSelected)
{
  SmageImgGrabber *app = (SmageImgGrabber *) userData;
  if (app == NULL)
    return;

  app->_grabPart->setEnabled(regionSelected);
}


void SmageImgGrabber::hideEvent(QHideEvent *)
{
  //_wSpace->closeAllWindows();

  QWidgetList wList = _wSpace->windowList();

  for (uint i=0; i<wList.count(); i++)
    ((QWidget *)wList.at(i))->close();
}


void SmageImgGrabber::grabPart()
{
  QWidget *widget = _wSpace->activeWindow();
  if (widget != NULL)
  {
    SmageImgSelectCopyWidget *tmpWidget = (SmageImgSelectCopyWidget *) widget;
    tmpWidget->grabPart();
  }
}
