#ifndef _SMAGE_IMG_GRABBER_H
#define _SMAGE_IMG_GRABBER_H

// project header
#include "SmageImgSelectCopyWidget.h"
#include "../smageSharedFiles/SmageConfig.h"

// qt header
#include <qmainwindow.h>
#include <qtoolbutton.h>
#include <qworkspace.h> 

class QButtonGroup;

/** Class SmageImgGrabber for opening an image and selecting regions of the image to be encoded into the cover image.
  *
  * @ingroup SmageApp
  *
  */class SmageImgGrabber : public QMainWindow
{
  Q_OBJECT

 public:
  
  SmageImgGrabber(QWidget *parent, const char* name, WFlags f = 0);
  ~SmageImgGrabber();

  // Setting a number of callback functions
  void setSelCopyImgCB(void *userData, TheS_SelectCopyImgCB *f)
    { _selCopyImgCBFunc = f; _selCopyImgUserData = userData; }

  public slots:
    void open();
    void selectPoly();
    void selectAutoPoly();
    void selectRect();
    void grabPart();

 protected:   
    virtual void hideEvent (QHideEvent *);

    void initialize();
    void initializeNewImgWidget(const char *name);

    /** A region has been selected.
      */
    static void regionSelectedCB(void *userData, const bool &regionSelected);
    
    /** A workspace widget is about to get closed
      */
    static void closeCB(void *userData);
    
    /** A new image has been opened or a different workspace widget is activated.
      * which selection type was set ("selectionType")
      */
    static void openImgCB(void *userData, const SmageSelectionType &selectionType, const bool &regionSelected);
    
    /** Assign a new type of image region selection
      */
    void assignNewSelectionType(const SmageSelectionType &selectionType);

    // Set callback functions and class instances
    TheS_SelectCopyImgCB *_selCopyImgCBFunc;
    void *_selCopyImgUserData;

    QWorkspace *_wSpace;
    QButtonGroup *_toggleButtonGroup;
    QToolButton *_selectRect, *_selectPoly, *_selectAutoPoly, *_grabPart;
};

#endif
