// project header
#include "SmageImgInputDlg.h"

// qt header
#include <qfiledialog.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qpushbutton.h>

// c/c++ header
#include <iostream>
#include <stdlib.h>

using namespace std;

SmageImgInputDlg::SmageImgInputDlg(QWidget *topLevel)
  : QDialog(topLevel, 0)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgInputDlg::SmageImgInputDlg()\n");
#endif

  _gridLayout = new QGridLayout(this,2,4,20);

  // Text input group

  _textGroupBox = new QGroupBox(tr("Unformatted text input"), this);

#ifdef SMAGE_DEMO
  _textGroupBox->setFixedSize(220, 70);

  _lineEdit = new QLineEdit(_textGroupBox);
  _lineEdit->setMaxLength(15);
  _lineEdit->setGeometry(10, 30, 200, _lineEdit->sizeHint().height());
  
#else  
  _textGroupBox->setFixedSize(420, 200);

  _textEdit = new QTextEdit(_textGroupBox);
  _textEdit->setTextFormat(QTextEdit::PlainText);
  _textEdit->setFocus();
  _textEdit->setGeometry(10, 30, 400, 150);
#endif

  // OK / Cancel button
  
  _okButton = new QPushButton(QObject::tr("OK"), this);
  connect(_okButton, SIGNAL(clicked()), this, SLOT(okPressed()));
  _okButton->setFixedSize(100, 40);
  _okButton->setDefault(TRUE);

  _cancelButton = new QPushButton(QObject::tr("Cancel"), this);
  connect(_cancelButton, SIGNAL(clicked()), this, SLOT(cancelPressed()));
  _cancelButton->setFixedSize(100, 40);

  _gridLayout->addMultiCellWidget(_textGroupBox, 0, 0, 0, 3, Qt::AlignHCenter);
  _gridLayout->addWidget(_okButton, 1, 1, Qt::AlignHCenter);
  _gridLayout->addWidget(_cancelButton, 1, 2, Qt::AlignHCenter);

  _textInput = QString("");
}


SmageImgInputDlg::~SmageImgInputDlg() 
{   
  // All the class' child-widgets are deleted automatically
}


void SmageImgInputDlg::okPressed()
{
#ifdef SMAGE_DEMO
  _textInput = _lineEdit->text();
#else
  _textInput = _textEdit->text();
#endif

  accept();
}


void SmageImgInputDlg::cancelPressed()
{
  reject();
}


bool SmageImgInputDlg::getTextInput(QString &textInput, QWidget *parent, const char*)
{
  textInput = QString("");
  bool returnValue = FALSE;

  SmageImgInputDlg *dlg = new SmageImgInputDlg(parent);
  
  if (dlg->exec() == QDialog::Accepted)
  {
    textInput = dlg->_textInput.ascii();
    returnValue = TRUE;
  }
  
  delete dlg;
  return returnValue;
}

bool SmageImgInputDlg::getTextInput(const QString &textToShow, QString &textInput, QWidget *parent, const char*)
{
  textInput = QString("");
  bool returnValue = FALSE;

  SmageImgInputDlg *dlg = new SmageImgInputDlg(parent);

#ifdef SMAGE_DEMO
  dlg->_lineEdit->setText(textToShow);
#else
  dlg->_textEdit->setText(textToShow);
#endif

  if (dlg->exec() == QDialog::Accepted)
  {
    textInput = dlg->_textInput.ascii();
    returnValue = TRUE;
  }
  
  delete dlg;
  return returnValue;
}
