// project header
#include "SmageImgHandler.h"
#include "SmageImgSelectCopyWidget.h"
#include "SmageMsgDlg.h"

// qt header
#include <qmessagebox.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageImgSelectCopyWidget::SmageImgSelectCopyWidget(QWidget *parent, const char* name, WFlags f)
  : SmageImgSelectionWidget(parent, name, f)
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgSelectCopyWidget::SmageImgSelectCopyWidget()\n");
#endif

  _openImgCBFunc = NULL;
  _openImgUserData = NULL;
  _selCopyImgCBFunc = NULL;
  _selCopyImgUserData = NULL;
  
  _img32BitsNeeded = FALSE;  
  _waitForClick = FALSE;
}

SmageImgSelectCopyWidget::~SmageImgSelectCopyWidget()
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgSelectCopyWidget::~SmageImgSelectCopyWidget()\n");
#endif
}

void SmageImgSelectCopyWidget::paintEvent(QPaintEvent *event)
{
  inherited::paintEvent(event);

  if (_imgLoaded)
  {
    // Update buttons in main application
    if (hasFocus())
    {
      if (_openImgCBFunc != NULL)
        (*_openImgCBFunc)(_openImgUserData, _selectionType, _regionSelected);
    }
  }
}

// Class specific operations when an image has been successfully loaded
void SmageImgSelectCopyWidget::doWhenSuccesImgLoading()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgSelectCopyWidget::doWhenSuccesImgLoading()\n");
#endif

  setCaption(_imgFileName);

  if (_openImgCBFunc != NULL)
    (*_openImgCBFunc)(_openImgUserData, _selectionType, _regionSelected);
}


void SmageImgSelectCopyWidget::grabPart()
{
  if (_selCopyImgCBFunc != NULL && _selCopyImgUserData != NULL)
    (*_selCopyImgCBFunc)(_selCopyImgUserData, _imgPointer, _tmpImgRelatedPointArray);
}


void SmageImgSelectCopyWidget::selectAutoPoly()
{
  int result;
  if (SmageMsgDlg::getResult(result, this, tr("Select Region"),
			     "Click on the image background ...", QObject::tr("OK")))
    {
      _waitForClick = TRUE;
    }
}

void SmageImgSelectCopyWidget::mousePressEvent(QMouseEvent *event)
{  
  if (!_waitForClick)
    SmageImgSelectionWidget::mousePressEvent(event);
}

void SmageImgSelectCopyWidget::mouseReleaseEvent(QMouseEvent *event)
{
  if (_waitForClick)
    {
      _waitForClick = FALSE;
      
      QPoint pos = event->pos() - _imgStartPos;
      if (_imgPointer->valid(pos.x(), pos.y()))
	{	  
	  QRgb color;
	  if (_imgPointer->depth() > 8)
	    color = _imgPointer->pixel(pos.x(), pos.y());
	  else
	    {
	      QRgb *colors = _imgPointer->colorTable();
	      int index = _imgPointer->pixelIndex(pos.x(), pos.y());
	      if (index>=0 && index<256)
		color = colors[index];
	    }

	  if (SmageImgHandler::findOutline(_imgPointer, _tmpImgRelatedPointArray, color))
	    {
	      _tmpWidgetRelatedPointArray = QPointArray(_tmpImgRelatedPointArray);
	      _tmpWidgetRelatedPointArray.detach();
	      _tmpWidgetRelatedPointArray.translate(_imgStartPos.x(), _imgStartPos.y());
	      _regionSelected = TRUE;
	      repaint();
	    }
	  else
	    QMessageBox::information(this, tr("Select Region"), 
				     tr("Unfortunately, no closed contour could be found."));
	}
    }
  else
    SmageImgSelectionWidget::mouseReleaseEvent(event);
}

void SmageImgSelectCopyWidget::mouseMoveEvent(QMouseEvent *event)
{
  if (!_waitForClick)
    SmageImgSelectionWidget::mouseMoveEvent(event);
}
