#ifndef _SMAGE_IMG_SELECTION_WIDGET_H
#define _SMAGE_IMG_SELECTION_WIDGET_H

// project header
#include "../smageSharedFiles/SmageImgWidget.h"

enum SmageSelectionType { SELECT_RECT, SELECT_POLY };

/** Class SmageImgSelectionWidget contains the image a user can interact with.
  *
  * @ingroup SmageApp
  *
  * Especially, this widget is thought for image region selections.
  */
class SmageImgSelectionWidget : public SmageImgWidget
{
  Q_OBJECT

    typedef SmageImgWidget inherited;

 public:
  
    SmageImgSelectionWidget(QWidget *parent, const char* name, WFlags f);
    ~SmageImgSelectionWidget();

    // Setting a number of callback functions
    void setSelectionType(const SmageSelectionType &type) { _selectionType = type; }
    void setRegionSelectedCB(void *userData, TheS_AllowActionCB *f)
      { _regionSelectedCBFunc = f; _regionSelectedUserData = userData; }

    virtual QString getWidgetType() const { return QString("SMAGE_SELECTOR_WIDGET"); }

    QPointArray getSelectedImgRelatedPoints()
      { return _tmpImgRelatedPointArray; }

 protected:
    // A couple of events
    virtual void paintEvent(QPaintEvent *event);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseReleaseEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);
    virtual void closeEvent(QCloseEvent *event);
    virtual void resizeEvent(QResizeEvent *event);

    virtual void incrementRegionID() = 0;

    // Set callback functions and class instances
    TheS_AllowActionCB *_regionSelectedCBFunc;
    void *_regionSelectedUserData;

    // Draw selected regions
    void drawRect(const QMouseEvent *event);
    void drawPoly(const QMouseEvent *event);

    // Reset the image widgets data (in particular, regions that have been selected are deleted)
    virtual void resetImgWidget();
    
    // Drawing tools 
    QPen *_pen;
    
    // Some booleans and positions that need to be stored
    bool _regionSelected, _rubberOn, _mouseHasBeenMoved;
    QPoint _rubberStart, _rubberEnd;

    // Storing the selected regions' points
    std::vector<QPoint> _pointVec;
    QPointArray _tmpImgRelatedPointArray;
    QPointArray _tmpWidgetRelatedPointArray;

    // Selection type for this widget
    SmageSelectionType _selectionType;

    bool _firstMousePress, _shouldIncRegionID;    
    int _regionID;
};

#endif
