// project header
#include "SmageImgSnip.h"

// qt header
#include <qcursor.h>
#include <qmessagebox.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageImgSnip::SmageImgSnip(QWidget *parent, const char* name, const QPoint &parentImgStartPos, 
			   const QSize &parentImgSize, WFlags f)
  : QWidget(parent, name, f)
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgSnip::SmageImgSnip()\n");
#endif

  setCursor(SizeAllCursor);
  setMouseTracking(FALSE);

  _pressedPos = QPoint(0,0);
  _parentImgStartPos = parentImgStartPos;
  _parentImgSize = parentImgSize;
}


SmageImgSnip::~SmageImgSnip()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageImgSnip::~SmageImgSnip()\n");
#endif  
}


bool SmageImgSnip::setImg(const QImage *img, const QPointArray &points, const QPoint &pos, const bool &sameSize)
{
  QRegion region(points);
  QRect rect = region.boundingRect();

  if (rect.width() > _parentImgSize.width() || rect.height() > _parentImgSize.height())
    {
      QMessageBox::warning(this, QObject::tr("Smage - Message"), tr("Selected image region is larger than the original image."),
			   QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);

      return FALSE;
    }

  _img = img->copy(rect);
  QPixmap pix(_img);

  setPaletteBackgroundPixmap(pix);

  if (sameSize)
    setGeometry(pos.x() + rect.x(), pos.y() + rect.y(), _img.width(), _img.height());
  else
    setGeometry(pos.x()-int(_img.width()/2), pos.y()-int(_img.height()/2), _img.width(), _img.height());

  _imgRegionPoints = points.copy();
  _imgRegionPoints.translate(-rect.left(), -rect.top());
  // Since points may become negative after translation, they are verified
  verifyRegionPoints(_imgRegionPoints);

  _imgRegion = QRegion(_imgRegionPoints);
  setMask(_imgRegion);

  return TRUE;
}


void SmageImgSnip::verifyRegionPoints(QPointArray &points)
{
  int x, y;
  bool pointAccepted;
  for (uint i=0; i<points.size(); i++)
  {
    pointAccepted = TRUE;
    points.point(i, &x, &y);

    if (x < 0)
    {
      x = 0; 
      pointAccepted = FALSE;
    }
    else if (x > _img.width()-1)
    {
      x = _img.width()-1;
      pointAccepted = FALSE;
    }
    if (y < 0)
    {
      y = 0;
      pointAccepted = FALSE;
    }
    else if (y > _img.height()-1)
    {
      y = _img.height()-1;
      pointAccepted = FALSE;
    }
  
    if (!pointAccepted)
      points.setPoint(i, x, y);
  }
}


void SmageImgSnip::mousePressEvent(QMouseEvent *event)
{
  _pressedPos = event->pos();

  event->accept();
}


void SmageImgSnip::mouseMoveEvent(QMouseEvent *event)
{
  QPoint newPos = pos() - _pressedPos + mapFromGlobal(QCursor::pos());

  if (newPos.x() < _parentImgStartPos.x())
    newPos.setX(_parentImgStartPos.x());
  else if ((newPos.x() + width()) > (_parentImgStartPos.x() + _parentImgSize.width()))
    newPos.setX(_parentImgStartPos.x() + _parentImgSize.width() - width());

  if (newPos.y() < _parentImgStartPos.y())
    newPos.setY(_parentImgStartPos.y());
  else if ((newPos.y() + height()) > (_parentImgStartPos.y() + _parentImgSize.height()))
    newPos.setY(_parentImgStartPos.y() + _parentImgSize.height() - height());

  move(newPos);
  
  event->accept();
}


void SmageImgSnip::mouseReleaseEvent(QMouseEvent *event)
{
  event->accept();
}
