// project header
#include "SmageInfoWidget.h"

// qt header
#include <qlayout.h>
#include <qpixmap.h>
#include <qpushbutton.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageInfoWidget::SmageInfoWidget(QWidget *parent, const char* name, WFlags f)
  : QWidget(parent, name, f)
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageInfoWidget::SmageInfoWidget()\n");
#endif
}


SmageInfoWidget::~SmageInfoWidget()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageInfoWidget::~SmageInfoWidget()\n");
#endif 
}


void SmageInfoWidget::setImg(const QString &imgPath, const bool &withOKButton)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageInfoWidget::setImg()\n");
#endif

  QPixmap img;
  if (img.load(imgPath, "PNG"))
  {
    int imgWidth = img.width();
    int imgHeight = img.height();

    // Setting the widget's position according to its parent widget

    QPoint relativePos(0, 0);
    QWidget *parent = parentWidget();
    if (parent != NULL)
    {
      relativePos.setX(int((parent->size().width() - imgWidth)/2));
      relativePos.setY(int((parent->size().height() - imgHeight)/2));
    }

    setGeometry(relativePos.x(), relativePos.y(), imgWidth, imgHeight);
    //setBackgroundPixmap(img);
    setBackgroundOrigin(QWidget::WidgetOrigin);
    setPaletteBackgroundPixmap(img);

    if (withOKButton)
    {
      int buttonWidth = 35;
      int buttonHeight = 25;
      QPushButton *okButton = new QPushButton(QObject::tr("OK"), this);
      connect(okButton, SIGNAL(clicked()), this, SLOT(hide()));
      okButton->setGeometry(int(imgWidth/2) - int(buttonWidth/2), imgHeight - buttonHeight - 5, 
        buttonWidth, buttonHeight);
    }
  }
  else
  {
#ifdef THE_S_PRINT_DEBUG
    _TheSmageLogFile.lprintf("*** Warning (SmageInfoWidget::setImg): could not load image ***\n");
#endif
  }
}


void SmageInfoWidget::updateGeometry()
{
  QPoint relativePos(0, 0);
  QWidget *parent = parentWidget();
  if (parent != NULL)
  {
    relativePos.setX(int((parent->size().width() - width())/2));
    relativePos.setY(int((parent->size().height() - height())/2));
  }

  setGeometry(relativePos.x(), relativePos.y(), width(), height());
}
