#ifndef _SMAGE_TEXT_EDITOR_H
#define _SMAGE_TEXT_EDITOR_H

#include "../smageSharedFiles/structs/TheS_BubbleData.h"

#include <qmainwindow.h>
#include <qfont.h>

class QComboBox;
class QTextEdit;
class QToolButton;
class QVBox;


typedef void TheS_BubbleTextEditor_PrevCB(void *, const TheS_BubbleData &);
typedef void TheS_BubbleTextEditor_OkCB(void *, const TheS_BubbleData &);
typedef void TheS_BubbleTextEditor_CancelCB(void *);

/** Widget class for specifying text properties.
  *
  * @ingroup SmageApp
  *
  */
class SmageTextEditor : public QMainWindow
{
  Q_OBJECT

 public:

  SmageTextEditor(QWidget *parent, const char* name, const QString &text, const QFont &font);
  ~SmageTextEditor();

  void setPrevCB(void *userData, TheS_BubbleTextEditor_PrevCB *f)
    { _prevCBFunc = f; _prevUserData = userData; }
  void setOkCB(void *userData, TheS_BubbleTextEditor_OkCB *f)
    { _okCBFunc = f; _okUserData = userData; }
  void setCancelCB(void *userData, TheS_BubbleTextEditor_CancelCB *f)
    { _cancelCBFunc = f; _cancelUserData = userData; }

  void setBackgroundColor(const QColor &color);
  void assignNewProps(const TheS_BubbleData &props);

//  static void copyBubbleData(const TheS_BubbleData &src, TheS_BubbleData &dst);

 private slots:
  void textFamily(const QString &font);
  void textSize(const QString &size);
  void textBold();
  void textItalic();
  void fontChanged(const QFont &);
  void alignmentChanged(int);
  void colorChanged(const QColor &);
  void textLeft();
  void textCenter();
  void textRight();
  void textColor();
  void backgroundColor();
  void transparentBackground();
  void outlineColor();
  void outlineWidth( const QString &width);
  void preview();
  void ok();
  void cancel();

 protected:
  void initTextEditor(const QString &text, const QFont &font);
  void initTextToolBar();
  void initBackgroundToolBar();
  void initButtonArea();

  QVBox *_vBoxWidget;
  QTextEdit *_textEditor;

  QComboBox *_comboFont, *_comboSize, *_comboWidth;

  QToolButton *_textBoldButton, *_textItalicButton;
  QToolButton *_textColorButton;
  QToolButton *_alignLeftButton, *_alignCenterButton, *_alignRightButton;
  QToolButton *_backgroundColorButton, *_backgroundTransparentButton, *_outlineColorButton;

  QColor _backgroundCol, _outlineCol;
  uint _lineWidth;
  unsigned char _backgroundAlpha;

  void *_prevUserData;
  TheS_BubbleTextEditor_PrevCB *_prevCBFunc;
  void *_okUserData;
  TheS_BubbleTextEditor_OkCB *_okCBFunc;
  void *_cancelUserData;
  TheS_BubbleTextEditor_CancelCB *_cancelCBFunc;
};

#endif
