// project header
#include "SmageDictEditor.h"
#include "SmageImgInputDlg.h"
#include "SmageTextHolderWidget.h"
#include "../smageSharedFiles/SmageXmlParser.h"

// qt header
#include <qmessagebox.h>
#include <qpixmap.h>

// c/c++ header
#include <assert.h>
#include <iostream>

using namespace std;

SmageTextHolderWidget::SmageTextHolderWidget(QWidget *parent, const char* name, const QString &text,
					     const char* pixXpm[], const bool &isDictEntry)
  : QWidget(parent, name)
{ 
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageTextHolderWidget::SmageTextHolderWidget()\n");
#endif

  _theText = text.copy();
  _isDictEntry = isDictEntry;
  _idNum = -1;
  _idString = QString::null;
  _language = -1;

  setImg(pixXpm);
}


SmageTextHolderWidget::~SmageTextHolderWidget()
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageTextHolderWidget::~SmageTextHolderWidget()\n");
#endif 
}


void SmageTextHolderWidget::setImg(const char* pixXpm[])
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageTextHolderWidget::setImg()\n");
#endif

  QPixmap img(pixXpm);
  if (!img.isNull())
  {
    int imgWidth = img.width();
    int imgHeight = img.height();

    setFixedSize(imgWidth, imgHeight);

    // Setting the widget's position according to its parent widget

    setPaletteBackgroundPixmap(img);
  }
  else
  {
#ifdef THE_S_PRINT_DEBUG
    _TheSmageLogFile.lprintf("*** Warning (SmageTextHolderWidget::setImg): could not load image ***\n");
#endif
  }
}


void SmageTextHolderWidget::mouseReleaseEvent(QMouseEvent *)
{
  if (!_isDictEntry)
    {
      QString textInput;
      if (SmageImgInputDlg::getTextInput(_theText, textInput, this))
	{
	  _theText = textInput.copy();
	}
    }
  else
    getDictEntry();
}


bool SmageTextHolderWidget::getDictEntry()
{
  int lang = _language;
  QString entry = _theText.copy();

  if (SmageDictEntryDlg::getDictEntry(lang, entry))
    {
      if (!entry.isEmpty())
	{
	  QString idString;
	  int idNum = -1;
	  
	  SmageXmlParser xmlParser;
	  if (xmlParser.searchForId("smageDict.xml", idString, idNum, lang, entry) ||
	      xmlParser.searchForId("smageDictPriv.xml", idString, idNum, lang, entry))
	    {
	      _language = lang;
	      _theText = entry.copy();
	      _idNum = idNum;
	      _idString = idString.copy();

	      return TRUE;
	    }
	  else
	    {
	      if (QMessageBox::information(this, QObject::tr("Smage - Message"), tr("The term is not in the dictionary yet.\nYou can now add it to your private dictionary ..."), QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton) == QMessageBox::Ok)
		{
		  QString idString, entry;
		  int language;

		  if (SmageDictEditor::getNewDictEntries("smageDictPriv.xml", idString, entry, language, this))
		    {
		      _language = language;
		      _idString = idString;
		      _theText = entry;
		      _idNum = -1;

		      return TRUE;
		    }
		}
	    }
	}
    }

  return FALSE;
}
