// project header
#include "SmageTextTypeDlg.h"
#include "./pixmaps/textDescrSpeech.xpm"
#include "./pixmaps/textDescrSimple.xpm"
#include "./pixmaps/textDescrThink.xpm"
#include "./pixmaps/textDescrEllipse.xpm"
#include "./pixmaps/textDescrRect.xpm"

// qt header
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qradiobutton.h>

// c/c++ header
#include <iostream>
#include <stdlib.h>

using namespace std;

SmageTextTypeDlg::SmageTextTypeDlg(QWidget *topLevel)
  : QDialog(topLevel, 0)
{
#ifdef THE_S_PRINT_DEBUG
  _TheSmageLogFile.lprintf("SmageTextTypeDlg::SmageTextTypeDlg()\n");
#endif

  setPaletteBackgroundColor(QColor(232,227,227));

  setCaption(tr("Choose A Text Type"));

  QButtonGroup *buttonGroup = new QButtonGroup(this);
  buttonGroup->hide();
  buttonGroup->setRadioButtonExclusive(TRUE);
  connect(buttonGroup, SIGNAL(clicked(int)), SLOT(textType(int)));
  QRadioButton *simpleTextRB = new QRadioButton(this, "Simple_Text");
  simpleTextRB->setPixmap(QPixmap(textDescrSimple_xpm));
  QRadioButton *simpleRectRB = new QRadioButton(this, "Simple_Rect");
  simpleRectRB->setPixmap(QPixmap(textDescrRect_xpm));
  QRadioButton *simpleEllipseRB = new QRadioButton(this, "Simple_Ellipse");
  simpleEllipseRB->setPixmap(QPixmap(textDescrEllipse_xpm));
  QRadioButton *speechBubbleRB = new QRadioButton(this, "Speech_Bubble");
  speechBubbleRB->setPixmap(QPixmap(textDescrSpeech_xpm));
  QRadioButton *thinkBubbleRB = new QRadioButton(this, "Think_Bubble");
  thinkBubbleRB->setPixmap(QPixmap(textDescrThink_xpm));  
  buttonGroup->insert(simpleTextRB, 0);
  buttonGroup->insert(simpleRectRB, 1);
  buttonGroup->insert(simpleEllipseRB, 2);
  buttonGroup->insert(speechBubbleRB, 3);
  buttonGroup->insert(thinkBubbleRB, 4);
  buttonGroup->setButton(0);

  // OK / Cancel button
  
  _okButton = new QPushButton(QObject::tr("OK"), this);
  connect(_okButton, SIGNAL(clicked()), this, SLOT(okPressed()));
  _okButton->setFixedSize(100, 40);
  _okButton->setDefault(TRUE);

  _cancelButton = new QPushButton(QObject::tr("Cancel"), this);
  connect(_cancelButton, SIGNAL(clicked()), this, SLOT(cancelPressed()));
  _cancelButton->setFixedSize(100, 40);

  _gridLayout = new QGridLayout(this,6,4,20);
  _gridLayout->addMultiCellWidget(simpleTextRB, 0, 0, 0, 3, Qt::AlignLeft);
  _gridLayout->addMultiCellWidget(simpleRectRB, 1, 1, 0, 3, Qt::AlignLeft);
  _gridLayout->addMultiCellWidget(simpleEllipseRB, 2, 2, 0, 3, Qt::AlignLeft);
  _gridLayout->addMultiCellWidget(speechBubbleRB, 3, 3, 0, 3, Qt::AlignLeft);
  _gridLayout->addMultiCellWidget(thinkBubbleRB, 4, 4, 0, 3, Qt::AlignLeft);
  _gridLayout->addRowSpacing (5, 40);
  _gridLayout->addWidget(_okButton, 5, 1, Qt::AlignHCenter);
  _gridLayout->addWidget(_cancelButton, 5, 2, Qt::AlignHCenter);

  _textType = SMAGE_PLAIN_TEXT;
}


SmageTextTypeDlg::~SmageTextTypeDlg() 
{   
}


void SmageTextTypeDlg::okPressed()
{
  accept();
}


void SmageTextTypeDlg::cancelPressed()
{
  reject();
}


bool SmageTextTypeDlg::getTextType(TextShapeType &textType, QWidget *parent)
{
  bool returnValue = FALSE;

  SmageTextTypeDlg *dlg = new SmageTextTypeDlg(parent);
  
  if (dlg->exec() == QDialog::Accepted)
  {
    textType = dlg->_textType;
    returnValue = TRUE;
  }
  
  delete dlg;
  return returnValue;
}

void SmageTextTypeDlg::textType(int type)
{
  switch (type)
    {
    case 0: _textType = SMAGE_PLAIN_TEXT; break;
    case 1: _textType = SMAGE_RECT; break;
    case 2: _textType = SMAGE_ELLIPSE; break;
    case 3: _textType = SMAGE_SPEECH_BUBBLE; break;
    case 4: _textType = SMAGE_THINK_BUBBLE; break;
    default: _textType = SMAGE_PLAIN_TEXT; break;
    }
}
