// project header
#include "SmageXmlHandler.h"
#include "../smageSharedFiles/SmageXmlParser.h"

// c/c++ header
#include <iostream>

#define SMAGE_XML_ENCODING "UTF-8"

SmageXmlHandler::SmageXmlHandler(const char* filename)
{
  QString dictFile;
  char *d = getenv("SMAGE_DIR");
  if ( d == NULL ) dictFile = QString( "dictionary/" + QString(filename));
  else dictFile = QString( QString(d) + "dictionary/" + QString(filename));

  _docPtr = xmlReadFile(dictFile, SMAGE_XML_ENCODING, 0);
  if (_docPtr != NULL)
    _rootNodePtr = xmlDocGetRootElement(_docPtr);
}

SmageXmlHandler::~SmageXmlHandler()
{
  xmlFreeDoc(_docPtr);
  xmlCleanupParser();
}

xmlNodePtr SmageXmlHandler::appendEntry(const DictIdEntriesMap &entriesMap, const QString &idName, 
					const int &idNum)
{  
  if (_docPtr != NULL)
    {
      xmlNodePtr dictEntry = 
	xmlNewChild(_rootNodePtr, NULL, BAD_CAST "DictItem", BAD_CAST (const char *)idName.utf8());
      xmlNodeAddContent(dictEntry, BAD_CAST  "\n");
      if (idNum >= 0)
	{
	  QString tmpId;
	  tmpId.setNum(idNum);
	  xmlNewProp(dictEntry, BAD_CAST "id", BAD_CAST (const char *)tmpId.utf8());
	}
      else if (idNum == -1)
	{
	  xmlNodePtr lastNode = xmlGetLastChild(_rootNodePtr)->prev->prev;
	  if (lastNode != NULL)
	    {
	      QString prevId((const char *)xmlGetProp(lastNode, BAD_CAST "id"));
	      QString newId;
	      newId.setNum(prevId.toInt()+1);
	      xmlNewProp(dictEntry, BAD_CAST "id", BAD_CAST (const char *)newId.utf8());
	    }
	  else
	    xmlNewProp(dictEntry, BAD_CAST "id", BAD_CAST "1");
	}

      QString tagName, tagPriName, tagSecName;
      xmlNodePtr langNode;
      std::vector<QString>::const_iterator stringVecIter;
      DictIdEntriesMap::const_iterator mapIter;
      for (mapIter=entriesMap.begin(); mapIter!=entriesMap.end(); mapIter++)
	{
	  if (_parser.getLangTagName(mapIter->first, tagName))
	    {
	      tagPriName = tagName + "_P";
	      tagSecName = tagName + "_S";

	      langNode = 
		xmlNewChild(dictEntry, NULL, BAD_CAST (const char *)tagName.utf8(), NULL);
	      
	      xmlNewChild(langNode, NULL, BAD_CAST (const char *)tagPriName.utf8(), 
			  BAD_CAST (const char *)mapIter->second.primeWord.utf8());

	      for (stringVecIter=mapIter->second.secWords.begin(); 
		   stringVecIter!=mapIter->second.secWords.end(); stringVecIter++)
		  xmlNewChild(langNode, NULL, BAD_CAST (const char *)tagSecName.utf8(), 
			      BAD_CAST (const char *)stringVecIter->utf8());
	      
	      xmlNodeAddContent(dictEntry, BAD_CAST  "\n");
	    }
	}

      xmlNodeAddContent(_rootNodePtr, BAD_CAST  "\n\n");
      
      return dictEntry;
    }

  return NULL;
}

int SmageXmlHandler::saveCurDoc(const char *fileName)
{
  if (_docPtr == NULL) return -1;
  QString dictFile;
  char *d = getenv("SMAGE_DIR");
  if ( d == NULL ) dictFile = QString( "dictionary/" + QString(fileName));
  else dictFile = QString( QString(d) + "dictionary/" + QString(fileName));
  return xmlSaveFileEnc(dictFile, _docPtr, SMAGE_XML_ENCODING);

}
